// ===========================================================================
//	3DPluginDrawUtil			(c) 2007 Mark F. Coniglio. All rights reserved
// ===========================================================================

#ifndef _H_3DPluginDrawUtil
#define	_H_3DPluginDrawUtil

#include "OpenGLIncludes.h"
#include "IsadoraTypes.h"
#include "ImageBufferCommon.h"
#include "GraphicsHelpers.h"
#include "IsadoraCallbacks.h"

#include "CoreImageCommon.h"
#if COMPILE_CORE_VIDEO
#include "CoreImageBase.h"
#endif

// ---------------------------------------------------------------------------

#if DEBUG && FALSE
	#define	PDU_CheckGLError(ip)		Plugin3DDrawBase::PDU_CheckOpenGLError_DebugOnly(ip)
#else
	#define	PDU_CheckGLError(ip)		{ }
#endif

class Plugin3DDrawBase
{
public:
				Plugin3DDrawBase() : ip(NULL) { }
	
#if DEBUG
	static void	PDU_CheckOpenGLError_DebugOnly(
					IsadoraParameters*	inIsadoraParams);
#endif

protected:
	IsadoraParameters*		ip;
};

// ---------------------------------------------------------------------------
// CPlugin3DRender
// 
// This allows a 3D Plugin manage it's relatonship to a render target


class CPlugin3DRender : public Plugin3DDrawBase
{
public:
	
					CPlugin3DRender();
	
	void			InitRender3DManager(
						IsadoraParameters*					inIzzyParams,
						ActorInfo*							inActorInfo);

	void			SetRenderInfo(
						Render3DCallback					inCallback,
						ThingPtr							inThing,
						void*								inRefCon1,
						void*								inRefCon2);

	void			SetRenderInfo(
						Render3DCallbackWithContextFlags	inCallback,
						ThingPtr							inThing,
						void*								inRefCon1,
						void*								inRefCon2);

	void			Attach();
				
	void			Detach();
	
	void			Update();
	
	void			MarkNeedsDraw();
	
	// CONTEXT TYPE
	void			SetPendingRenderTarget(
						RenderTarget		inPendingContext)
							{ mPendingRenderTarget = inPendingContext; }
	
	RenderTarget	GetPendingRenderTarget() const
						{ return mPendingRenderTarget; }
					
	RenderTarget	GetCurrentRenderTarget() const
						{ return mCurrentRenderTarget; }
					
	// CHANNEL
	void			SetPendingChannel(
						RenderChannel		inPendingChannelIndex1)
							{ mPendingChannelIndex1 = inPendingChannelIndex1; }
	
	RenderChannel	GetPendingChannel() const
						{ return mPendingChannelIndex1; }
	
	RenderChannel	GetCurrentChannel() const
						{ return mCurrentChannellIndex1; }
	
	bool			CurrentChannelIsValid() const
						{ return (mCurrentChannellIndex1 >= 1); }
				
	// LAYER
	void			SetPendingLayer(
						SInt32			inPendingLayer)
							{ mPendingLayer = inPendingLayer; }
				
	SInt32			GetPendingLayer() const
						{ return mPendingLayer; }
					
	SInt32			GetCurrentLayer() const
						{ return mCurrentLayer; }
	
	// VISIBLE
	void			SetPendingVisible(
						bool			inPendingVisible)
							{ mPendingVisible = inPendingVisible; }
					
	bool			GetPendingVisible() const
						{ return mPendingVisible; }
					
	bool			GetCurrentVisible() const
						{ return mCurrentVisible; }
					
private:
	ActorInfo*				mActorInfo;
	
	bool					mUseWithContextRender;
	
	union {
		Render3DInfo			mRenderInfo;
		Render3DWithContextInfo	mRenderInfoWithContext;
	};
	
	RenderTarget			mCurrentRenderTarget;
	RenderTarget			mPendingRenderTarget;
	
	RenderChannel			mCurrentChannellIndex1;
	RenderChannel			mPendingChannelIndex1;
	
	bool					mCurrentVisible;
	bool					mPendingVisible;

	SInt32					mCurrentLayer;
	SInt32					mPendingLayer;
};

// ---------------------------------------------------------------------------
// CPluginTextureMap
// 
// This allows a 3D Plugin manage a texture map.

class CPluginTextureMap : public Plugin3DDrawBase
{
public:

								CPluginTextureMap();
	
	void						InitPluginTextureMap(
									IsadoraParameters*		inIzzyParams,
									ActorInfo*				inActorInfo,
									PropertyIndex			inRendererPropertyIndex1,
									PropertyIndex			inStagePropertyIndex1,
									bool					inWantStageDisposeCalllback = false);

	void						InitTextureMapInfo();
	
	void						Dispose(
									bool					inDisableIfInvalid = false);
	
	void						InvalidatePluginTextureMap();
	
	inline bool					HasValidRenderTargetAndChannel() const
									{ return (mActualRenderTarget != kRenderTarget_Invalid && mActualChannelIndex1 != kRenderChannel_Invalid); }
	
// RENDER CONTEXT TYPE
	
	inline RenderTarget			GetRenderTarget() const
									{ return mRequestedRenderTarget; }
					
	RenderTarget				GetActualRenderTarget(
									RenderTarget			inRequestedRenderTarget);
	
	void						SetRenderTarget(
									RenderTarget			inRequestedRenderTarget,
									CPlugin3DRender&		inRenderer)
								{
									SetRenderTarget(inRequestedRenderTarget);
									inRenderer.SetPendingRenderTarget(mActualRenderTarget);
								}
	
	void						SetRenderTarget(
									RenderTarget			inRequestedRenderTarget);

// RENDER CHANNEL
	inline RenderChannel		GetRenderChannel() const
									{ return mRequestedChannelndex1; }
									
	RenderChannel				GetActualRenderChannelIndex1(
									RenderChannel			inRequestedRenderChannelIndex1);

	void						SetRenderChannel(
									RenderChannel			inRequestedRenderChannelIndex1,
									CPlugin3DRender&		inRenderer)
								{
									SetRenderChannel(inRequestedRenderChannelIndex1);
									inRenderer.SetPendingChannel(mActualChannelIndex1);
								}
	
	void						SetRenderChannel(
									RenderChannel			inRequestedRenderChannelIndex1);

// CONTEXT/CHANNEL OVERRIDE
	bool						ForceRenderTargetAndChannelIfNeeded(
									CPlugin3DRender&	inRenderer) const;
	

// TEXXTURE MAP CREATION/DESTRUCTION	
	void						CreateTextureMap(
									bool					inStoreFirstTextureMapImage = false,
									bool					inRepeat = false,
									bool					inClampToBorder = false,
									bool					inDisableAppleDMA = false);

	void						DisposeTextureMap(
									bool					inDisableIfInvalid = false);
	
	void						SetInputValue(
									const Value*			inValue);
	
	void						HandleTextureMapInput(
									const Value*			inValue,
									bool					inDisableIfInvalid,
									CPlugin3DRender&		inRenderer)
								{
									HandleTextureMapInput(inValue, inDisableIfInvalid);
									ForceRenderTargetAndChannelIfNeeded(inRenderer);
								}
								
	void						HandleTextureMapInput(
									const Value*			inValue,
									bool					inDisableIfInvalid = false);
	
	void						HandleTextureMapInput(
									ImageBufferPtr			inImageBufferPtr);
								
	void						HandleTextureMapInput(
									TextureBufferPtr		inImageBufferPtr);
	
	#if COMPILE_CORE_VIDEO
	void						HandleTextureMapInput(
									CoreImageBufferPtr		inImageBufferPtr);
	#endif

	void						UpdateTextureMap(
									bool					inRepeat = false,
									bool					inClampToBofder = false,
									bool					inDisableAppleDMA = false);

	bool						PrepareForRender(
									PluginContextInfo*		inContextInfo,
									float&					outHScale,
									float&					outVScale,
									bool					inInvertTextureCoordsIfBufferTypeIsTexture = false);

	void						UnprepareAfterRender();
	
	inline bool					TextureIsPremultiplied() const
									{ return (mTextureMapInfo.mGLTexture != NULL && mTextureMapInfo.mGLTexture->GetTextureIsPremultiplied()); }
								
	bool						SetBlendFunction_DEPRECATED(
									RenderBlendMode			inBlendMode) const
										{ return SetBlendFunction_DEPRECATED(inBlendMode,  TextureIsPremultiplied()); }
	
	static bool					SetBlendFunction_DEPRECATED(
									RenderBlendMode			inBlendMode,
									bool					inIsPremultiplied);
	
	void						SetBlendColor(
									RenderBlendMode			inBlendMode,
									const float*			inCurColor,
									const float				inBrightness) const;
	
	inline void					MakeInputValueDirty()							{ mCurrentVideoInputValueDirty = true; }
	
	inline bool					IsAttached() const								{ return mTextureMapInfo.mGLTextureAttached != FALSE; }
	
	inline bool					HasValidInputBuffer() const						{ return mHasValidInputBuffer; }

	inline bool					HasValidBitmapInput() const
								{
									return mTextureMapInfo.mMIB.mVideoBufferType == kVideoBuffer_Bitmap
										&& mTextureMapInfo.mMIB.mImageBuffer != NULL
										&& mTextureMapInfo.mMIB.mImageBuffer->mMagic == kImageBufferMagic
										&& mTextureMapInfo.mMIB.mImageBuffer->mBaseAddress != NULL
										&& mTextureMapInfo.mMIB.mImageBuffer->mWidth > 0
										&& mTextureMapInfo.mMIB.mImageBuffer->mHeight > 0;
								}
									
	inline bool					HasValidTextureInput() const
								{
									return mTextureMapInfo.mMIB.mVideoBufferType == kVideoBuffer_Texture
										&& mTextureMapInfo.mMIB.mTextureBuffer != NULL
										&& mTextureMapInfo.mMIB.mTextureBuffer->mMagic == kTextureBufferMagic
										&& mTextureMapInfo.mMIB.mTextureBuffer->mTexture != NULL
										&& mTextureMapInfo.mMIB.mTextureBuffer->mTargetStageIndex0 != kInvalidStageIndex; 
								}
									
	inline bool					HasValidCoreImageInput() const
								{
									#if COMPILE_CORE_VIDEO
									return mTextureMapInfo.mMIB.mVideoBufferType == kVideoBuffer_CoreImage
										&& mTextureMapInfo.mMIB.mCoreImageBuffer != NULL
										&& mTextureMapInfo.mMIB.mCoreImageBuffer->mMagic == kCoreImageBufferMagic
										&& mTextureMapInfo.mMIB.mCoreImageBuffer->mImage != NULL
										&& mTextureMapInfo.mMIB.mCoreImageBuffer->mTargetStageIndex0 != kInvalidStageIndex;
									#else
										return false;
									#endif
								}
	
	inline VideoBufferType		GetTextureMapVideoBufferType() const			{ return mTextureMapInfo.mMIB.mVideoBufferType; }

	inline ImageBufferPtr		GetTextureMapImageBufferInput() const			{ return mTextureMapInfo.mMIB.mImageBuffer; }
	inline TextureBufferPtr		GetTextureMapTextureBufferInput() const			{ return mTextureMapInfo.mMIB.mTextureBuffer; }
	#if COMPILE_CORE_VIDEO
	inline CoreImageBufferPtr	GetTextureMapCoreImageBufferInput() const		{ return mTextureMapInfo.mMIB.mCoreImageBuffer; }
	#endif
	
	inline bool					CanSetRenderTargetAndChannel() const
								{
									return ip->mSharedGLContexts
										|| !HasValidInputBuffer()
										|| mTextureMapInfo.mMIB.mVideoBufferType == kVideoBuffer_Invalid
										|| mTextureMapInfo.mMIB.mVideoBufferType == kVideoBuffer_Bitmap;
								}

	bool						RenderChannelMatches(
									StageIndexT		inStageIndex0);
	
	inline bool					HasValidTexture() const
									{ return (mTextureMapInfo.mGLTexture != NULL); }
								
	inline IsadoraTexture*		GetTexture() const
									{ return mTextureMapInfo.mGLTexture; }
								
	inline GLuint				GetTextureMapName() const
									{ return (mTextureMapInfo.mGLTexture != NULL) ? mTextureMapInfo.mGLTexture->GetTextureName() : 0; }
	
	inline GLenum				GetTextureTarget() const
									{ return (mTextureMapInfo.mGLTexture != NULL) ? mTextureMapInfo.mGLTexture->GetTextureTarget() : 0; }

	inline bool					RequiresHAPQShader() const
									{ return mTextureMapInfo.mRequiresHAPQShader ? true : false; }
								
	inline bool					GetSize(
									UInt32&		outWidth,
									UInt32&		outHeight) const
								{
									if (mTextureMapInfo.mGLTexture != NULL) {
										outWidth = mTextureMapInfo.mGLTexture->GetTextureVisibleWidth();
										outHeight = mTextureMapInfo.mGLTexture->GetTextureVisibleHeight();
										return true;
									} else {
										outWidth = outHeight = 0;
										return false;
									}
								}
							
	inline bool					GetSize(
									float&		outWidth,
									float&		outHeight) const
								{
									if (mTextureMapInfo.mGLTexture != NULL) {
										outWidth = (float) mTextureMapInfo.mGLTexture->GetTextureVisibleWidth();
										outHeight = (float) mTextureMapInfo.mGLTexture->GetTextureVisibleHeight();
										return true;
									} else {
										outWidth = outHeight = 0.0f;
										return false;
									}
								}

	inline float				GetAspectRatio() const
								{
									if (mTextureMapInfo.mGLTexture != NULL) {
										return (float(mTextureMapInfo.mGLTexture->GetTextureVisibleWidth()) / float(mTextureMapInfo.mGLTexture->GetTextureVisibleHeight()));
									}
									return 0;
								}

	inline bool					GetTexCoordSize(
									float&		outTexWidth,
									float&		outTexHeight) const
								{
									if (mTextureMapInfo.mGLTexture != NULL) {
										mTextureMapInfo.mGLTexture->GetTextureCoordinateExtent(outTexWidth, outTexHeight);
										return true;
									} else {
										outTexWidth = outTexHeight = 0.0f;
										return false;
									}
								}
private:
	ActorInfo*				mActorInfo;
	
	RenderTarget			mRequestedRenderTarget;
	RenderTarget			mActualRenderTarget;

	RenderChannel			mRequestedChannelndex1;
	RenderChannel			mActualChannelIndex1;
	
	Value					mCurrentVideoInputValue;
	bool					mCurrentVideoInputValueDirty;
	
	bool					mHasValidInputBuffer;
	TextureMapInfo			mTextureMapInfo;
	IBFormatInfo			mLastFormat;
	UInt32					mLastDataChangeCount;
	UInt32					mLastTextureUniqueID;
	IzzyTexture				mLastTexture;
	bool					mTextureMatrixNeedsUnprepare;
	
	bool					mStageRendererInputsAreApplicable;
	PropertyIndex			mRendererPropertyIndex1;
	PropertyIndex			mStagePropertyIndex1;

	MessageReceiverRef		mMessageReceiver;
};

void
HardwareIDAlert(
	const char*	inErrorString,
	const char*	inInfoString);
	
#endif
