// ===========================================================================
//	CFLoatXY					 (C)2009 Mark F. Coniglio. All rights reserved.
// ===========================================================================

#ifndef _H_CFLOATXY
#define _H_CFLOATXY

#include "CPlusPlusMacros.h"
#include "izzy_math.h"

// ---------------------------------------------------------------------------
//		CFloatXY
// ---------------------------------------------------------------------------
CLASSHDR CFloatXY
{
PUBLIC

#ifdef __cplusplus

	inline				CFloatXY()
							{ x = y = 0.0f; }
							
	inline				CFloatXY(
							const CFloatXY&		inXY)
								{ x = inXY.x; y = inXY.y; }
	
	inline				CFloatXY(
							const float*			inXYZArray)
						{
							x = inXYZArray[0];
							y = inXYZArray[1];
						}
				
	inline				CFloatXY(
							float	inX,
							float	inY)
								{ x = inX; y = inY; }
	
	void				set(
							float	inX,
							float	inY)
						{
							x = inX;
							y = inY;
						}
	
	inline CFloatXY&	limitAndScale(
							float	inLimitMinX,
							float	inLimitMaxX,
							float	inLimitMinY,
							float	inLimitMaxY,
							float	inScaleMinX,
							float	inScaleMaxX,
							float	inScaleMinY,
							float	inScaleMaxY)
						{
							float ratio;
							
							if (x < inLimitMinX) x = inLimitMinX;
							else if (x > inLimitMaxX) x = inLimitMaxX;
							
							ratio = (x - inLimitMinX) / (inLimitMaxX - inLimitMinX);
							x = inScaleMinX + (inScaleMaxX - inScaleMinX) * ratio;
							
							if (y < inLimitMinY) y = inLimitMinY;
							else if (y > inLimitMaxY) y = inLimitMaxY;

							ratio = (y - inLimitMinY) / (inLimitMaxY - inLimitMinY);
							y = inScaleMinY + (inScaleMaxY - inScaleMinY) * ratio;
							
							return *this;
						}
						
	inline CFloatXY&	limit(
							float	inMinX,
							float	inMaxX,
							float	inMinY,
							float	inMaxY)
						{
							if (x < inMinX) x = inMinX;
							else if (x > inMaxX) x = inMaxX;
							
							if (y < inMinY) y = inMinY;
							else if (y > inMaxY) y = inMaxY;

							return *this;
						}
						
	inline CFloatXY&	limit(
							float	inMin,
							float	inMax)
						{
							if (x < inMin) x = inMin;
							else if (x > inMax) x = inMax;
							
							if (y < inMin) y = inMin;
							else if (y > inMax) y = inMax;

							return *this;
						}
	
	inline CFloatXY&	operator =(
							const CFloatXY&	inValue)
						{
							x = inValue.x;
							y = inValue.y;
							return *this;
						}
						
	inline CFloatXY&	operator =(
							const float			inValue)
						{
							x = y = inValue;
							return *this;
						}
						
	// --------------------
	// float*
	// --------------------
						operator const float*() const
								{ return &x; }
	// --------------------
	// ADDITION
	// --------------------
	inline CFloatXY		operator +(
							float inValue) const
								{ return CFloatXY(x + inValue, y + inValue); }
				
	
	inline CFloatXY		operator +(
							const CFloatXY&	inValue) const
								{ return CFloatXY(x + inValue.x, y + inValue.y); }
				
	
	inline CFloatXY&	operator +=(
							float inValue)
						{
							x += inValue;
							y += inValue;
							return (*this);
						}
				
	inline CFloatXY&	operator +=(
							const CFloatXY&	inValue)
						{
							x += inValue.x;
							y += inValue.y;
							return (*this);
						}
				
	
	// --------------------
	// SUBTRACTION
	// --------------------
	inline CFloatXY		operator -() const
							{ return CFloatXY(-x, -y); }
							
	inline CFloatXY		operator -(
							float inValue) const
								{ return CFloatXY(x - inValue, y - inValue); }
				
	
	inline CFloatXY		operator -(
							const CFloatXY&	inValue) const
								{ return CFloatXY(x - inValue.x, y - inValue.y); }
				
	
	inline CFloatXY&	operator -=(
							float inValue)
						{
							x -= inValue;
							y -= inValue;
							return (*this);
						}
						
	inline CFloatXY&	operator -=(
							const CFloatXY&	inValue)
						{
							x -= inValue.x;
							y -= inValue.y;
							return (*this);
						}
	
	// --------------------
	// MULTIPLICATION
	// --------------------
	inline CFloatXY		operator *(
							float inValue) const
								{ return CFloatXY(x * inValue, y * inValue); }
				
	inline CFloatXY		operator *(
							const CFloatXY&	inValue) const
								{ return CFloatXY(x * inValue.x, y * inValue.y); }
				
	inline CFloatXY&	operator *=(
							float inValue)
						{
							x *= inValue;
							y *= inValue;
							return (*this);
						}
				
	inline CFloatXY&	operator *=(
							const CFloatXY&	inValue)
						{
							x *= inValue.x;
							y *= inValue.y;
							return (*this);
						}
				
	// --------------------
	// DIVISION
	// --------------------
	inline CFloatXY		operator /(
							float inValue) const
								{ return CFloatXY(x / inValue, y / inValue); }
				
	inline CFloatXY		operator /(
							const CFloatXY&	inValue) const
							{ return CFloatXY(x / inValue.x, y / inValue.y); }
						
	inline CFloatXY&	operator /=(
							float inValue)
						{
							x /= inValue;
							y /= inValue;
							return (*this);
						}
				
	inline CFloatXY&	operator /=(
							const CFloatXY&	inValue)
						{
							x /= inValue.x;
							y /= inValue.y;
							return (*this);
						}
	
	// --------------------
	// COMPARISONS
	// --------------------
	
	inline bool	operator ==(
					const CFloatXY&	inValue) const
						{ return x == inValue.x && y == inValue.y; }
	
	inline bool	operator !=(
					const CFloatXY&	inValue) const
						{ return x != inValue.x || y != inValue.y; }
	
	inline void	constrain(
						float	inMinX,
						float	inMaxX,
						float	inMinY,
						float	inMaxY)
					{
						if (x < inMinX) {
							x = inMinX;
						} else if (x > inMaxX) {
							x = inMaxX;
						}
						if (y < inMinY) {
							y = inMinY;
						} else if (y > inMaxY) {
							y = inMaxY;
						}
					}
					
	inline void	constrain(
						const CFloatXY&	inMin,
						const CFloatXY&	inMax)
					{
						if (x < inMin.x) {
							x = inMin.x;
						} else if (x > inMax.x) {
							x = inMax.x;
						}
						if (y < inMin.y) {
							y = inMin.y;
						} else if (y > inMax.y) {
							y = inMax.y;
						}
					}
					
	// --------------------
	// GEOMETRIC OPERATIONS
	// --------------------
	
	inline float		distance(const CFloatXY& inRhs) const
						{
							CFloatXY p = (*this) - inRhs;
							return std::sqrt((p.x * p.x) + (p.y * p.y));
						}
	
	inline float		magnitude() const
							{ return std::sqrt((x * x) + (y * y)); }
				
	inline CFloatXY&	normalize()
						{
							float d = std::sqrt((x * x) + (y * y));
							if (d != 0.0f) {
								x /= d;
								y /= d;
							} else {
								x = y = 0.0f;
							}
							return *this;
						}
						
	inline CFloatXY		interpolate(
							const CFloatXY&		inRhs,
							const float			t) const
						{
							return CFloatXY(x + (inRhs.x - x) * t, y + (inRhs.y - y) * t);
						}
	
	// see http://stackoverflow.com/questions/243945/calculating-a-2d-vectors-cross-product for explination
	
	inline float		crossProduct(
							const CFloatXY&		b) const
						{
							return (x * b.y) - (y * b.x);
						}
	
	inline double		dotProduct(
							const CFloatXY&	b) const
						{
							return ( x * b.x ) + ( y * b.y );
						}
						
	inline void			rotate(
							float			inAngle,
							const CFloatXY&	inAroundPoint)
						{
							const float s = sin(inAngle);
							const float c = cos(inAngle);

							// translate point back to origin:
							x -= inAroundPoint.x;
							y -= inAroundPoint.y;

							// rotate point
							const float xnew = x * c - y * s;
							const float ynew = x * s + y * c;

							// translate point back:
							x = xnew + inAroundPoint.x;
							y = ynew + inAroundPoint.y;
							
						}
	
	inline void			rotate_scaled(
							float			inAngle,
							const CFloatXY&	inAroundPoint,
							const CFloatXY&	inScaling)
						{
							if (inScaling.x != 0.0 && inScaling.y != 0.0) {
							
								*this *= inScaling;
								
								const float s = sin(inAngle);
								const float c = cos(inAngle);

								// translate point back to origin:
								x -= inAroundPoint.x;
								y -= inAroundPoint.y;

								// rotate point
								const float xnew = x * c - y * s;
								const float ynew = x * s + y * c;

								// translate point back:
								x = xnew + inAroundPoint.x;
								y = ynew + inAroundPoint.y;
								
								*this /= inScaling;
							
							}
							
						}
	
	static inline CFloatXY	find_line_segment_intersection(
								const CFloatXY&	s1,				// line segment 1: start
								const CFloatXY&	e1,				// line segment 1: end
								const CFloatXY&	s2,				// line segment 2: start
								const CFloatXY&	e2,				// line segment 2: end
								float*			p1,				// intersection if found
								float*			p2)				// intersection if found
							{
								float det1, det2;

								float det = (e1-s1).crossProduct(s2-e2);
								// if no intersection found, set p1 and p2 to
								// INFINITY to indicate there no intersection
								// was found
								if (det == 0.0) {
									*p1 = *p2 = INFINITY;
									return CFloatXY(INFINITY, INFINITY);
								}
								
								det1 = (s2-s1).crossProduct(s2-e2);
								det2 = (e1-s1).crossProduct(s2-s1);
								
								*p1 = det1 / det;
								*p2 = det2 / det;
								
								return CFloatXY(s1.x + *p1 * (e1.x-s1.x), s2.y + *p2 * (e2.y-s2.y));
							}

	// --------------------
	// ENDIAN SWAPPING
	// --------------------
	
	CFloatXY			get_native_from_little() const
						{
							#if TARGET_RT_BIG_ENDIAN
								return CFloatXY(endian_swap_float(x), endian_swap_float(y));
							#else
								return *this;
							#endif
						}
						
	CFloatXY			get_little_from_native() const
						{
							#if TARGET_RT_BIG_ENDIAN
								return CFloatXY(endian_swap_float(x), endian_swap_float(y));
							#else
								return *this;
							#endif
						}
						
	CFloatXY			get_big_from_native() const
						{
							#if TARGET_RT_LITTLE_ENDIAN
								return CFloatXY(endian_swap_float(x), endian_swap_float(y));
							#else
								return *this;
							#endif
						}
						
	CFloatXY			get_native_from_big() const
						{
							#if TARGET_RT_LITTLE_ENDIAN
								return CFloatXY(endian_swap_float(x), endian_swap_float(y));
							#else
								return *this;
							#endif
						}
						
	// --------------------
	// DEBUGGING
	// --------------------
	inline bool			has_nan_value() const
						{
							return is_nan(x) || is_nan(y);
						}
						
#endif	// #ifdef __cplusplus

	float	x;
	float	y;
};



#endif
