// ===========================================================================
//	CGLColor					(C)2011 Mark F. Coniglio. All rights reserved.
// ===========================================================================

#ifndef _H_CGLColor
#define _H_CGLColor

#include "CPlusPlusMacros.h"

// ---------------------------------------------------------------------------
//		CGLColor
// ---------------------------------------------------------------------------
CLASSHDR CGLColor
{
PUBLIC

#ifdef __cplusplus
	
	static inline CGLColor	Black_ZeroAlpha()		{ return CGLColor(0.0f, 0.0f, 0.0f, 0.0f); }
	static inline CGLColor	Black()					{ return CGLColor(0.0f, 0.0f, 0.0f, 1.0f); }
	static inline CGLColor	White()					{ return CGLColor(1.0f, 1.0f, 1.0f, 0.0f); }
							
	inline				CGLColor()
						{
							r = g = b = a = 1.0f;
						}
	
	inline				CGLColor(
							UInt32				inColorRef)
						{
							a = ((inColorRef >> 24) & 0xFF) / 255.0f;		// alpha
							r = ((inColorRef >> 16) & 0xFF) / 255.0f;		// red
							g = ((inColorRef >> 8) & 0xFF) / 255.0f;		// green
							b = ((inColorRef >> 0) & 0xFF) / 255.0f;		// blue
						}
	
	inline				CGLColor(
							float			inRed,
							float			inGrn,
							float			inBlu,
							float			inAlp = 1.0f)
						{
							r = inRed;
							g = inGrn;
							b = inBlu;
							a = inAlp;
						}
	
	inline				CGLColor(
							const CGLColor&	inColor)
						{
							r = inColor.r;
							g = inColor.g;
							b = inColor.b;
							a = inColor.a;
						}
				
	inline				CGLColor(
							const CGLColor&	inColor,
							float			inAlphaOverride)
						{
							r = inColor.r;
							g = inColor.g;
							b = inColor.b;
							a = inAlphaOverride;
						}
				
	inline				CGLColor(
							const RGBColor&		inColor)
						{
							r = (float)(inColor.red / 65535.0);
							g = (float)(inColor.green / 65535.0);
							b = (float)(inColor.blue / 65535.0);
							a = 1.0f;
						}
				
	// --------------------
	// ADDITION
	// --------------------
	
	inline CGLColor		operator +(
							float inValue) const
								{ return CGLColor(r + inValue, g + inValue, b + inValue, a + inValue); }
				
	
	inline CGLColor		operator +(
							const CGLColor&	inValue) const
								{ return CGLColor(r + inValue.r, g + inValue.g, b + inValue.b, a + inValue.a); }
				
	
	inline CGLColor&	operator +=(
							float inValue)
						{
							r += inValue;
							g += inValue;
							b += inValue;
							a += inValue;
							return (*this);
						}
						
	
	inline CGLColor&	operator +=(
							const CGLColor&	inValue)
						{
							r += inValue.r;
							g += inValue.g;
							b += inValue.b;
							a += inValue.a;
							return (*this);
						}
				
	// --------------------
	// SUBTRACTION
	// --------------------
	
	inline CGLColor		operator -(
							float inValue) const
								{ return CGLColor(r - inValue, g - inValue, b - inValue, a - inValue); }
						
	
	inline CGLColor		operator -(
							const CGLColor&	inValue) const
								{ return CGLColor(r - inValue.r, g - inValue.g, b - inValue.b, a - inValue.a); }
						
	
	inline CGLColor&	operator -=(
							float inValue)
						{
							r -= inValue;
							g -= inValue;
							b -= inValue;
							a -= inValue;
							return (*this);
						}
				
	
	inline CGLColor&	operator -=(
							const CGLColor&	inValue)
						{
							r -= inValue.r;
							g -= inValue.g;
							b -= inValue.b;
							a -= inValue.a;
							return (*this);
						}
				
	
	// --------------------
	// MULTIPLICATION
	// --------------------
	
	inline CGLColor		operator *(
							float inValue) const
								{ return CGLColor(r * inValue, g * inValue, b * inValue, a * inValue); }
				
	inline CGLColor		operator *(
							const CGLColor&	inValue) const
								{ return CGLColor(r * inValue.r, g * inValue.g, b * inValue.b, a * inValue.a); }
				
	inline CGLColor&	operator *=(
							float inValue)
						{
							r *= inValue;
							g *= inValue;
							b *= inValue;
							a *= inValue;
							return (*this);
						}
				
	inline CGLColor&	operator *=(
							const CGLColor&	inValue)
						{
							r *= inValue.r;
							g *= inValue.g;
							b *= inValue.b;
							a *= inValue.a;
							return (*this);
						}
				
	// --------------------
	// DIVISION
	// --------------------
	
	inline CGLColor		operator /(
							float inValue) const
								{ return CGLColor(r / inValue, g / inValue, b / inValue, a / inValue); }
				
	inline CGLColor		operator /(
							const CGLColor&	inValue) const
								{ return CGLColor(r / inValue.r, g / inValue.g, b / inValue.b, a / inValue.a); }
						
	inline CGLColor&	operator /=(
							float inValue)
						{
							r /= inValue;
							g /= inValue;
							b /= inValue;
							a /= inValue;
							return (*this);
						}
				
	inline CGLColor&	operator /=(
							const CGLColor&	inValue)
						{
							r /= inValue.r;
							g /= inValue.g;
							b /= inValue.b;
							a /= inValue.a;
							return (*this);
						}
	
	// --------------------
	// COMPARISONS
	// --------------------
	
	inline bool			operator ==(
							const CGLColor&	inValue) const
								{ return r == inValue.r && g == inValue.g && b == inValue.b && a == inValue.a; }
	
	inline bool			operator !=(
							const CGLColor&	inValue) const
								{ return r != inValue.r || g != inValue.g || b != inValue.b || a != inValue.a; }
								
	
	// --------------------
	// INDEXED ACCESS
	// --------------------
	
	inline				operator float*()
							{ return &r; }
							
	inline				operator const float*() const
							{ return &r; }
							
	inline				operator const CGLColor*() const
							{ return this; }
							
	inline float&		operator[](
							int					inIndex0)
								{ return ((float*) &r)[inIndex0]; }
						
	// --------------------
	// OTHER OPERATIONS
	// --------------------
	
	inline CGLColor&	CopyRGB_OverrideAlpha(
							const CGLColor&	c2,
							const float		inAlpha)
						{
							r = c2.r;
							g = c2.g;
							b = c2.b;
							a = inAlpha;
							return *this;
						}
	
	// ensure the color values are all within range
	inline CGLColor&	Limit()
						{
							if (r < 0.0f) r = 0.0f;
							else if (r > 1.0f) r = 1.0f;
							
							if (g < 0.0f) g = 0.0f;
							else if (g > 1.0f) g = 1.0f;
							
							if (b < 0.0f) b = 0.0f;
							else if (b > 1.0f) b = 1.0f;
							
							if (a < 0.0f) a = 0.0f;
							else if (a > 1.0f) a = 1.0f;
							
							return (*this);
						}
	
	// --------------------
	// MIXING FUNCTIONS
	// --------------------
	
	// mix two colors: as inRatio approaches 1.0
	// the result will approach the color of c2
	// 
	// inRatio = 0.00 = 100% this, 0% inRhs
	// inRatio = 0.25 = 75% this, 25% inRhs
	// inRatio = 0.75 = 25% this, 75% inRhs
	// inRatio = 1.00 = 0% this, 100% inRhs
	
	inline CGLColor		MixARGB(
							const CGLColor&	c2,
							const float		in_mix) const
						{
							return CGLColor(
								r + (c2.r - r) * in_mix,
								g + (c2.g - g) * in_mix,
								b + (c2.b - b) * in_mix,
								a + (c2.a - a) * in_mix);
						}
	
	// same as mix,_argb except that the output alpha 
	// component is set to the supplied alpha value,
	// which defaults to 1.0
	inline CGLColor		MixRGB(
							const CGLColor&	c2,
							const float		in_mix,
							const float		inAlpha = 1.0f) const
						{
							return CGLColor(
								r + (c2.r - r) * in_mix,
								g + (c2.g - g) * in_mix,
								b + (c2.b - b) * in_mix,
								inAlpha);
						}
	
	// same as mix_argb except that the output alpha 
	// component is supplied in by the caller
	inline CGLColor&	SetWithMixedRGB_OverrideAlpha(
							const CGLColor&	inStart,
							const CGLColor&	inEnd,
							const float		inRatio,
							const float		inAlpha)
						{
							*this = inStart.MixRGB(inEnd, inRatio, inAlpha);
							return (*this);
						}
	
	inline CGLColor		GetModulatedRGB(
							const float in_mod) const
						{
							return CGLColor(r * in_mod, g * in_mod, b * in_mod, a);
						}
					
	CGLColor			GetRGB_OverrideAlpha(
							const float	in_new_alpha) const
						{
							return CGLColor(r, g, b, in_new_alpha);
						}
						
	// --------------------
	// COLOR REF FUNCTIONS
	// --------------------
	
	inline void			SetColorRef_ForceFullAlpha(
							const UInt32		inColorRef)
						{
							a = 1.0f;										// alpha
							r = ((inColorRef >> 16) & 0xFF) / 255.0f;		// red
							g = ((inColorRef >> 8) & 0xFF) / 255.0f;		// green
							b = ((inColorRef >> 0) & 0xFF) / 255.0f;		// blue
						}
					
	inline void			SetColorRef(
							const UInt32		inColorRef)
						{
							a = ((inColorRef >> 24) & 0xFF) / 255.0f;		// alpha
							r = ((inColorRef >> 16) & 0xFF) / 255.0f;		// red
							g = ((inColorRef >> 8) & 0xFF) / 255.0f;		// green
							b = ((inColorRef >> 0) & 0xFF) / 255.0f;		// blue
						}
				
	inline UInt32		GetColorRef() const
						{
							const UInt32 cr_r = (UInt32)(r * 255.0 + 0.5f);
							const UInt32 cr_g = (UInt32)(g * 255.0 + 0.5f);
							const UInt32 cr_b = (UInt32)(b * 255.0 + 0.5f);
							const UInt32 cr_a = (UInt32)(a * 255.0 + 0.5f);
							
							return ((cr_a << 24) | (cr_r << 16) | (cr_g << 8) | cr_b);
						}
						
	
	inline RGBColor		GetRGBColor() const
						{
							RGBColor rgb;
							
							rgb.red		= (UInt16) round(65535.0 * r);
							rgb.green	= (UInt16) round(65535.0 * g);
							rgb.blue	= (UInt16) round(65535.0 * b);
							
							return rgb;
						}
	
#endif

public:
	float	r;
	float	g;
	float	b;
	float	a;
};

#endif
