// ===========================================================================
//	CGLShader					(c) 2012 Mark F. Coniglio. All rights reserved
// ===========================================================================

#ifndef _H_CGLShader
#define	_H_CGLShader
#pragma once

#include "OpenGLIncludes.h"

#include "IzzyFrameBufferBase.h"

#if TARGET_OS_WIN
#include "WGLProcAccess.h"
#endif

#include <string>
#include <vector>

class CGLContext;

// ---------------------------------------------------------------------------------

static const size_t kCGLShader_InvalidUniformIndex = ((size_t) -1);

class CGLUniformParam
{
public:

	CGLUniformParam() :
		mParamName(),
		mParamSize(0),
		mUniformLocation(-1)
	{
	}
	
	CGLUniformParam(
		const GLcharARB*	inParamName,
		GLuint				inParamSize,
		GLint				inUniformLocation) :
			mParamName(inParamName),
			mParamSize(inParamSize),
			mUniformLocation(inUniformLocation)
	{
	}

	std::string mParamName;
	GLuint		mParamSize;
	GLint		mUniformLocation;
};

typedef std::vector<CGLUniformParam>	CUniformParamList;

class CGLShader : public CUniformParamList
{
public:
							enum GLSLErrorLogType {
								kGLSL_ErrorLog_Shader,
								kGLSL_ErrorLog_Program
							};
							
							CGLShader(
								CIZGLFunctions*		inCtxAccessor);
		
	virtual					~CGLShader();
	
	GLhandleARB				CompileShaderProgram(
								const GLenum		inType,
								const GLcharARB*	inSource);
	
	bool					CreateShader(
								const GLcharARB*	inVertexShaderSource,
								const GLcharARB*	inFragmentShaderSource);
	
	GLhandleARB				GetShaderProgramObj() const
								{ return mShaderObj; }
	
	void					EnableShader(
								const GLcharARB*	inParamName1 = NULL,
								GLint				inSize1 = 0,
								float*				inData1 = NULL);
	
	void					SetTextureUnit(
								const char*			inTextureUniformName,
								GLuint				inTextureUnit);
	
	GLint					GetUniformLocation(
								const GLcharARB*	inParamName1);

	GLint					GetAttributeLocation(
								const GLcharARB*	inParamName1);

	size_t					AttachUniform(
								const GLcharARB*	inParamName1,
								GLint				inSize);

	void					SetUniformValue(
								size_t				inParamIndex0,
								const float*		inParamValueArray) const;
	
	void					SetUniformValue(
								size_t				inParamIndex0,
								const GLint*		inParamValueArray) const;
	
	void					DisableShader();
	
	void					DisposeShader();
	
	void					LogErrorMessage(
								GLSLErrorLogType	inErrorLogType,
								GLhandleARB			inObject);
	
	std::string				GetErrors() const;

private:
	CIZGLFunctions*						__ctx__accessor;
	GLhandleARB							mShaderObj;
	bool								mShaderEnabled;
	std::string							mErrors;
};
#endif
