// ===========================================================================
//	COpenGLUtil					 (C)2003 Mark F. Coniglio. All rights reserved.
// ===========================================================================
#ifndef _H_COpenGLUtil
#define _H_COpenGLUtil
#pragma once

#include "OpenGLIncludes.h"
#include "CFloatRect.h"
#include "CGLColor.h"
#include "ColorRefs.h"

// ---------------------------------------------------------------------------
// COpenGLUtil DEFINES
// ---------------------------------------------------------------------------

// to send OpenGL errors to Isaodra's Monitor Window, turn this flag on
#define SEND_GL_ERRORS_TO_MONITOR			FALSE

// check for OpenGL errors in the debug version if the first flag
// is TRUE
#define CHECK_GL_ERRORS_IN_DEBUG			(TRUE || DEBUG)

// check for OpenGL errors in the release version if the first flag
// is TRUE or if sending errors to Isadora's Monitor Window.
#define CHECK_GL_ERRORS_IN_RELEASE			(FALSE || SEND_GL_ERRORS_TO_MONITOR)

// we need to read and display the file and line numbers if the
// CHECK_GL_ERRORS_IN_RELEASE flag is true
#define	CHECK_GL_ERRORS_FILE_LINE			TRUE // CHECK_GL_ERRORS_IN_RELEASE

// enables or disables OpenGL error checking. In general, this
// should be off for the release version, and on when debugging
#if DEBUG
	#define	CHECK_GL_ERRORS					CHECK_GL_ERRORS_IN_DEBUG
#else
	#define	CHECK_GL_ERRORS					CHECK_GL_ERRORS_IN_RELEASE
#endif

// OPENGL ERROR CHECKING
#if CHECK_GL_ERRORS
	#if CHECK_GL_ERRORS_FILE_LINE
		#define	CheckOpenGLError()			COpenGLUtil::OpenGLReportError(__FILE__, __LINE__)
	#else
		#define	CheckOpenGLError()			COpenGLUtil::OpenGLReportError()
	#endif
#else
	#define	CheckOpenGLError()				{ }
#endif

// if this flag is true, also output verbose OpenGL checks
#define DEBUG_GL_VERBOSE					(FALSE & CHECK_GL_ERRORS)

// VERBOSE OPENGL INFORMATION
#if DEBUG_GL_VERBOSE
	#define DebugLogOpenGLVerbose(...)		COpenGLUtil::GLLogInfo(__VA_ARGS__)
#else
	#define DebugLogOpenGLVerbose(...)		{ }
#endif

// if this flag is true, also output verbose OpenGL checks
#define DEBUG_GL_VERBOSE_FUNCTIONS			(FALSE & DEBUG_GL_VERBOSE)

// VERBOSE OPENGL INFORMATION
#if DEBUG_GL_VERBOSE_FUNCTIONS
	#define DebugLogOpenGLVerboseFunc(...)	COpenGLUtil::GLLogInfo(__VA_ARGS__)
#else
	#define DebugLogOpenGLVerboseFunc(...)	{ }
#endif

// MAC SPECIFIC ERROR CHECKING
#define	CHECK_MAC_AGL_CGL_ERRORS			(TARGET_OS_MAC && CHECK_GL_ERRORS)

#if CHECK_MAC_AGL_CGL_ERRORS
	#define	CheckAGLError()					COpenGLUtil::AGLReportError()
	#define	CheckCGLError(err)				COpenGLUtil::CGLReportError(err)
#else
	#define	CheckAGLError()					{ }
	#define	CheckCGLError()					{ }
#endif

// ---------------------------------------------------------------------------
// COpenGLUtil class
// ---------------------------------------------------------------------------

class COpenGLUtil 
{
public:

	#if CHECK_GL_ERRORS_FILE_LINE
		static void		OpenGLReportError(const char* inFileName, int inLineNumber);
	#else
		static void		OpenGLReportError();
	#endif

		static void		GLLogInfo(const char* fmt, ...);

	#if TARGET_OS_MAC
		static void		AGLReportError();
	
		static void		CGLReportError(CGLError	inError);
	#endif
};

#endif

void
DrawRectGL(
	float				left,
	float				top,
	float				right,
	float				bottom);

void
DrawRectGL(
	const CFloatRect&	r);

void
DrawCheckerboardGL(
	float				left,
	float				top,
	float				right,
	float				bottom,
	float				cbSize,
	const CFloatXY&		offset = CFloatXY());

void
DrawCheckerboardGL(
	const CFloatRect&	r,
	float				cbSize,
	const CFloatXY&		offset = CFloatXY());


void
DrawColoredRect(
	const CFloatRect&	r,
	CGLColor			inColor);

void
DrawHighlightAlongLeftEdge(
	CFloatRect			inRect,
	ColorRef			inColor,
	float				inHighlightBorderInset,
	float				inHighlightBorderSize);

void
DrawHighlightAlongRightEdge(
	CFloatRect			inRect,
	ColorRef			inColor,
	float				inHighlightBorderInset,
	float				inHighlightBorderSize);

void
DrawHighlightAlongTopEdge(
	CFloatRect			inRect,
	ColorRef			inColor,
	float				inHighlightBorderInset,
	float				inHighlightBorderSize);

void
DrawHighlightAlongBottomEdge(
	CFloatRect			inRect,
	ColorRef			inColor,
	float				inHighlightBorderInset,
	float				inHighlightBorderSize);

ColorRef
HSBToColorRef(
	float hue,
	float saturation,
	float brightness);