// ===========================================================================
// CRect
// (C) 2002 Mark F. Coniglio.  all rights reserved.
// ===========================================================================
//
//	Handy rect encapsulating class

#ifndef _H_CRect
#define _H_CRect

#include "IZGraphics.h"

#include "GraphicsHelpers.h"

class CRect
{
public:
					inline CRect()
					{
						::IzzySetRect(&mRect, 0,0,0,0);
					}
					
					inline CRect(const IzRect& inRect)
					{
						mRect = inRect;
					}
					
#if TARGET_OS_MAC
					inline CRect(const CGRect& inRect)
					{
						mRect.left = floor(inRect.origin.x);
						mRect.top = floor(inRect.origin.y);
						mRect.right = ceil(inRect.origin.x + inRect.size.width);
						mRect.bottom = ceil(inRect.origin.y + inRect.size.height);
					}
#endif

					inline CRect(const RECT& inRect)
					{
						mRect.left = inRect.left;
						mRect.top = inRect.top;
						mRect.right = inRect.right;
						mRect.bottom = inRect.bottom;
					}
					
					inline CRect(IzCoord inLeft, IzCoord inTop, IzCoord inRight, IzCoord inBottom)
					{
						mRect.left = inLeft;
						mRect.top = inTop;
						mRect.right = inRight;
						mRect.bottom = inBottom;
					}
	
#if TARGET_OS_MAC
	inline operator	CGRect() const
					{
						return CGRectMake(mRect.left, mRect.top, mRect.right - mRect.left, mRect.bottom - mRect.top);
					}

#ifdef _OBJC_OBJC_H_
#if IZ_32BIT
	inline operator	NSRect() const
					{
						return NSMakeRect(mRect.left, mRect.top, mRect.right - mRect.left, mRect.bottom - mRect.top);
					}
	#endif
#endif

#endif

	inline operator	RECT() const
					{
						RECT outRect;
						IzzySetRect(&outRect, mRect.left, mRect.top, mRect.right, mRect.bottom);
						return outRect;
					}
	
	inline operator	void*()					{ return &mRect; }
	inline operator	const void*() const		{ return &mRect; }
	inline operator	IzRect*()				{ return &mRect; }
	inline operator	const IzRect*() const	{ return &mRect; }
	inline operator	IzRect&()				{ return mRect; }
	inline operator	const IzRect&() const	{ return mRect; }
	
	inline CRect&	operator = ( const CRect &inRhs)
						{ mRect = inRhs.mRect; return *this; }

	inline CRect&	operator = ( const IzRect* inRect )
						{ mRect = *inRect; return *this; }

	inline CRect&	operator = ( const IzRect& inRect )
						{ mRect = inRect; return *this; }

	inline CRect	operator + ( const IzRect inRhs)
					{
						CRect outRect;
						::IzzyUnionRect(&inRhs, &mRect, outRect);
						return outRect;
					}
	
	inline CRect&	operator += ( const IzRect inRhs)
						{ ::IzzyUnionRect(&inRhs, &mRect, &mRect); return *this; }
	
	inline bool		operator == ( const CRect& inRhs ) const
					{
						return ::IzzyEqualRect(inRhs, &mRect);
					}
				
	inline bool		operator == ( const IzRect& inRhs ) const
					{
						return ::IzzyEqualRect(&inRhs, &mRect);
					}

	inline bool		operator != ( const CRect& inRhs ) const
					{
						return !::IzzyEqualRect(inRhs, &mRect);
					}
				
	inline bool		operator != ( const IzRect& inRhs ) const
					{
						return !::IzzyEqualRect(&inRhs, &mRect);
					}

	CRect			operator * (float inScalingFactor) const
					{
						return CRect(
							mRect.left * inScalingFactor,
							mRect.top * inScalingFactor,
							mRect.right * inScalingFactor,
							mRect.bottom * inScalingFactor);
					}

	CRect&			operator *= (float inScalingFactor)
					{
						mRect.left *= inScalingFactor;
						mRect.top *= inScalingFactor;
						mRect.right *= inScalingFactor;
						mRect.bottom *= inScalingFactor;
						return *this;
					}

	CRect			operator / (float inScalingFactor) const
					{
						return CRect(
							mRect.left / inScalingFactor,
							mRect.top / inScalingFactor,
							mRect.right / inScalingFactor,
							mRect.bottom / inScalingFactor);
					}

	CRect&			operator /= (float inScalingFactor)
					{
						mRect.left /= inScalingFactor;
						mRect.top /= inScalingFactor;
						mRect.right /= inScalingFactor;
						mRect.bottom /= inScalingFactor;
						return *this;
					}

	CRect			operator & (const CRect& inRect) const
					{
						CRect outRect;
						::IzzySectRect(*this, inRect, outRect);
						return outRect;
					}

	CRect&			operator &= (const CRect& inRect)
					{
						::IzzySectRect(*this, inRect, *this);
						return *this;
					}

	inline IzCoord	left() const
						{ return mRect.left; }
					
	inline IzCoord	top() const
						{ return mRect.top; }
	
	inline IzCoord	right() const
						{ return mRect.right; }
					
	inline IzCoord	bottom() const
						{ return mRect.bottom; }
					
	inline IzCoord	width() const
						{ return mRect.right - mRect.left; }
		
	inline IzCoord	height() const
						{ return mRect.bottom - mRect.top; }
	
	inline float	horz_center_float() const
						{ return mRect.left + float(width()) / 2.0f; }
	
	inline float	vert_center_float() const
						{ return mRect.top + float(height()) / 2.0f; }
	
	inline IzCoord	horz_center() const
						{ return mRect.left + width() / 2; }
						
	inline IzCoord	vert_center() const
						{ return mRect.top + height() / 2; }
						
	inline IzPoint	get_top_left() const
					{
						IzPoint p;
						p.h = mRect.left;
						p.v = mRect.top;
						return p;
					}
						
	inline IzPoint	get_bottom_right() const
					{
						IzPoint p;
						p.h = mRect.right;
						p.v = mRect.bottom;
						return p;
					}
						
	inline IzPoint	get_negated_top_left() const
					{
						IzPoint p;
						p.h = -mRect.left;
						p.v = -mRect.top;
						return p;
					}
						
	inline IzPoint	get_center() const
					{
						IzPoint p;
						p.h = mRect.left + width() / 2;
						p.v = mRect.top + height() / 2;
						return p;
					}
						
	inline void		set_center(
						IzCoord	inHorz,
						IzCoord	inVert)
					{
						const IzPoint center = get_center();
						move_by(inHorz - center.h, inVert - center.v);
					}
	
	inline bool		is_zero_rect() const
					{
						return (mRect.left == 0
							&& mRect.top == 0
							&& mRect.right == 0
							&& mRect.bottom == 0);
					}
	
	inline void		reset()
						{ mRect.left = mRect.top = mRect.right = mRect.bottom = 0; }
					
	inline bool		is_empty() const
						{ return ::IzzyEmptyRect(&mRect); }
	
	inline void		move_by(
						IzCoord	inDeltaH,
						IzCoord	inDeltaV)
					{
						mRect.left += inDeltaH;
						mRect.right += inDeltaH;
						mRect.top += inDeltaV;
						mRect.bottom += inDeltaV;
					}
	
	inline void		move_to(
						IzCoord	inLeft,
						IzCoord	inTop)
					{
						IzCoord deltaH = inLeft - mRect.left;
						IzCoord deltaV = inTop - mRect.top;
						mRect.left += deltaH;
						mRect.right += deltaH;
						mRect.top += deltaV;
						mRect.bottom += deltaV;
					}
	
	inline UInt32	area() const
					{
						return (UInt32) width() * (UInt32) height();
					}
	
	inline float	aspect() const
					{
						IzCoord h = height();
						if (h != 0) {
							return float(width()) / float(h);
						} else {
							return 0.0f;
						}
					}
		
	enum {
		kLocateNear_HorzPreferLeft			= (1 << 0),
		kLocateNear_HorzPreferRight			= (1 << 1),
		kLocateNear_HorzPreferCenter		= (1 << 2),
		kLocateNear_VertPreferAbove			= (1 << 3),
		kLocateNear_VertPreferBelow			= (1 << 4),
		kLocateNear_VertPreferCenter		= (1 << 5)
	};
	
	typedef UInt32	LocateNearMode;
	
	inline bool		ensure_within(
						const IzRect&	inLimitRect,
						IzCoord			inHorzMargin = 0,
						IzCoord			inVertMargin = 0)
					{
						IzRect outer = inLimitRect;
						
						outer.left += inHorzMargin;
						outer.right -= inHorzMargin;
						outer.top += inVertMargin;
						outer.bottom -= inVertMargin;
						
						IzCoord deltaH = 0;
						if (mRect.left < outer.left) {
							deltaH = outer.left - mRect.left;
						} else if (mRect.right > outer.right) {
							deltaH = outer.right - mRect.right;
						}
						IzCoord deltaV = 0;
						if (mRect.top < outer.top) {
							deltaV = outer.top - mRect.top;
						} else if (mRect.bottom > outer.bottom) {
							deltaV = outer.bottom - mRect.bottom;
						}
						
						offset(deltaH, deltaV);
						
						return (deltaH != 0 || deltaV != 0);
					}
	
	inline void		locate_near_point(
						LocateNearMode	inLocateMode,
						const IzRect&	inLimitRect,
						IzPoint			inPoint,
						IzCoord			inMinDistance)
					{
						IzCoord deltaHorzMoveToLeft = (inPoint.h - inMinDistance) - mRect.right;
						IzCoord deltaHorzMoveToRight = (inPoint.h + inMinDistance) - mRect.left;
						IzCoord deltaHorzMoveCenter = inPoint.h - (mRect.left + width() / 2);
						
						IzCoord finalDeltaH = 0;
						
						IzCoord testLeft, testRight;
						
						if ((inLocateMode & kLocateNear_HorzPreferRight) != 0) {
							testLeft = mRect.left + deltaHorzMoveToLeft;
							testRight = mRect.right + deltaHorzMoveToRight;
							if (testLeft >= inLimitRect.left && testRight <= inLimitRect.right) {
								finalDeltaH = deltaHorzMoveToRight;
							} else if (testLeft >= inLimitRect.left) {
								finalDeltaH = deltaHorzMoveToLeft;
							} else if (testRight <= inLimitRect.right) {
								finalDeltaH = deltaHorzMoveToRight;
							}
						}
						
						if ((inLocateMode & kLocateNear_HorzPreferLeft) != 0) {
							testLeft = mRect.left + deltaHorzMoveToLeft;
							testRight = mRect.right + deltaHorzMoveToRight;
							if (testLeft >= inLimitRect.left && testRight <= inLimitRect.right) {
								finalDeltaH = deltaHorzMoveToLeft;
							} else if (testRight <= inLimitRect.right) {
								finalDeltaH = deltaHorzMoveToRight;
							} else if (testLeft >= inLimitRect.left) {
								finalDeltaH = deltaHorzMoveToLeft;
							}
						}
						
						if ((inLocateMode & kLocateNear_HorzPreferCenter) != 0) {
							testLeft = mRect.left + deltaHorzMoveCenter;
							testRight = mRect.right + deltaHorzMoveCenter;
							if (testLeft >= inLimitRect.left && testRight <= inLimitRect.right) {
								finalDeltaH = deltaHorzMoveCenter;
							} else if (testLeft < inLimitRect.left) {
								finalDeltaH = inLimitRect.left - mRect.left;
							} else if (testRight > inLimitRect.right) {
								finalDeltaH = inLimitRect.right - mRect.right;
							}
						}
						
						IzCoord deltaVertMoveAbove = (inPoint.v - inMinDistance) - mRect.bottom;
						IzCoord deltaVertMoveBelow = (inPoint.v + inMinDistance) - mRect.top;
						IzCoord deltaVertMoveCenter = inPoint.v - (mRect.top + height() / 2);
						
						IzCoord testTop, testBottom;
						
						IzCoord finalDeltaV = 0;
						
						if ((inLocateMode & kLocateNear_VertPreferAbove) != 0) {
							testTop = mRect.top + deltaVertMoveAbove;
							testBottom = mRect.bottom + deltaVertMoveBelow;
							if (testTop >= inLimitRect.top && testBottom <= inLimitRect.bottom) {
								finalDeltaV = deltaVertMoveBelow;
							} else if (testTop >= inLimitRect.top) {
								finalDeltaV = deltaVertMoveAbove;
							} else if (testBottom <= inLimitRect.bottom) {
								finalDeltaV = deltaVertMoveBelow;
							}
						}
						
						if ((inLocateMode & kLocateNear_VertPreferBelow) != 0) {
							testTop = mRect.top + deltaVertMoveAbove;
							testBottom = mRect.bottom + deltaVertMoveBelow;
							if (testTop >= inLimitRect.top && testBottom <= inLimitRect.bottom) {
								finalDeltaV = deltaVertMoveAbove;
							} else if (testBottom <= inLimitRect.bottom) {
								finalDeltaV = deltaVertMoveBelow;
							} else {
								finalDeltaV = deltaVertMoveAbove;
							}
						}
						
						if ((inLocateMode & kLocateNear_VertPreferCenter) != 0) {
							testTop = mRect.top + deltaVertMoveCenter;
							testBottom = mRect.bottom + deltaVertMoveCenter;
							if (testTop >= inLimitRect.top && testBottom <= inLimitRect.bottom) {
								finalDeltaV = deltaVertMoveCenter;
							} else if (testTop < inLimitRect.top) {
								finalDeltaV = inLimitRect.top - mRect.top;
							} else if (testBottom > inLimitRect.bottom) {
								finalDeltaV = inLimitRect.bottom - mRect.bottom;
							}
						}
						
						move_by(finalDeltaH, finalDeltaV);
					}
					
	inline void		limit_position(
						const IzRect&	inLimitRect)
					{
						IzCoord deltaH = 0, deltaV = 0;
						if (mRect.right > inLimitRect.right) {
							deltaH = inLimitRect.right - mRect.right;
						}
						if (mRect.left < inLimitRect.left) {
							deltaH = inLimitRect.left - mRect.left;
						}
						if (mRect.bottom > inLimitRect.bottom) {
							deltaV = inLimitRect.bottom - mRect.bottom;
						}
						if (mRect.top < inLimitRect.top) {
							deltaV = inLimitRect.top - mRect.top;
						}
						
						move_by(deltaH, deltaV);
					}
					
	inline void		set_center_position_with_limits(
						IzCoord			inHorz,
						IzCoord			inVert,
						const IzRect&	inLimitRect)
					{
						set_center(inHorz, inVert);
						limit_position(inLimitRect);
					}
					
	inline void		set_position(
						IzCoord	inLeft,
						IzCoord	inTop)
							{
								IzCoord offsetH = inLeft - mRect.left;
								IzCoord offsetV = inTop - mRect.top;
								mRect.left += offsetH;
								mRect.right += offsetH;
								mRect.top += offsetV;
								mRect.bottom += offsetV;
							}
	
	
	inline void		set_size(
						IzCoord	inWidth,
						IzCoord	inHeight)
					{
						mRect.right = mRect.left + inWidth;
						mRect.bottom = mRect.top + inHeight;
					}
				
	inline void		set_position_and_size(
						IzCoord	inLeft,
						IzCoord	inTop,
						IzCoord	inWidth,
						IzCoord	inHeight)
					{
						mRect.left = inLeft;
						mRect.top = inTop;
						mRect.right = inLeft + inWidth;
						mRect.bottom = inTop + inHeight;
					}
				
	inline void		set(
						IzCoord	inLeft,
						IzCoord	inTop,
						IzCoord	inRight,
						IzCoord	inBottom)
					{
						mRect.left = inLeft;
						mRect.top = inTop;
						mRect.right = inRight;
						mRect.bottom = inBottom;
					}
				
	inline void		inset(
						IzCoord	inH,
						IzCoord	inV)
							{ ::IzzyInsetRect(&mRect, inH, inV); }

	inline void		inset(
						IzCoord	inLeft,
						IzCoord	inTop,
						IzCoord	inRight,
						IzCoord	inBottom)
					{
						mRect.left += inLeft;
						mRect.right -= inRight;
						mRect.top += inTop;
						mRect.bottom -= inBottom;
					}

	inline void		offset(
						IzCoord	inH,
						IzCoord	inV)
							{ ::IzzyOffsetRect(&mRect, inH, inV); }

	inline void		offset(
						IzPoint	inPoint)
							{ ::IzzyOffsetRect(&mRect, inPoint.h, inPoint.v); }
	
	inline void		offset(
						IzSize	inSize)
							{ ::IzzyOffsetRect(&mRect, inSize.cx, inSize.cy); }
	
	inline bool		contains(
						const IzPoint&		inPoint) const
					{
						return (inPoint.h >= mRect.left && inPoint.h < mRect.right) && (inPoint.v >= mRect.top && inPoint.v < mRect.bottom);
					}
					
	inline bool		contains(
						const IzRect&			inRect) const
					{
						return (inRect.left >= mRect.left && inRect.right <= mRect.right) && (inRect.top >= mRect.top && inRect.bottom <= mRect.bottom);
					}
					

	inline void		set_left(IzCoord inValue) { mRect.left = inValue; }
	inline void		set_right(IzCoord inValue) { mRect.right = inValue; }
	inline void		set_top(IzCoord inValue) { mRect.top = inValue; }
	inline void		set_bottom(IzCoord inValue) { mRect.bottom = inValue; }

	inline void		offset_left(IzCoord inValue) { mRect.left += inValue; }
	inline void		offset_right(IzCoord inValue) { mRect.right += inValue; }
	inline void		offset_top(IzCoord inValue) { mRect.top += inValue; }
	inline void		offset_bottom(IzCoord inValue) { mRect.bottom += inValue; }

	inline void		set_width(
						IzCoord	inWidth)
							{ mRect.right = mRect.left + inWidth; }

	inline void		set_height(
						IzCoord	inHeight)
							{ mRect.bottom = mRect.top + inHeight; }

	inline void		resize_by(
						IzCoord	inDeltaWidth,
						IzCoord	inDeltaHeight)
					{
						mRect.right = mRect.left + (mRect.right - mRect.left) + inDeltaWidth;
						mRect.bottom = mRect.top + (mRect.bottom - mRect.top) + inDeltaHeight;
					}
	
	inline void		resize_width_by(
						IzCoord	inDeltaWidth)
							{ mRect.right = mRect.left + (mRect.right - mRect.left) + inDeltaWidth; }

	inline void		resize_height_by(
						IzCoord	inDeltaHeight)
							{ mRect.bottom = mRect.top + (mRect.bottom - mRect.top) + inDeltaHeight; }

	void			center_inside(
						const CRect&	inRect);
					
	enum HorzAlignCode {
		kHorzAlign_None,
		kHorzAlign_Left,
		kHorzAlign_Center,
		kHorzAlign_Right
	};
	
	enum VertAlignCode {
		kVertAlign_None,
		kVertAlign_Top,
		kVertAlign_Middle,
		kVertAlign_Bottom
	};
	
	void			AlignWithin(
						const CRect&	inLargerRect,
						HorzAlignCode	inHorzAlignCode,
						VertAlignCode	inVertAlignCode);
	
	
	void			LimitRectToMaxWidthAndHeight(
						IzCoord			inMaxWidth,
						IzCoord			inMaxHeight,
						bool			inCenterWithinOriginalBounds = false);
					
	IzRect		mRect;
};

class CRECT
{
public:
					inline CRECT()
					{
						::IzzySetRect(&mRect, 0,0,0,0);
					}
					
					inline CRECT(const RECT& inRect)
					{
						mRect = inRect;
					}
	
					inline CRECT(const IzRect& inRect)
					{
						mRect.left = inRect.left;
						mRect.top = inRect.top;
						mRect.right = inRect.right;
						mRect.bottom = inRect.bottom;
					}
					
					inline CRECT(LONG inLeft, LONG inTop, LONG inRight, LONG inBottom)
					{
						mRect.left = inLeft;
						mRect.top = inTop;
						mRect.right = inRight;
						mRect.bottom = inBottom;
					}
	
#if TARGET_OS_MAC
					inline CRECT(const CGRect& inRect)
					{
						mRect.left = floor(inRect.origin.x);
						mRect.top = floor(inRect.origin.y);
						mRect.right = ceil(inRect.origin.x + inRect.size.width);
						mRect.bottom = ceil(inRect.origin.y + inRect.size.height);
					}
#endif

	inline operator	IzRect() const
					{
						IzRect outRect;
						IzzySetRect(&outRect, mRect.left, mRect.top, mRect.right, mRect.bottom);
						return outRect;
					}
	
	#if TARGET_OS_MAC
	inline operator	CGRect() const
					{
						return CGRectMake(mRect.left, mRect.top, mRect.right - mRect.left, mRect.bottom - mRect.top);
					}
	#endif

	inline operator	void*()					{ return &mRect; }
	inline operator	const void*() const		{ return &mRect; }
	inline operator	RECT*()					{ return &mRect; }
	inline operator	const RECT*() const		{ return &mRect; }
	inline operator	RECT&()					{ return mRect; }
	inline operator	const RECT&() const		{ return mRect; }
	
	inline CRECT&	operator = ( const CRECT &inRhs)
						{ mRect = inRhs.mRect; return *this; }

	inline CRECT&	operator = ( const RECT* inRect )
						{ mRect = *inRect; return *this; }

	inline CRECT&	operator = ( const RECT& inRect )
						{ mRect = inRect; return *this; }

	inline CRECT&	operator += ( const RECT inRhs)
						{ ::IzzyUnionRect(&inRhs, &mRect, &mRect); return *this; }
	
	inline bool		operator == ( const CRECT& inRhs ) const
					{
						return ::IzzyEqualRect(inRhs, &mRect);
					}
				
	inline bool		operator == ( const RECT& inRhs ) const
					{
						return ::IzzyEqualRect(&inRhs, &mRect);
					}

	inline bool		operator != ( const CRECT& inRhs ) const
					{
						return !::IzzyEqualRect(inRhs, &mRect);
					}
				
	inline bool		operator != ( const RECT& inRhs ) const
					{
						return !::IzzyEqualRect(&inRhs, &mRect);
					}

	
	inline LONG		left() const
						{ return mRect.left; }
					
	inline LONG		top() const
						{ return mRect.top; }
	
	inline LONG		right() const
						{ return mRect.right; }
					
	inline LONG		bottom() const
						{ return mRect.bottom; }
					
	inline LONG		width() const
						{ return mRect.right - mRect.left; }
		
	inline LONG		height() const
						{ return mRect.bottom - mRect.top; }
	
	inline float	horz_center_float() const
						{ return mRect.left + float(width()) / 2.0; }
	
	inline float	vert_center_float() const
						{ return mRect.top + float(height()) / 2.0; 	}
	
	inline LONG		horz_center() const
						{ return mRect.left + width() / 2; }
						
	inline LONG		vert_center() const
						{ return mRect.top + height() / 2; }
						
	inline POINT	get_center() const
					{
						POINT p;
						p.x = mRect.left + width() / 2;
						p.y = mRect.top + height() / 2;
						return p;
					}
						
	inline void		set_center(
						LONG	inHorz,
						LONG	inVert)
					{
						const POINT center = get_center();
						move_by(inHorz - center.x, inVert - center.y);
					}
	
	inline bool		is_zero_rect() const
					{
						return (mRect.left == 0
							&& mRect.top == 0
							&& mRect.right == 0
							&& mRect.bottom == 0);
					}
	
	inline void		reset()
						{ mRect.left = mRect.top = mRect.right = mRect.bottom = 0; }
					
	inline bool		is_empty() const
						{ return ::IzzyEmptyRect(&mRect); }
	
	inline void		move_by(
						LONG	inDeltaH,
						LONG	inDeltaV)
					{
						mRect.left += inDeltaH;
						mRect.right += inDeltaH;
						mRect.top += inDeltaV;
						mRect.bottom += inDeltaV;
					}
	
	inline void		move_to(
						LONG	inLeft,
						LONG	inTop)
					{
						LONG deltaH = inLeft - mRect.left;
						LONG deltaV = inTop - mRect.top;
						mRect.left += deltaH;
						mRect.right += deltaH;
						mRect.top += deltaV;
						mRect.bottom += deltaV;
					}
	inline LONG		area() const
					{
						return (LONG) width() * (LONG) height();
					}
	
	inline float	aspect() const
					{
						LONG h = height();
						if (h != 0) {
							return float(width()) / float(h);
						} else {
							return 0.0f;
						}
					}
						
	inline void		set_center_position(
						LONG	inHorz,
						LONG	inVert)
					{
						POINT center = get_center();
						move_by(inHorz - center.x, inVert - center.y);
					}
	
	enum {
		kLocateNear_HorzPreferLeft			= (1 << 0),
		kLocateNear_HorzPreferRight			= (1 << 1),
		kLocateNear_HorzPreferCenter		= (1 << 2),
		kLocateNear_VertPreferAbove			= (1 << 3),
		kLocateNear_VertPreferBelow			= (1 << 4),
		kLocateNear_VertPreferCenter		= (1 << 5)
	};
	
	typedef UInt32	LocateNearMode;
	
	inline void		locate_near_point(
						LocateNearMode	inLocateMode,
						const RECT&		inLimitRect,
						IzPoint			inPoint,
						LONG			inMinDistance)
					{
						LONG deltaHorzMoveToLeft = (inPoint.h - inMinDistance) - mRect.right;
						LONG deltaHorzMoveToRight = (inPoint.h + inMinDistance) - mRect.left;
						LONG deltaHorzMoveCenter = inPoint.h - (mRect.left + width() / 2);
						
						LONG finalDeltaH = 0;
						
						LONG testLeft, testRight;
						
						if ((inLocateMode & kLocateNear_HorzPreferRight) != 0) {
							testLeft = mRect.left + deltaHorzMoveToLeft;
							testRight = mRect.right + deltaHorzMoveToRight;
							if (testLeft >= inLimitRect.left && testRight <= inLimitRect.right) {
								finalDeltaH = deltaHorzMoveToRight;
							} else if (testLeft >= inLimitRect.left) {
								finalDeltaH = deltaHorzMoveToLeft;
							} else if (testRight <= inLimitRect.right) {
								finalDeltaH = deltaHorzMoveToRight;
							}
						}
						
						if ((inLocateMode & kLocateNear_HorzPreferLeft) != 0) {
							testLeft = mRect.left + deltaHorzMoveToLeft;
							testRight = mRect.right + deltaHorzMoveToRight;
							if (testLeft >= inLimitRect.left && testRight <= inLimitRect.right) {
								finalDeltaH = deltaHorzMoveToLeft;
							} else if (testRight <= inLimitRect.right) {
								finalDeltaH = deltaHorzMoveToRight;
							} else if (testLeft >= inLimitRect.left) {
								finalDeltaH = deltaHorzMoveToLeft;
							}
						}
						
						if ((inLocateMode & kLocateNear_HorzPreferCenter) != 0) {
							testLeft = mRect.left + deltaHorzMoveCenter;
							testRight = mRect.right + deltaHorzMoveCenter;
							if (testLeft >= inLimitRect.left && testRight <= inLimitRect.right) {
								finalDeltaH = deltaHorzMoveCenter;
							} else if (testLeft < inLimitRect.left) {
								finalDeltaH = inLimitRect.left - mRect.left;
							} else if (testRight > inLimitRect.right) {
								finalDeltaH = inLimitRect.right - mRect.right;
							}
						}
						
						LONG deltaVertMoveAbove = (inPoint.v - inMinDistance) - mRect.bottom;
						LONG deltaVertMoveBelow = (inPoint.v + inMinDistance) - mRect.top;
						LONG deltaVertMoveCenter = inPoint.v - (mRect.top + height() / 2);
						
						LONG testTop, testBottom;
						
						LONG finalDeltaV = 0;
						
						if ((inLocateMode & kLocateNear_VertPreferAbove) != 0) {
							testTop = mRect.top + deltaVertMoveAbove;
							testBottom = mRect.bottom + deltaVertMoveBelow;
							if (testTop >= inLimitRect.top && testBottom <= inLimitRect.bottom) {
								finalDeltaV = deltaVertMoveBelow;
							} else if (testTop >= inLimitRect.top) {
								finalDeltaV = deltaVertMoveAbove;
							} else if (testBottom <= inLimitRect.bottom) {
								finalDeltaV = deltaVertMoveBelow;
							}
						}
						
						if ((inLocateMode & kLocateNear_VertPreferBelow) != 0) {
							testTop = mRect.top + deltaVertMoveAbove;
							testBottom = mRect.bottom + deltaVertMoveBelow;
							if (testTop >= inLimitRect.top && testBottom <= inLimitRect.bottom) {
								finalDeltaV = deltaVertMoveAbove;
							} else if (testBottom <= inLimitRect.bottom) {
								finalDeltaV = deltaVertMoveBelow;
							} else {
								finalDeltaV = deltaVertMoveAbove;
							}
						}
						
						if ((inLocateMode & kLocateNear_VertPreferCenter) != 0) {
							testTop = mRect.top + deltaVertMoveCenter;
							testBottom = mRect.bottom + deltaVertMoveCenter;
							if (testTop >= inLimitRect.top && testBottom <= inLimitRect.bottom) {
								finalDeltaV = deltaVertMoveCenter;
							} else if (testTop < inLimitRect.top) {
								finalDeltaV = inLimitRect.top - mRect.top;
							} else if (testBottom > inLimitRect.bottom) {
								finalDeltaV = inLimitRect.bottom - mRect.bottom;
							}
						}
						
						move_by(finalDeltaH, finalDeltaV);
					}
					
	inline void		limit_position(
						const RECT&	inLimitRect)
					{
						LONG deltaH = 0, deltaV = 0;
						if (mRect.right > inLimitRect.right) {
							deltaH = inLimitRect.right - mRect.right;
						}
						if (mRect.left < inLimitRect.left) {
							deltaH = inLimitRect.left - mRect.left;
						}
						if (mRect.bottom > inLimitRect.bottom) {
							deltaV = inLimitRect.bottom - mRect.bottom;
						}
						if (mRect.top < inLimitRect.top) {
							deltaV = inLimitRect.top - mRect.top;
						}
						
						move_by(deltaH, deltaV);
					}
					
	inline void		set_center_position_with_limits(
						LONG		inHorz,
						LONG		inVert,
						const RECT&	inLimitRect)
					{
						set_center_position(inHorz, inVert);
						limit_position(inLimitRect);
					}
					
	inline void		set_position(
						LONG	inLeft,
						LONG	inTop)
							{
								LONG offsetH = inLeft - mRect.left;
								LONG offsetV = inTop - mRect.top;
								mRect.left += offsetH;
								mRect.right += offsetH;
								mRect.top += offsetV;
								mRect.bottom += offsetV;
							}
	
	
	inline void		set_size(
						LONG	inWidth,
						LONG	inHeight)
					{
						mRect.right = mRect.left  + inWidth;
						mRect.bottom = mRect.top + inHeight;
					}
				
	inline void		set_position_and_size(
						LONG	inLeft,
						LONG	inTop,
						LONG	inWidth,
						LONG	inHeight)
					{
						mRect.left = inLeft;
						mRect.top = inTop;
						mRect.right = inLeft + inWidth;
						mRect.bottom = inTop + inHeight;
					}
				
	inline void		SetRect(
						LONG	inLeft,
						LONG	inTop,
						LONG	inRight,
						LONG	inBottom)
					{
						mRect.left = inLeft;
						mRect.top = inTop;
						mRect.right = inRight;
						mRect.bottom = inBottom;
					}
				
	inline void		inset(
						LONG	inH,
						LONG	inV)
							{ ::IzzyInsetRect(&mRect, inH, inV); }

	inline void		offset(
						LONG	inH,
						LONG	inV)
							{ ::IzzyOffsetRect(&mRect, inH, inV); }
	
	inline void		offset(
						POINT	inPoint)
							{ ::IzzyOffsetRect(&mRect, inPoint.x, inPoint.y); }
	
	inline void		offset(
						SIZE	inSize)
							{ ::IzzyOffsetRect(&mRect, inSize.cx, inSize.cy); }
	
	inline void		inset(
						LONG	inLeft,
						LONG	inTop,
						LONG	inRight,
						LONG	inBottom)
					{
						mRect.left += inLeft;
						mRect.right -= inRight;
						mRect.top += inTop;
						mRect.bottom -= inBottom;
					}

	inline bool		contains(
						const IzPoint&		inPoint) const
					{
						return (inPoint.h >= mRect.left && inPoint.h < mRect.right) && (inPoint.v >= mRect.top && inPoint.v < mRect.bottom);
					}
					

	inline bool		contains(
						const RECT&			inRect) const
					{
						return (inRect.left >= mRect.left && inRect.right <= mRect.right) && (inRect.top >= mRect.top && inRect.bottom <= mRect.bottom);
					}

	inline void		set_left(LONG inValue) { mRect.left = inValue; }
	inline void		set_right(LONG inValue) { mRect.right = inValue; }
	inline void		set_top(LONG inValue) { mRect.top = inValue; }
	inline void		set_bottom(LONG inValue) { mRect.bottom = inValue; }

	inline void		offset_left(LONG inValue) { mRect.left += inValue; }
	inline void		offset_right(LONG inValue) { mRect.right += inValue; }
	inline void		offset_top(LONG inValue) { mRect.top += inValue; }
	inline void		offset_bottom(LONG inValue) { mRect.bottom += inValue; }

	inline void		set_height(
						LONG	inHeight)
							{ mRect.bottom = mRect.top + inHeight; }

	inline void		set_width(
						LONG	inWidth)
							{ mRect.right = mRect.left + inWidth; }

	void			center_inside(
						const CRECT&	inRect);
					
	void			LimitRectToMaxWidthAndHeight(
						LONG			inMaxWidth,
						LONG			inMaxHeight,
						bool			inCenterWithinOriginalBounds = false);
					
	RECT		mRect;
};

#ifdef __ISADORA_APP__
void
UnionRect32(
	SRect32*	inA,
	SRect32*	inB,
	SRect32*	outRect);
#endif

#endif
