//===========================================================================
//	CoreImageBuffer				   (C)2005 Mark F. Coniglio. All rights reserved.
// ===========================================================================

#ifndef _H_CoreImageBuffer
#define	_H_CoreImageBuffer
#pragma once

#include "CoreImageCommon.h"

#if COMPILE_CORE_VIDEO

// system includes
#include "OpenGLIncludes.h"

// isadora includes
#include "IsadoraTypes.h"
#include "IsadoraDebugging.h"

// other
#include "CoreImageBase.h"
#include "ValueCommon.h"
#include "ColorRefs.h"

#include "IsadoraDebugging.h"

// ===========================================================================
// FORWARD DECLARATIONS

struct ImageBuffer;

// ===========================================================================
// DEBUGGING

#define ENABLE_CI_LOG_INFO					(FALSE && DEBUG)

#if ENABLE_CI_LOG_INFO
	#define	ci_log_info						IzzyLogInfo_
#else
	#define ci_log_info(...)				{ }
#endif

// ===========================================================================
// DEFINES

#ifndef CHECK_CIBUFFER_BUFFER_MAGIC
#define CHECK_CIBUFFER_BUFFER_MAGIC		TRUE
#endif

#define	DebugCheckCIBufferClear(a)	{ IzzyDebugAssert_((a)->mMagic == 0 && (a)->mImage == NULL); }

#if (DEBUG || CHECK_CIBUFFER_BUFFER_MAGIC)
	#define CIBuffer_CheckMagic(ib)	IzzyAssert_(ib == NULL || (ib)->mMagic == kCoreImageBufferMagic)
#else
	#define CIBuffer_CheckMagic(ib)	{ }
#endif

// ===========================================================================
// GLOBALS

extern CoreImageBuffer	sCoreImageBuffer_CLEAR;

// ===========================================================================
// FUNCTIONS

#ifdef __cplusplus
extern "C" {
#endif

void
ReleaseCIFilter(
	CI_Filter	inFilter);

void
ReleaseCIFilterIfNotNull(
	CI_Filter	inFilter);

CI_Filter
CreateConstantColorFilter();

void
UpdateConstantColorWithARGBColorRef(
	CI_Filter		inConstantColorFilter,
	ColorRef		inColorRef);

CI_Image
GetConstantColorImage(
	CI_Filter		inConstantColorFilter);

CI_Image
GetConstantColorImageOfSize(
	CI_Filter		inConstantColorFilter,
	float			inWidth,
	float			inHeight,
	float			inLeft,
	float			inBottom);
	
CI_Filter
CreateCropFilter();

CI_Image
CropImageAbsolute(
	CI_Filter		inCropFilter,
	CI_Image		inImage,
	float			inWidth,
	float			inHeight,
	float			inLeft,
	float			inBottom,
	bool			inReleaseInputImage);
	
CI_Image
CropImageByPercentage(
	CI_Filter		inCropFilter,
	CI_Image		inImage,
	float			inPercentWidth,
	float			inPercentHeight,
	float			inPercentCtrH,
	float			inPercentCtrV,
	bool			inReleaseInputImage);

CI_Image
CropImage(
	CI_Image		inImage,
	float			inWidth,
	float			inHeight,
	float			inLeft,
	float			inBottom,
	bool			inReleaseInputImage);

CI_Image
TranslateImage(
	CI_Image	inImage,
	float		inTranslateX,
	float		inTranslateY);

CI_Image
ScaleImage(
	CI_Image	inImage,
	float		inScaleX,
	float		inScaleY);

CI_Image
ScaleImageTo(
	CI_Image	inImage,
	float		inDesiredWidth,
	float		inDesiredHeight);

CI_Image
FlipImage(
	CI_Image	inImage,
	bool		inFlipHorz,
	bool		inFlipVert);

// ------------------------------------------------------------

CI_Filter
CreateHueFilter();

CI_Image
HueFilter_AdjustHue(
	CI_Filter	inHueFilter,
	CI_Image	inImage,
	float		inAngle);

CI_Filter
CreateColorControlsFilter();
	
CI_Image
ColorControls_AdjustColor(
	CI_Filter	inColorControlsFilter,
	CI_Image	inImage,
	float		inSaturation,
	float		inBrightness,
	float		inContrast);

CI_Filter
CreateDifferenceBlendModeFilter();
	
CI_Image
DifferenceBlendMode_GetDiff(
	CI_Filter	inColorControlsFilter,
	CI_Image	inImage1,
	CI_Image	inImage2);

CI_Filter
CreateAdditionCompositingFilter();

CI_Image
AdditionCompositing_GetSum(
	CI_Filter	inAdditionCompositingFilter,
	CI_Image	inImage1,
	CI_Image	inImage2);

CI_Filter
CreateMultiplyBlendMode();

CI_Image
MultiplyBlendMode_Multiply(
	CI_Filter	inMultiplyCompositingFilter,
	CI_Image	inImage1,
	CI_Image	inImage2);


CI_Filter
CreateMaskToAlphaFilter();

CI_Image
MaskToAlpha_GetOutput(
	CI_Filter	inMaskToAlphaFilter,
	CI_Image	inImage1);

CI_Filter
CreateBlendWithAlphaMaskFilter();

CI_Filter
CreateBlendWithMaskFilter();

CI_Image
BlendWithAlphaMask_GetOutput(
	CI_Filter	inAdditionCompositingFilter,
	CI_Image	inImageSource,
	CI_Image	inImageBackground,
	CI_Image	inImageMask);

CI_Filter
CreateSourceOverCompositingFilter();

CI_Image
SourceOver_GetOutput(
	CI_Filter	inAdditionCompositingFilter,
	CI_Image	inImage1,
	CI_Image	inImage2);

// ------------------------------------------------------------

Boolean
IsCIBufferValid(
	const CoreImageBuffer*	inCIBuffer);
	
void
InitCIBuffer(
	CoreImageBuffer*		inCIBuffer);

void
UninitCIBuffer(
	CoreImageBuffer*		inCIBuffer);

void
ClearCIBuffer(
	CoreImageBuffer*		inCIBuffer);

void
CIBufferIncrementFrameCount(
	CoreImageBuffer*		inCIBuffer);

CI_Image
CIImageWithImageBuffer(
	struct ImageBuffer*		inImageBuffer,
	CI_Filter				inFilter);
	
CI_Image
CreateCIImageFromCVImageBufferRef(
	CV_ImageBufferRef		inCVImageBuffer,
	CI_Filter				inFilter);
	

/*
CI_Image
CreateCIImageFromCGmageRef(
	CGImageRef			inImageRef,
	CI_Filter			inFilter);
*/

UInt32
CIImageRetainCount(
	CI_Image				inCIImage);
	
void
RetainCIImage(
	CI_Image				inCIImage);

void
ReleaseCIImage(
	CI_Image				inCIImage);

bool
SetCIBufferFromCIImageWithTexture(
	CoreImageBuffer*	inCIBuffer,
	StageIndexT			inTargetStageIndex0,
	GLuint				inTextureName,
	GLuint				inWidth,
	GLuint				inHeight);

bool												// true if buffer was changed
SetCIBufferFromCIImage(
	CoreImageBuffer*		inCIBuffer,
	CI_Image				inCIImage,
	StageIndexT				inTargetStageIndex0);

void
ReleaseCIContext(
	void*					inCIContext);

void
ReleaseNSOpenGLContext(
	void*					inNSOpenGLContext);
		
void
DrawCIImage_InCIContext_ForGLContext_InfinityLimited(
	CI_Image				inImage,
	CI_Context				inContext,
	const CGRect&			inDstRect);
	
/*
void
DrawCIImageInCGContext_NotInfinityLimited(
	CI_Image				inImage,
	CGContextRef			inContext,
	const CGRect&			inSrcRect,
	const CGRect&			inDstRect);
*/

// N.B. returns true if either dimension was infinite
// output width/height will be FLT_MAX for infinite dimensions
bool
GetCIImageSizeFloat(
	CI_Image				inImage,
	float*					outWidth,
	float*					outHeight);
	
// N.B. returns true if either dimension was infinite
// output width/height will be INT_MAX for infinite dimensions
bool
GetCIImageSize(
	CI_Image				inImage,
	SInt32*					outWidth,
	SInt32*					outHeight);

// N.B. returns true if source image was limited in either dimension
bool
GetCIImageSizeInfinityLimitedFloat(
	CI_Image				inImage,
	float*					outWidth,
	float*					outHeight);

// N.B. returns true if source image was limited in either dimension
bool
GetCIImageSizeInfinityLimited(
	CI_Image				inImage,
	SInt32*					outWidth,
	SInt32*					outHeight);

// COLOR MATRIX FILTER
CI_ColorMatrixFilter
CreateColorMatrixFilter();

CI_Image
ColorMatrixFilter_ModulateImageARGB_Individual(
	CI_ColorMatrixFilter	inFilter,
	CI_Image				inImage,
	float*					inARGBArray);

CI_Image
ModulateImageAlphaWithColorMatrixFilter(
	CI_ColorMatrixFilter	inFilter,
	CI_Image				inImage,
	float					inAlpha);
	
CI_Image
ModulateImageRGBWithColorMatrixFilter(
	CI_ColorMatrixFilter	inFilter,
	CI_Image				inImage,
	float					inAlpha);
	
CI_Image
ModulateImageARGBWithColorMatrixFilter(
	CI_ColorMatrixFilter	inFilter,
	CI_Image				inImage,
	float					inAlpha);
	
void
DestroyColorMatrixFilter(
	CI_ColorMatrixFilter	inFilter);
	
#ifdef __cplusplus
}
#endif	// cplusplus

#endif	// #if COMPILE_CORE_VIDEO

#endif	// #ifndef _H_CoreImageBuffer
