// ===========================================================================
// GraphicsHelpers				(C) 2010 Mark F. Coniglio.  all rights reserved.
// ===========================================================================

#ifndef _H_GraphicsHelpers
#define _H_GraphicsHelpers
#pragma once

#include "IZDefinitions.h"
#include "IZGraphics.h"

#include <math.h>

#include "MacCompatibility.h"

#ifdef __cplusplus

#define _min2(a,b) ((a) < (b)) ? (a) : (b)
#define _max2(a,b) ((a) > (b)) ? (a) : (b)
// #define _sect_lines_(a1, a2, b1, b2, o1, o2)	{ (o1) = (o2) = 0; if ((a1) < (a2) && (b1) < (b2)) { if ((a1) < (b1) && (a2) > (b1)) { (o1) = max_2(b1); (o2) = (a2); } else if ((b1) < (a1) && (b2) > (a1)) { (o1) = (a1); (o2) = (b2); } } }

inline void
_sect_lines_(
	IzCoord		a1,
	IzCoord		a2,
	IzCoord		b1,
	IzCoord		b2,
	IzCoord&	o1,
	IzCoord&	o2)
{
	o1 = o2 = 0;
	// if the two line lengths are > 0
	if ((a1) < (a2) && (b1) < (b2)) {
		// if startpoint of segment a is to the left of b, and endpoint of a > startpoint of b
		if ((a1) <= (b1) && (a2) > (b1)) {
			o1 = _max2(a1, b1);
			o2 = _min2(a2, b2);
		// if startpoint of segment b is to the left of a, and endpoint of a > startpoint of b
		} else if ((b1) <= (a1) && (b2) > (a1)) {
			o1 = _max2(a1, b1);
			o2 = _min2(a2, b2);
		}
	}
}

inline void
_sect_lines_(
	LONG	a1,
	LONG	a2,
	LONG	b1,
	LONG	b2,
	LONG&	o1,
	LONG&	o2)
{
	o1 = o2 = 0;
	// if the two line lengths are > 0
	if ((a1) < (a2) && (b1) < (b2)) {
		// if startpoint of segment a is to the left of b, and endpoint of a > startpoint of b
		if ((a1) <= (b1) && (a2) > (b1)) {
			o1 = _max2(a1, b1);
			o2 = _min2(a2, b2);
		// if startpoint of segment b is to the left of a, and endpoint of a > startpoint of b
		} else if ((b1) <= (a1) && (b2) > (a1)) {
			o1 = _max2(a1, b1);
			o2 = _min2(a2, b2);
		}
	}
}

// #pragma mark ----- POINT FUNCTIONS -----

// POINT FUNCTIONS

inline void
IzzySetPt(
	IzPoint*		p,
	IzCoord			h,
	IzCoord			v)
{
	p->h = h;
	p->v = v;
}

// return true if the specified point lies within the specified rectangle
inline bool
IzzyPtInRect(
	IzPoint			inPoint,
	const IzRect*	inRect)
{
	return (inPoint.h >= inRect->left && inPoint.h < inRect->right) && (inPoint.v >= inRect->top && inPoint.v < inRect->bottom);
}

// add point a and b
inline bool
IzzyEqualPt(
	IzPoint			inA,
	IzPoint			inB)
{
	return (inA.h == inB.h && inA.v == inB.v);
}


// add point a and b
inline void
IzzyAddPoints(
	IzPoint			inA,
	IzPoint			inB,
	IzPoint*		outPt)
{
	outPt->h = inA.h + inB.h;
	outPt->v = inA.v + inB.v;
}

// subtracts point b from point a
inline void
IzzySubPoints(
	IzPoint			inFromPt,
	IzPoint			inSubPt,
	IzPoint*		outPt)
{
	outPt->h = inFromPt.h - inSubPt.h;
	outPt->v = inFromPt.v - inSubPt.v;
}

// #pragma mark ----- LINE FUNCTIONS -----

// LINE FUNCTIONS

// returns true if the two coincident lines overlap; note that it is not 
// required that 'a' and 'b' are specified in ascending order

static inline bool
IzzySectLines(
	IzCoord		a1,		// starting coordinate of first line
	IzCoord		b1,		// ending coordinate of first line
	IzCoord		a2,		// starting coordinate of second line
	IzCoord		b2,		// ending coordinate of second line
	IzCoord&	outA,	// the intersection, lower coordinate
	IzCoord&	outB)	// the intersection, higher coordinate
{
	IzCoord temp;
	
	if (a1 > b1) {
		temp = a1;
		a1 = b1;
		b1 = temp;
	}
	if (a2 > b2) {
		temp = a2;
		a2 = b2;
		b2 = temp;
	}

	_sect_lines_(a1, b1, a2, b2, outA, outB);
	
	return outA < outB;
}


// #pragma mark ----- RECTANGLE MANIPULATION (IZRECT / MAC OS RECT) -----

// RECTANGLE MANIPULATION (IZRECT / MAC OS RECT)

inline void
IzzySetRect(
	IzRect*				ioRect,
	IzCoord				inLeft,
	IzCoord				inTop,
	IzCoord				inRight,
	IzCoord				inBottom)
{
	ioRect->left	= inLeft;
	ioRect->right	= inRight;
	ioRect->top		= inTop;
	ioRect->bottom	= inBottom;
}

inline void
IzzyOffsetRect(
	IzRect*		ioRect,
	IzCoord		dx,
	IzCoord		dy)
{
	ioRect->left	+= dx;
	ioRect->right	+= dx;
	ioRect->top		+= dy;
	ioRect->bottom	+= dy;
}

inline void
IzzyInsetRect(
	IzRect*		ioRect,
	IzCoord		dx,
	IzCoord		dy)
{
	ioRect->left	+= dx;
	ioRect->right	-= dx;
	ioRect->top		+= dy;
	ioRect->bottom	-= dy;
}

inline bool
IzzyEmptyRect(
	const IzRect*			inRect)
{
	return (inRect->left >= inRect->right || inRect->top >= inRect->bottom);
}

inline bool
IzzyEqualRect(
	const IzRect*			r1,
	const IzRect*			r2)
{
	return ((r1)->left == (r2)->left && (r1)->top == (r2)->top && (r1)->right == (r2)->right && (r1)->bottom == (r2)->bottom);
}

inline void
IzzyUnionRect(
  const IzRect *  s1,
  const IzRect *  s2,
  IzRect *        d)
{
	bool empty1 = IzzyEmptyRect(s1);
	bool empty2 = IzzyEmptyRect(s2);
	
	if (empty1 == false && empty2 == false) {
		(d)->left = _min2((s1)->left, (s2)->left); (d)->top = _min2((s1)->top, (s2)->top); (d)->right = _max2((s1)->right, (s2)->right); (d)->bottom = _max2((s1)->bottom, (s2)->bottom);
	} else if (empty1 == true && empty2 == false) {
		*d = *s2;
	} else if (empty1 == false && empty2 == true) {
		*d = *s1;
	} else {
		(d)->left = (d)->top = (d)->right = (d)->bottom = 0;
	}
}

inline bool
IzzySectRect(
  const IzRect *  s1,
  const IzRect *  s2,
  IzRect *        d)
{
	_sect_lines_((s1)->left, (s1)->right, (s2)->left, (s2)->right, (d)->left, (d)->right);
	_sect_lines_((s1)->top, (s1)->bottom, (s2)->top, (s2)->bottom, (d)->top, (d)->bottom);
	if ((d)->left >= (d)->right || (d)->top >= (d)->bottom) { (d)->left = (d)->right = (d)->top = (d)->bottom = 0; }
	return IzzyEmptyRect(d) == false;
}

inline void
IzzyCalcRectCenter(
	const IzRect&	inRect,
	IzPoint*		outPt)
{
	outPt->h = inRect.left + (inRect.right - inRect.left) / 2;
	outPt->v = inRect.top + (inRect.bottom - inRect.top) / 2;
}

// #pragma mark ----- RECTANGLE MANIPULATION (WIN32 RECT) -----

// RECTANGLE MANIPULATION (WIN32 RECT)

inline void
IzzySetRect(
	RECT*				ioRect,
	LONG				inLeft,
	LONG				inTop,
	LONG				inRight,
	LONG				inBottom)
{
	ioRect->left	= inLeft;
	ioRect->right	= inRight;
	ioRect->top		= inTop;
	ioRect->bottom	= inBottom;
}

inline void
IzzyOffsetRect(
	RECT*		ioRect,
	LONG		dx,
	LONG		dy)
{
	ioRect->left	+= dx;
	ioRect->right	+= dx;
	ioRect->top		+= dy;
	ioRect->bottom	+= dy;
}

inline void
IzzyInsetRect(
	RECT*		ioRect,
	LONG		dx,
	LONG		dy)
{
	ioRect->left	+= dx;
	ioRect->right	-= dx;
	ioRect->top		+= dy;
	ioRect->bottom	-= dy;
}

inline bool
IzzyEmptyRect(
	const RECT*			inRect)
{
	return (inRect->left >= inRect->right || inRect->top >= inRect->bottom);
}

inline bool
IzzyEqualRect(
	const RECT*			r1,
	const RECT*			r2)
{
	return ((r1)->left == (r2)->left && (r1)->top == (r2)->top && (r1)->right == (r2)->right && (r1)->bottom == (r2)->bottom);
}

inline void
IzzyUnionRect(
  const RECT *  s1,
  const RECT *  s2,
  RECT *        d)
{
	bool empty1 = IzzyEmptyRect(s1);
	bool empty2 = IzzyEmptyRect(s2);
	
	if (empty1 == false && empty2 == false) {
		(d)->left = _min2((s1)->left, (s2)->left); (d)->top = _min2((s1)->top, (s2)->top); (d)->right = _max2((s1)->right, (s2)->right); (d)->bottom = _max2((s1)->bottom, (s2)->bottom);
	} else if (empty1 == true && empty2 == false) {
		*d = *s2;
	} else if (empty1 == false && empty2 == true) {
		*d = *s1;
	} else {
		(d)->left = (d)->top = (d)->right = (d)->bottom = 0;
	}
}

inline bool
IzzySectRect(
  const RECT *  s1,
  const RECT *  s2,
  RECT *        d)
{
	_sect_lines_((s1)->left, (s1)->right, (s2)->left, (s2)->right, (d)->left, (d)->right);
	_sect_lines_((s1)->top, (s1)->bottom, (s2)->top, (s2)->bottom, (d)->top, (d)->bottom);
	if ((d)->left >= (d)->right || (d)->top >= (d)->bottom) { (d)->left = (d)->right = (d)->top = (d)->bottom = 0; }
	return IzzyEmptyRect(d) == false;
}

inline void
IzzyCalcRectCenter(
	const RECT&		inRect,
	IzPoint*		outPt)
{
	outPt->h = IzCoord(inRect.left + (inRect.right - inRect.left) / 2);
	outPt->v = IzCoord(inRect.top + (inRect.bottom - inRect.top) / 2);
}

// #pragma mark ----- RECT CONVERSION: WIN32 RECT <-> IzRect (WINDOWS ONLY) -----

// RECT CONVERSION: WIN32 RECT <-> IzRect

inline RECT
IzRectToRECT(
	const IzRect&	inRect)
{
	RECT outRect;
	outRect.left = (IzCoord) inRect.left;
	outRect.top = (IzCoord) inRect.top;
	outRect.right = (IzCoord) inRect.right;
	outRect.bottom = (IzCoord) inRect.bottom;
	return outRect;
}

inline SInt16
LimitIzCoordToSInt16(
	IzCoord		inCoord)
{
	return (SInt16) inCoord;
}

inline SInt16
LimitFloatToSInt16(
	float		inCoord)
{
	if (inCoord < 32768.0f) {
		return -32768;
	} else if (inCoord > 32767.0f) {
		return 32767;
	} else {
		return (SInt16) inCoord;
	}
}

inline IzPoint
QDPointToIzPoint(
	const QDPoint&	inPoint)
{
	IzPoint outPoint;
	outPoint.h = inPoint.h;
	outPoint.v = inPoint.v;
	return outPoint;
}

inline QDPoint
IzPointToQDPoint(
	const IzPoint&	inPoint)
{
	QDPoint outPoint;
	outPoint.h = LimitIzCoordToSInt16(inPoint.h);
	outPoint.v = LimitIzCoordToSInt16(inPoint.v);
	return outPoint;
}

inline IzRect
QDRectToIzRect(
	const QDRect&	inRect)
{
	IzRect outRect;
	outRect.left = (IzCoord) inRect.left;
	outRect.top = (IzCoord) inRect.top;
	outRect.right = (IzCoord) inRect.right;
	outRect.bottom = (IzCoord) inRect.bottom;
	return outRect;
}


inline QDRect
IzRectToQDRect(
	const IzRect&	inRect)
{
	QDRect outRect;
	outRect.left = LimitIzCoordToSInt16(inRect.left);
	outRect.top =  LimitIzCoordToSInt16(inRect.top);
	outRect.right = LimitIzCoordToSInt16(inRect.right);
	outRect.bottom = LimitIzCoordToSInt16(inRect.bottom);
	return outRect;
}


inline IzRect
RECTToIzRect(
	const RECT&	inRect)
{
	IzRect outRect;
	outRect.left = (IzCoord) inRect.left;
	outRect.top = (IzCoord) inRect.top;
	outRect.right = (IzCoord) inRect.right;
	outRect.bottom = (IzCoord) inRect.bottom;
	return outRect;
}

// #pragma mark ----- RECT CONVERSION: NSRect <-> IzRect (MAC OS ONLY) -----

// RECT CONVERSION: NSRect <-> IzRect (MAC OS ONLY)

#ifdef _OBJC_OBJC_H_

inline NSRect
QDRectToNSRect(
	const IzRect& inRect)
{
	NSRect outRect;
	outRect.origin.x = (float) inRect.left;
	outRect.origin.y = (float) inRect.top;
	outRect.size.width = (float) (inRect.right - inRect.left);
	outRect.size.height = (float) (inRect.bottom - inRect.top);
	return outRect;
}

inline IzRect
NSRectToQDRect(
	const NSRect&	inRect)
{
	IzRect outRect;
	outRect.left = LimitFloatToSInt16(floor(inRect.origin.x));
	outRect.top = LimitFloatToSInt16(floor(inRect.origin.y));
	outRect.right = LimitFloatToSInt16(ceil(inRect.origin.x + inRect.size.width));
	outRect.bottom = LimitFloatToSInt16(ceil(inRect.origin.y + inRect.size.height));
	return outRect;
}

#endif	// #ifdef _OBJC_OBJC_H_

// #pragma mark ----- RECT CONVERSION: CGRect <-> IzRect (MAC OS ONLY) -----

// RECT CONVERSION: CGRect <-> IzRect (MAC OS ONLY)

inline CGRect
IzRectToCGRect(
	const IzRect& inRect)
{
	CGRect outRect;
	outRect.origin.x = (float) inRect.left;
	outRect.origin.y = (float) inRect.top;
	outRect.size.width = (float) (inRect.right - inRect.left);
	outRect.size.height = (float) (inRect.bottom - inRect.top);
	return outRect;
}

inline IzRect
CGRectToIzRect(
	const CGRect&	inRect)
{
	IzRect outRect;
	outRect.left = (IzCoord)(floor(inRect.origin.x));
	outRect.top = (IzCoord)(floor(inRect.origin.y));
	outRect.right = (IzCoord)(ceil(inRect.origin.x + inRect.size.width));
	outRect.bottom = (IzCoord)(ceil(inRect.origin.y + inRect.size.height));
	return outRect;
	
}

// #pragma mark ----- CGRECT INSET FUNCTION (MAC OS ONLY) -----

// CGRECT INSET FUNCTION (MAC OS ONLY)

inline void
InsetCGRect(
	CGRect& 	ioRect,
	float		dx,
	float		dy)
{
	ioRect.origin.x += dx;
	ioRect.origin.y += dy;
	ioRect.size.width -= dx * 2;
	ioRect.size.height -= dy * 2;
}

#else

#define IzzySetPt(pt, h, v)							{ (pt)->h = (h); (pt)->v = (v); }
#define	IzzyPtInRect(inPoint, inRect)				( ((inPoint).h >= (inRect)->left && (inPoint).h < (inRect)->right) && ((inPoint).v >= (inRect)->top && (inPoint).v < (inRect)->bottom) )
#define	IzzyEqualPt(inA, inB)						( ((inA).h == (inB).h && (inA).v == (inB).v) )
#define IzzyAddPoints(inA, inB, outPt)				{ (outPt)->h = (inA).h + (inB).h; (outPt)->v = (inA).v + (inB).v; }
#define IzzySubPoints(inA, inB, outPt)				{ (outPt)->h = (inA).h - (inB).h; (outPt)->v = (inA).v - (inB).v; }

#define	IzzySetRect(rect, l, t, r, b)				{ (rect)->left = (l); (rect)->top = (t); (rect)->right = (r); (rect)->bottom = (b); }
#define	IzzyOffsetRect(rect, h, v)					{ (rect)->left += (h); (rect)->top += (v); (rect)->right += (h); (rect)->bottom += (v); }
#define	IzzyInsetRect(rect, h, v)					{ (rect)->left += (h); (rect)->top += (v); (rect)->right -= (h); (rect)->bottom -= (v); }
#define IzzyEmptyRect(rect)							( (rect)->left >= (rect)->right || (rect)->top >= (rect)->bottom )
#define IzzyEqualRect(r1, r2)						( ((r1)->left == (r2)->left && (r1)->top == (r2)->top && (r1)->right == (r2)->right && (r1)->bottom == (r2)->bottom) )
#define IzzyUnionRect(s1, s2, d)					{ (d)->left = _min2((s1)->left, (s2)->left); (d)->top = _min2((s1)->top, (s2)->top); (d)->right = _max2((s1)->right, (s2)->right); (d)->bottom = _max2((s1)->bottom, (s2)->bottom); }
		
#define	WinToMacRect(inRect, outRect)				{ (outRect)->left = (inRect).left; (outRect).top = (inRect)->top; (outRect)->right = (inRect).right; (outRect)->bottom = (inRect).bottom; } 
#define	MacToWinRect(inRect, outRect)				{ (outRect)->left = (inRect).left; (outRect).top = (inRect)->top; (outRect)->right = (inRect).right; (outRect)->bottom = (inRect).bottom; } 

#define IzRectToCGRect(inRect, outRect)				{ (outRect)->origin.x = (inRect).left; (outRect)->origin.y = (inRect).top; (outRect)->size.width = (inRect).right - (inRect).left; (outRect)->size.height = (inRect).bottom - (inRect).top; }
#define CGRectToIzRect(inRect, outRect)				{ (outRect)->left = floor((inRect).origin.x); (outRect)->top = floor((inRect).origin.y); (outRect)->right = ceil((inRect).origin.x + (inRect).size.width); (outRect)->bottom = ceil((inRect).origin.y + (inRect).size.height); }
#define InsetCGRect(ioRect, dx, dy)					{ (ioRect).origin.x += dx; (ioRect).origin.y += dy; (ioRect).size.width -= dx * 2; (ioRect).size.height -= dy * 2; }

#endif


#endif
