// =================================================================================
//	IZEvent						(c) 2018 Mark F. Coniglio -- All Rights Reserved
// =================================================================================

#ifndef _H_IZEvent
#define _H_IZEvent

#ifdef __cplusplus

#include "IZDefinitions.h"
#include "IZTime.h"
#include "IZSystem.h"

#include <string>

// ---------------------------------------------------------------------------
//	IzEvent ENUM

enum IzEvent {

	kIzEvent_Null = 0,
	
	kIzEvent_Draw,
	kIzEvent_EraseBackground,
	
	kIzEvent_LeftMouseDown,
	kIzEvent_LeftMouseDrag,
	kIzEvent_LeftMouseUp,
	
	kIzEvent_RightMouseDown,
	kIzEvent_RightMouseDrag,
	kIzEvent_RightMouseUp,

	kIzEvent_MiddleMouseDown,
	kIzEvent_MiddleMouseDrag,
	kIzEvent_MiddleMouseUp,
	
	kIzEvent_MouseEnter,
	kIzEvent_MouseMove,
	kIzEvent_MouseLeave,
	
	kIzEvent_FlagsChanged,		// command, shift, or alt flags changed

	kIzEvent_ScrollWheel,
	kIzEvent_TabletPoint,
	kIzEvent_TabletProximity,
	
#if TARGET_OS_WIN
	kIzEvent_SysKeyDown,		// system key down events: windows only, system keyboard events (WM_KEYDOWN/KEYUP/SYSKEYDOWN/SYSKEYUP)
	kIzEvent_SysKeyUp,			// system key up events: windows only, system keyboard events (WM_KEYDOWN/KEYUP/SYSKEYDOWN/SYSKEYUP)
#endif

	kIzEvent_KeyDown,			// standard key down events: cross platform compatible character strokes
	kIzEvent_KeyUp				// standard key up events: cross platform compatible character strokes
};

// ---------------------------------------------------------------------------
//	IzKeyCode typedef
// ---------------------------------------------------------------------------

typedef uint16_t	IzKeyCode;		// IzKeyCode is a platform specific key code
									// indicating the physical the key was pressed

// ---------------------------------------------------------------------------
//	MODIFIER BITFIEL MASKS: IzModifiers ENUM
// ---------------------------------------------------------------------------

enum {
	kIzActiveFlagBit		= 0,	// activeFlagBit: activate (activateEvt and mouseDown)
	kIzBtnStateBit			= 7,	// btnStateBit: state of button
	kIzCmdKeyBit			= 8,	// cmdKeyBit: command key down
	kIzShiftKeyBit			= 9,	// shiftKeyBit: shift key down
	kIzAlphaLockBit			= 10,	// alphaLockBit: alpha lock down
	kIzOptionKeyBit			= 11,	// optionKeyBit: option key down
	kIzControlKeyBit		= 12	// controlKeyBit: control key down
};

typedef uint16_t IzModifiers;
enum
{
	kIzModifier_None			= 0,
	kIzModifier_Command			= (1 << kIzCmdKeyBit),		// 0x0100: MacOS -> cmdKey, Windows -> Control Key
	kIzModifier_Shift			= (1 << kIzShiftKeyBit),	// 0x0200:
	kIzModifier_AlphaLock		= (1 << kIzAlphaLockBit),	// 0x0400:
	kIzModifier_Alt				= (1 << kIzOptionKeyBit),	// 0x0800:
	kIzModifier_Control			= (1 << kIzControlKeyBit),	// 0x1000: MacOS -> controlKey, Windows -> controlKey + altKey
	
	kIzModifier_AltCommand		= (kIzModifier_Command | kIzModifier_Alt),
	kIzModifier_ShiftCommand	= (kIzModifier_Command | kIzModifier_Shift),
	kIzModifier_ShiftAlt		= (kIzModifier_Shift | kIzModifier_Alt),
	kIzModifier_ShiftAltCommand	= (kIzModifier_Command | kIzModifier_Shift | kIzModifier_Alt)
};

typedef UInt16 MacEventModifers;

// ---------------------------------------------------------------------------
//	CROSS-PLATFORM CHARACTER CODES FOR NON-PRINTABLE CHARACTERS
// ---------------------------------------------------------------------------
// A cross-platform set of unicode characters used to represent non-printable
// keys. When one of these keys is pressed, the matching value from the list
// below will appear in the mKeypressChar member of the CIzEvent class.
//
// Note that this does not include 'tab' or 'return', which are passed back
// in the mKeypressChar as their ASCII eqivalents.
//
// (Adopted from NSEvent.h on MacOS)

enum {
	kIzUpArrowFunctionKey		= 0xF700,
	kIzDownArrowFunctionKey		= 0xF701,
	kIzLeftArrowFunctionKey		= 0xF702,
	kIzRightArrowFunctionKey	= 0xF703,
	kIzF1FunctionKey			= 0xF704,
	kIzF2FunctionKey			= 0xF705,
	kIzF3FunctionKey			= 0xF706,
	kIzF4FunctionKey			= 0xF707,
	kIzF5FunctionKey			= 0xF708,
	kIzF6FunctionKey			= 0xF709,
	kIzF7FunctionKey			= 0xF70A,
	kIzF8FunctionKey			= 0xF70B,
	kIzF9FunctionKey			= 0xF70C,
	kIzF10FunctionKey			= 0xF70D,
	kIzF11FunctionKey			= 0xF70E,
	kIzF12FunctionKey			= 0xF70F,
	kIzF13FunctionKey			= 0xF710,
	kIzF14FunctionKey			= 0xF711,
	kIzF15FunctionKey			= 0xF712,
	kIzF16FunctionKey			= 0xF713,
	kIzF17FunctionKey			= 0xF714,
	kIzF18FunctionKey			= 0xF715,
	kIzF19FunctionKey			= 0xF716,
	kIzF20FunctionKey			= 0xF717,
	kIzF21FunctionKey			= 0xF718,
	kIzF22FunctionKey			= 0xF719,
	kIzF23FunctionKey			= 0xF71A,
	kIzF24FunctionKey			= 0xF71B,
	kIzF25FunctionKey			= 0xF71C,
	kIzF26FunctionKey			= 0xF71D,
	kIzF27FunctionKey			= 0xF71E,
	kIzF28FunctionKey			= 0xF71F,
	kIzF29FunctionKey			= 0xF720,
	kIzF30FunctionKey			= 0xF721,
	kIzF31FunctionKey			= 0xF722,
	kIzF32FunctionKey			= 0xF723,
	kIzF33FunctionKey			= 0xF724,
	kIzF34FunctionKey			= 0xF725,
	kIzF35FunctionKey			= 0xF726,
	kIzInsertFunctionKey		= 0xF727,
	kIzDeleteFunctionKey		= 0xF728,
	kIzHomeFunctionKey			= 0xF729,
	kIzBeginFunctionKey			= 0xF72A,
	kIzEndFunctionKey			= 0xF72B,
	kIzPageUpFunctionKey		= 0xF72C,
	kIzPageDownFunctionKey		= 0xF72D,
	kIzPrintScreenFunctionKey	= 0xF72E,
	kIzScrollLockFunctionKey	= 0xF72F,
	kIzPauseFunctionKey			= 0xF730,
	kIzSysReqFunctionKey		= 0xF731,
	kIzBreakFunctionKey			= 0xF732,
	kIzResetFunctionKey			= 0xF733,
	kIzStopFunctionKey			= 0xF734,
	kIzMenuFunctionKey			= 0xF735,
	kIzUserFunctionKey			= 0xF736,
	kIzSystemFunctionKey		= 0xF737,
	kIzPrintFunctionKey			= 0xF738,
	kIzClearLineFunctionKey		= 0xF739,
	kIzClearDisplayFunctionKey	= 0xF73A,
	kIzInsertLineFunctionKey	= 0xF73B,
	kIzDeleteLineFunctionKey	= 0xF73C,
	kIzInsertCharFunctionKey	= 0xF73D,
	kIzDeleteCharFunctionKey	= 0xF73E,
	kIzPrevFunctionKey			= 0xF73F,
	kIzNextFunctionKey			= 0xF740,
	kIzSelectFunctionKey		= 0xF741,
	kIzExecuteFunctionKey		= 0xF742,
	kIzUndoFunctionKey			= 0xF743,
	kIzRedoFunctionKey			= 0xF744,
	kIzFindFunctionKey			= 0xF745,
	kIzHelpFunctionKey			= 0xF746,
	kIzModeSwitchFunctionKey	= 0xF747,
	
	kIzEnterFunctionKey			= 0xF780		// on MacOS, fn "Return" or "Enter" key on extended keyboard
												// on Windows, "Enter" key on numeric keypad
};

#if TARGET_OS_WIN

enum
{
	kVK_LeftArrow = VK_LEFT,
	kVK_RightArrow = VK_RIGHT,
	kVK_DownArrow = VK_DOWN,
	kVK_UpArrow = VK_UP,
	kVK_Home = VK_HOME,
	kVK_End = VK_END,
	kVK_PageUp = VK_PRIOR,
	kVK_PageDown = VK_NEXT,
	kVK_ForwardDelete = VK_DELETE,
	kVK_Escape = VK_ESCAPE,
	kVK_Delete = VK_BACK
};

#endif


// MOUSE TRACKING CONSTANTS

// pass these as options to IzTrackMouseLocationWithOptions
typedef uint32_t IzMouseTrackingOptions;

enum {
	// indicates that mouse-up events should be left in the queue.
	kIzTrackMouseLocationOptionDontConsumeMouseUp = (1 << 0),

	// indicates that kEventMouseWheelMoved and kEventMouseScroll events
	// should cause the tracking API to stop tracking and return
	// kIzMouseTrackingScrollWheel.
	kIzTrackMouseLocationOptionIncludeScrollWheel = (1 << 3)
};

typedef uint32_t IzMouseTrackingResult;

enum {

	// returned when the user presses any mouse button.
	kIzMouseTrackingMouseDown = 1,

	// returned when the user releases a mouse button.
	kIzMouseTrackingMouseUp = 2,

	// returned when the user moves the mouse out of the specified region.
	kIzMouseTrackingMouseExited = 3,

	// returned when the user moves the mouse into the specified region.
	kIzMouseTrackingMouseEntered = 4,

	// returned when the user moves the mouse while holding down a mouse button.
	kIzMouseTrackingMouseDragged = 5,

	// returned when the user changes the keyboard modifiers.
	kIzMouseTrackingKeyModifiersChanged = 6,

	// returned if the user presses the escape or cmd-period keys during
	// mouse tracking.
	kIzMouseTrackingUserCancelled = 7,

	// returned if the timeout expires.
	kIzMouseTrackingTimedOut = 8,

	// returned when the user moves the mouse while no mouse button is pressed.
	kIzMouseTrackingMouseMoved = 9,

	// eturned when a caller-specified event was received during mouse tracking.
	kIzMouseTrackingClientEvent = 10,

	// returned when the user scrolls a mouse wheel. This value is only
	// returned when the kTrackMouseLocationOptionIncludeScrollWheel option
	// bit is specified in a call to IzTrackMouseLocationWithOptions
	kIzMouseTrackingScrollWheel = 11,
	
	// returned when a tablet point event is received
	kIzMouseTrackingTabletPoint = 12,
	
	// returned when a tablet proximity event is received
	kIzMouseTrackingTabletProximity = 13
};

enum {
	kIzMacActiveFlagBit			= 0,	// activate
	kIzMacBtnStateBit			= 7,	// state of button
	kIzMacCmdKeyBit				= 8,	// command key down
	kIzMacShiftKeyBit			= 9,	// shift key down
	kIzMacAlphaLockBit			= 10,	// alpha lock down
	kIzMacOptionKeyBit			= 11,	// option key down
	kIzMacControlKeyBit			= 12	// control key down
};

enum {
	kIzMacActiveFlag			= 1 << kIzMacActiveFlagBit,
	kIzMacBtnState				= 1 << kIzMacBtnStateBit,
	kIzMacCmdKey				= 1 << kIzMacCmdKeyBit,
	kIzMacShiftKey				= 1 << kIzMacShiftKeyBit,
	kIzMacAlphaLock				= 1 << kIzMacAlphaLockBit,
	kIzMacOptionKey				= 1 << kIzMacOptionKeyBit,
	kIzMacControlKey			= 1 << kIzMacControlKeyBit,
};

typedef uint16_t IzEventMouseWheelAxis;
enum {
	kIzEventMouseWheelAxisX         = 0,		// The X axis (left or right)
	kIzEventMouseWheelAxisY         = 1			// The Y axis (up or down)
};

#if PRAGMA_STRUCT_ALIGN
    #pragma options align=mac68k
#elif PRAGMA_STRUCT_PACKPUSH
    #pragma pack(push, 2)
#elif PRAGMA_STRUCT_PACK
    #pragma pack(2)
#endif

#ifdef __EVENTS__

	typedef EventRecord		IzMacEventRecord;
	typedef EventKind		IzMacEventKind;
	typedef EventMask		IzMacEventMask;
	typedef EventModifiers	IzMacEventModifiers;

#else

	typedef UInt16		IzMacEventKind;
	typedef UInt16		IzMacEventMask;
	typedef UInt16		IzMacEventModifiers;

	struct IzMacEventRecord {
		IzMacEventKind		what;
		uint32_t			message;
		uint32_t			when;
		IzPoint				where;
		IzMacEventModifiers	modifiers;
	};
	typedef struct IzMacEventRecord IzMacEventRecord;

#endif

// TABLET BUTTON MASKS (IZButtonMask)
enum {
	kIzPenTipMask				= 0x01,
	kIzPenLowerSideMask			= 0x02,
	kIzPenUpperSideMask			= 0x04
};
typedef uint16_t IZButtonMask;

// POINTER DEVICE TYPE (IZPointerType)
enum {
    IZUnknownPointingDevice     = 0,
    IZPenPointingDevice         = 1,
    IZCursorPointingDevice      = 2,
    IZEraserPointingDevice      = 3
};
typedef uint8_t IZPointerType;

// TABLET CAPABILITIES FLAGS (IZTabletCapability)
enum {
	kTableCapabilityMask_DEVICEIDMASK			= 0x0001,
	kTableCapabilityMask_ABSXMASK				= 0x0002,
	kTableCapabilityMask_ABSYMASK				= 0x0004,
	kTableCapabilityMask_VENDOR1MASK			= 0x0008,
	kTableCapabilityMask_VENDOR2MASK			= 0x0010,
	kTableCapabilityMask_VENDOR3MASK			= 0x0020,
	kTableCapabilityMask_BUTTONSMASK			= 0x0040,
	kTableCapabilityMask_TILTXMASK				= 0x0080,
	kTableCapabilityMask_TILTYMASK				= 0x0100,
	kTableCapabilityMask_ABSZMASK				= 0x0200,
	kTableCapabilityMask_PRESSUREMASK			= 0x0400,
	kTableCapabilityMask_TANGENTIALPRESSUREMASK	= 0x0800,
	kTableCapabilityMask_ORIENTINFOMASK			= 0x1000,
	kTableCapabilityMask_ROTATIONMASK			= 0x2000
};
typedef uint16_t IZTabletCapability;

struct IzTabletPointInfo {
	int32_t			mAbsX;					// absolute x coordinate in tablet space at full tablet resolution
	int32_t			mAbsY;					// absolute y coordinate in tablet space at full tablet resolution
	int32_t			mAbsZ;					// absolute z coordinate in tablet space at full tablet resolution
	IZButtonMask	mButtons;				// one bit per button - bit 0 is first button - 1 = closed
	float			mPressure;				// pressure from 0.0 to 1.0
	float			mTiltX;					// tilt X from -1.0 to +1.0
	float			mTiltY;					// tilt Y from -1.0 to +1.0
	float			mRotation;				// device rotation in degrees
	float			mTangentialPressure;	// from -1.0 to +1.0
	uint16_t		mDeviceID;				// system-assigned unique device ID - matches to deviceID field in proximity event
	int16_t			mVendor1;				// vendor-defined signed 16-bit integer
	int16_t			mVendor2;				// vendor-defined signed 16-bit integer
	int16_t			mVendor3;				// vendor-defined signed 16-bit integer
};
typedef struct IzTabletPointInfo           IzTabletPointInfo;

struct IzTabletProximityInfo {
	uint16_t			mVendorID;				// vendor-defined ID - typically will be USB vendor ID
	uint16_t			mTabletID;				// vendor-defined tablet ID - typically will be USB product ID for the tablet
	uint16_t			mPointerID;				// vendor-defined ID of the specific pointing device
	uint16_t			mDeviceID;				// system-assigned unique device ID - matches to deviceID field in tablet event
	uint16_t			mSystemTabletID;		// system-assigned unique tablet ID
	uint16_t			mVendorPointerType;		// vendor-defined pointer type
	uint32_t			mPointerSerialNumber;	// vendor-defined serial number of the specific pointing device
	uint64_t			mUniqueID;				// vendor-defined unique ID for this pointer
	IZTabletCapability	mCapabilityMask;		// mask representing the capabilities of the device.
	IZPointerType		mPointerType;			// type of pointing device - enum to be defined
	uint8_t				mEnterProximity;		// non-zero = entering; zero = leaving
};
typedef struct IzTabletProximityInfo       IzTabletProximityInfo;

#if PRAGMA_STRUCT_ALIGN
	#pragma options align=reset
#elif PRAGMA_STRUCT_PACKPUSH
	#pragma pack(pop)
#elif PRAGMA_STRUCT_PACK
	#pragma pack()
#endif

// ---------------------------------------------------------------------------
//	CIzEvent CLASS

#if TARGET_OS_WIN

class CWinKeyInfo
{
public:
	CWinKeyInfo(
		bool	inKeyDown = false,
		WPARAM	inKeyCode = 0,
		LPARAM	inFlags = 0) :
			mKeyDown(inKeyDown),
			mKeyCode(inKeyCode),
			mFlags(inFlags)
	{

	}

	bool	mKeyDown;
	WPARAM	mKeyCode;
	LPARAM	mFlags;
};

const OSType kWinKeyInfoPropertyID = FOUR_CHAR_CODE('KEYI');

#endif

// ---------------------------------------------------------------------------
//	CIzEvent CLASS

class CIzEvent
{
public:
						// default constructor creates a valid null
						// event with the current global mouse
						// position, event time, and modifiers
						CIzEvent();
	
						// default constructor or other types of event
						CIzEvent(
							IzEvent				inEvent,
							IzModifiers			inModifiers = kIzModifier_None);
						
						// mouse event: kIzEvent_MouseXX
						CIzEvent(
							IzEvent				inEvent,
							IzModifiers			inModifiers,
							NativeWindow		inWindow,
							IzPoint				inGlobalPoint);
			
						// draw event: kIzEvent_Draw
						CIzEvent(
							NativeView			inView,
							IzEvent				inEvent,
							IzModifiers			inModifiers,
							NativeContextRef	inContext,
							IzRect				inRect);
	
	// MAC SPECIFIC FUNCTIONS (in IZEvent.mm)
	#if TARGET_OS_MAC && defined(_OBJC_OBJC_H_)
						CIzEvent(
							NSView*		inView,
							NSEvent*	inNSEvent);
	#endif

	#if TARGET_OS_WIN
	static bool			CanCreateIzEvent(
							HWND		inHWND,
							UINT		inMessage,
							WPARAM		inWParam,
							LPARAM		inLParam);

						CIzEvent(
							HWND		inHWND,
							UINT		inMessage,
							WPARAM		inWParam,
							LPARAM		inLParam);

						CIzEvent(
							MSG			inMsg);
	#endif
	
	// WINDOWS SPECIFIC FUNCTIONS (in IZEvent.cp)

	void				SetNativeView(
							NativeView			inNativeView);
	
	IzEvent				GetEventType() const					{ return mEvent; }
	
	bool				IsKeyboardEvent() const
						{
							#if TARGET_OS_MAC
								return (mEvent == kIzEvent_KeyDown || mEvent == kIzEvent_KeyUp);
							#elif TARGET_OS_WIN
								return (mEvent == kIzEvent_KeyDown || mEvent == kIzEvent_KeyUp || mEvent == kIzEvent_SysKeyDown || mEvent == kIzEvent_SysKeyUp);
							#endif
						}

	bool				IsKeyDownEvent() const
						{
							return (mEvent == kIzEvent_KeyDown);
						}

	bool				IsKeyUpEvent() const
						{
							#if TARGET_OS_MAC
								return (mEvent == kIzEvent_KeyUp);
							#elif TARGET_OS_WIN
								return (mEvent == kIzEvent_KeyUp ||  mEvent == kIzEvent_SysKeyUp);
							#endif
						}

	bool				IsStandardKeyboardEvent() const
						{
							return (mEvent == kIzEvent_KeyDown || mEvent == kIzEvent_KeyUp);
						}

	bool				IsSystemKeyboardEvent() const
						{
							#if TARGET_OS_MAC
								return (mEvent == kIzEvent_KeyDown || mEvent == kIzEvent_KeyUp);
							#else
								return (mEvent == kIzEvent_SysKeyDown || mEvent == kIzEvent_SysKeyUp);
							#endif
						}

	// for keypress events, returns the standardized IzKeyCode for the event. 
	IzKeyCode			GetKeypressKeycode() const				{ return mKeyCode; }
	
	// for keypress events, returns the unicode representation of the current keystroke.
	IzUniChar			GetKeypressCharacter() const			{ return mKeypressChar; }
	
	// for keypress events, returns the unicode representation of the current keystroke
	// as a UTF8 string
	std::string			GetKeypressString() const				{ return mKeypressString; }
	
	IzPoint				GetMousePoint() const		{ return mWindowPoint; }
	
	// 3.0.3b9: hardcode control-left-click as equivalent to right click on macOS
	#if TARGET_OS_MAC
	bool				IsMouseRightClick() const
							{ return (mEvent == kIzEvent_RightMouseDown) || (mEvent == kIzEvent_LeftMouseDown && mModifiers == kIzModifier_Control); }
	#elif TARGET_OS_WIN
	bool				IsMouseRightClick() const
							{ return mEvent == kIzEvent_RightMouseDown; }
	#endif
	
	IzMacEventModifiers	GetMacEventModifiers() const;
	
	IzModifiers			GetModifiers() const		{ return mModifiers; }
	
	bool				IsShiftKeyDown() const		{ return ((mModifiers & kIzModifier_Shift) != 0); }
	bool				IsCommandKeyDown() const	{ return ((mModifiers & kIzModifier_Command) != 0); }
	bool				IsControlKeyDown() const	{ return ((mModifiers & kIzModifier_Control) != 0); }
	bool				IsAltKeyDown() const		{ return ((mModifiers & kIzModifier_Alt) != 0); }
	
	bool				IsTabletEvent() const		{ return mIsTabletEvent; }
#if DELETE_ME
	EventRecord			PopuplateOldEventRecord() const;
#endif
	
	IzEvent					mEvent;					// event type
	IzSeconds				mWhen;					// when event happened, in seconds
	IzQTTicks				mWhenTicks;				// when in 1/60ths of a second since the program started
	IzModifiers				mModifiers;				// modifier keys at time of event
	bool					mIsTabletEvent;			// true if this is a WM_POINTERXXX message masquerading as a WM_MOUSEXXX event
	
	// MOUSE EVENTS
	NativeWindow			mWindow;				// mouse events: top level window that received mouse event
	NativeView				mView;					// mouse events: view that received mouse event
	IzPoint					mGlobalPoint;			// mouse events: point within screens
	IzPoint					mWindowPoint;			// mouse events: point within window
	
	// SCROLL WHEEL
	bool					mHighPrecisionScroll;	// only valid if mEvent = kIzEvent_ScrollWheel
	float					mScrollDeltaX;			// only valid if mEvent = kIzEvent_ScrollWheel
	float					mScrollDeltaY;			// only valid if mEvent = kIzEvent_ScrollWheel
	
	// TABLET EVENT
	uint32_t				mTabletDeviceID;		// valid for kIzEvent_TabletPoint or kIzEvent_TabletProximity
	float					mPressure;				// valid for mouse events, kIzEvent_TabletPoint or kIzEvent_TabletProximity
	bool					mTabletPointValid;		// true if mTabletPointInfo is valid
	bool					mTabletProximityValid;	// true mTabletProximityInfo is valid
	IzTabletPointInfo		mTabletPointInfo;		// valid only for kIzEvent_TabletPoint
	IzTabletProximityInfo	mTabletProximityInfo;	// valid only for kIzEvent_TabletProximity
	
	// KEYPRESS EVENTS
	IzKeyCode				mKeyCode;				// platform depdendant key code
	
	IzUniChar				mKeypressChar;			// unicode keyboard event character. note that
													// this will also contain unicode representations
													// of special characters like the arrow keys, etc.
													// see the kIz...FunctionKey constants defined
													// above for a complete list
	
	std::string				mKeypressString;		// unicode string representation of mKeypressChar.
													// in some situations a single kepress can generate
													// more than one character; usually this would be a
													// failed diacritical (i.e., umlaut followed by a
													// char) but there may be others. generally one
													// should only pay attention to the mKeypressChar
	
	// DRAWING EVENTS
	NativeContextRef		mContext;				// CGContextRef on MacOS, HDC on Windows
	IzRect					mDrawRect;				// only valid if mEvent == kIzEvent_Draw
	
#if TARGET_OS_MAC
	EventRecord				mEventRecord;
#elif TARGET_OS_WIN
	HWND					mHWND;
	UINT					mHWNDMessage;
	WPARAM					mWParam;
	LPARAM					mLParam;
#endif

private:
	operator void*()
	{
		return this;
	}
};

// a return value of 'true' indicatest the event was handled

typedef bool (*IzEventHandler)(const CIzEvent* inEvent, NativeView inSender, void* inUserData);

const OSType kIzViewEventHandlerInfo = FOUR_CHAR_CODE('EHDI');

class EventHandlerInfo
{
public:
	EventHandlerInfo(
		IzEventHandler	inEventHandler = NULL,
		void*			inUserData = NULL) :
		mEventHandler(inEventHandler),
		mUserData(inUserData)
			{	}

	IzEventHandler	mEventHandler;
	void*			mUserData;
};

// GENERAL EVENT INFORMATION

double			IZ_EventSecondsSinceProgramStartup();

IzPoint			IZ_EventGetGlobalMouse();

IzModifiers		IZ_EventGetCurrentModifiers();

std::string		IZ_EventModifiersToString(
					IzModifiers			inModifiers);

// KEYCODE CONVERSION

// attempts to converts a platform specific keycode into
// an ** uppercase ** unicode character. if the function
// succeeds, the result is non-zero. set the inValidASCIIOnly
// param to limit valid results to a range of 0x20 to 0x7F.

IzUniChar		IZ_EventKeyCodeToUppercaseChar(
					IzKeyCode			inKeycode,
					bool				inValidASCIIOnly = false);

// convenience function returns a UTF8 string representation
// of the result of IZ_EventKeyCodeToUppercaseString. if the conversion
// fails, the string will be empty.

std::string		IZ_EventKeyCodeToUppercaseString(
					IzKeyCode			inKeycode,
					bool				inValidASCIIOnly = false);


// MOUSE EVENTS
CIzEvent		IZ_EventCreateMacOSNullEvent();			// Mac+Win common implementation

bool			IZ_EventMouseStillDown();

bool			IZ_EventGetMouseUp(
					CIzEvent*				outEvent);

bool			IZ_EventGetMouseUp(
					CIzEvent*			outEvent);	// Mac+Win common implementation

// MOUSE TRACKING

#define kIzMouseTracking_DontTimeout		DBL_MAX

OSStatus		IzTrackMouseLocationWithOptions(
					NativeWindow			inWindow,
					IzMouseTrackingOptions	inOptions,
					double					inTimeoutSeconds,		// use negative number or DBL_MAX to indicate no timeout
					IzPoint*				outPt,
					uint32_t*				outModifiers,			// CAN BE NULL
					IzMouseTrackingResult*	outResult,
					CIzEvent*				outEvent = NULL);		// CAN BE NULL

// EVENT PROCESSING

// process events unti the specified IzCondition becomes true or
// the specified timeout period elapses. if inOnlyForModalWindow
// is non NULL, only events for that window are processed; others
// are discarded

void			IZ_EventProcessEvents(
					const IzCondition&		inUntilCondition = IzCondition(),
					IzMilliseconds			inTimeout_mS = kIzMilliseconds_TimeoutInfinite,
					NativeWindow			inOnlyForModalWindow = NULL);

// idles the system without actually processing any events
void			IZ_EventIdleSystem(
					IzMilliseconds		inTimeout_mS);

// TABLET SUPPORT

void			IZ_EventSetTabletProximityInfo(
					const IzTabletProximityInfo&	inTabletProximityInfo);
							
bool			IZ_EventGetTabletProximityInfo(
					IzTabletProximityInfo*			outTabletProximityInfo);
							

// ---------------------------------------------------------------------------

#if TARGET_OS_MAC && defined(_OBJC_OBJC_H_)

	IzModifiers	NSModifierMaskToIzModifiers(NSUInteger inMask);
	NSUInteger	IzModifiersToNSModifierMask(IzModifiers	inMask);

#endif

// ---------------------------------------------------------------------------

#if TARGET_OS_WIN && DEBUG

#include <string>

void			IZ_DebugPrintWindowsKeyboardMessageInfo(
					MSG				msg,
					std::string		inPrefix = std::string());

#endif

// ---------------------------------------------------------------------------

// ENTER KEY HANDLING
//
// Oh this Enter key thing is a mess. We'd like to use Enter to
// close dialog boxes where there is a text field that needs the
// normal enter key (return char) as a linebreak. But this broke
// on windows when you choose Ctrl-C for copy and nothing was
// selected, because when Ctrl-C isn't seen as an accelerator, it
// is passed on to as a normal character event, with a char code
// of 0x03 which is what macOS calls "Enter" (as opposed to "Return")
//
// macOS standard:
//
// pressing fn+Return on the main keyboard or the Enter key on the
// extended keyboard produces a keycode of kVK_ANSI_KeypadEnter and
// a character code of 0x03.
//
// Windows standard:
//
// Windows normally makes no distinction between the Enter key on the
// main keyboard and the Enter key on the numeric keypad, returning a
// virtual key code of VK_ENTER for both. However, we can discern if
// it was the numeric keypad Enter key by looking at the extended keyboard
// flag found in the lParam of a WM_KEYDOWN or WM_KEYUP message.
//
// CIzEvent
//
// In our system, pressing the Enter key on either platform will produce
// a CIzEvent will contain a keycode of VK_ENTER and a character code of
// kIzEnterFunctionKey, which is defined in IzSystem.h.

#if TARGET_OS_MAC

// use the standard constant defined in HIToolbox.h
#define VK_ENTER		kVK_ANSI_KeypadEnter

#elif TARGET_OS_WIN

// winuser.h indicates that virtual keys 0x88 - 0x8F are
// unassigned. so we'll steal one of those here to indicate
// that the keypad enter key was pressed. (Hopefully this is
// safe since the virtual key list seems pretty much etched
// in stone at this late date. Fingers crossed I'm not making
// a stupid assumption.)

#define VK_ENTER		0x8F

#endif

#endif	// #ifdef __cplusplus

#endif	// #ifndef _H_IZEvent