// =================================================================================
//	IZQTResources					(c) 2016 Mark F. Coniglio -- All Rights Reserved
// =================================================================================
// Replacement for QuickTime that deal with resource files

#ifndef _H_IZQTResources
#define _H_IZQTResources

#include "IZDefinitions.h"
#ifdef __ISADORA_APP__
#include "IZMenus.h"
#endif
#include "IZFile.h"

// --------------------------------------------------------------------------------
// ENABLE_QT_MEMORY_FUNCTIONS: this flag determines if
// the alternate QuickTime memory functions will be used

#if TARGET_OS_MAC

	#define		ENABLE_QT_RESOURCE_FUNCTIONS		TRUE

#elif TARGET_OS_WIN

	#define		ENABLE_QT_RESOURCE_FUNCTIONS		!IZ_ALLOW_QUICKTIME

#endif

// --------------------------------------------------------------------------------
// FORCE_QT_RESOURCE_FUNCTIONS: set this flag to true to force the
// alternate QuickTime memory functions to be used, overriding the
// setting of ENABLE_QT_RESOURCE_FUNCTIONS

#define		FORCE_QT_RESOURCE_FUNCTIONS				FALSE

// --------------------------------------------------------------------------------
//
// IZQT RESOURCE FUNCTIONS
//
// --------------------------------------------------------------------------------

#if FORCE_QT_RESOURCE_FUNCTIONS || ENABLE_QT_RESOURCE_FUNCTIONS

#define RESOURCE_FUNCTIONS_USE_QTML		FALSE

#if PRAGMA_STRUCT_PACKPUSH
#  pragma pack(push, 2)
#else
#  pragma options align=mac68k
#endif

// RESOURCE FLIPPING NOTE: From Letters from the Icefloe - Dispatch 25

// As of QuickTime 4.0.3, resource flippers are built-in for the following resource types:
// ALRT, BNDL, cdci, cicn, clut, CNTL, cpix, crsr, CURS, DITL, DLOG, FREF, fttl, gnht, icl4,
// icl8, ICN#, ICON, ics4, ics8, MBAR, MENA, MENU, nini, pnot, qfmt, qmtr, qtet, snd , stg#
// stgp, STR , STR#, styl, TEXT, thar, thga, thnd, thng, thnr, vers, WIND, and wxim.

typedef struct {

   unsigned int stage1_SoundNumber : 2;
   unsigned int stage1_Visible: 1;
   unsigned int stage1_BoldItem: 1;
	
   unsigned int stage2_SoundNumber : 2;
   unsigned int stage2_Visible: 1;
   unsigned int stage2_BoldItem: 1;

   unsigned int stage3_SoundNumber : 2;
   unsigned int stage3_Visible: 1;
   unsigned int stage3_BoldItem: 1;

   unsigned int stage4_SoundNumber : 2;
   unsigned int stage4_Visible: 1;
   unsigned int stage4_BoldItem: 1;
	
} QT_ALRT_StageInfo;

typedef struct {
	QDRect				bounds;
	uint16_t			ditl;			// ID of the DITL resource which contains the
	QT_ALRT_StageInfo	stageInfo;		// ????
	uint16_t			placement;		// placement code
} QT_ALRT, **QT_ALRT_Handle;

typedef struct {
	QDRect				bounds;
	uint16_t			value;			// scrollbar value
	uint8_t				visible;		// 0 = not visible
	uint8_t				unused;
	uint16_t			max_value;		// scrollbar max
	uint16_t			min_value;		// scrollbar min
	uint16_t			cdef;			// control type, see below
	uint32_t			refCon;			// reference constant
	uint8_t				title[1];		// variable sized pascal-string title
} QT_CNTL, **QT_CNTL_Handle;

#define CDEF_button    0x0000
#define CDEF_checkbox  0x0001
#define CDEF_radio     0x0002
#define CDEF_scrollbar 0x0010
#define CDEF_custom    0x0800	// square button, used for exits and commands

typedef struct {
    uint32_t			reserved;
	QDRect				bounds;
    uint8_t				type;			// type of control, see below (only 7 bits used).
	uint8_t				title[1];		// variable sized pascal-string title
} QT_DITLItem;

typedef struct {
    uint16_t	numItems;		// num items minus 1
    QT_DITLItem	items[1];
} QT_DITL, **QT_DITL_Handle;

#define DITL_button   0x04
#define DITL_label    0x08
#define DITL_textedit 0x10

typedef struct {
	QDRect				bounds;
	uint16_t			type;			// type of dialog, see WIND
	uint8_t				visible;		// 0 = invisible
	uint8_t				unused1;		// unused byte
	uint8_t				closeBox;		// 0 = no close box
	uint8_t				unused2;		// unused byte
	uint32_t			refCon;			// Reference Constant
	uint16_t			ditl;			// ID of the DITL resource for this dialog
	uint8_t				title[1];		// variable sized pascal-string title
} QT_DLOG, **QT_DLOG_Handle;

typedef struct {
	uint8_t		title[1];		// variable sized pascal-string title
} IZ_MENUItem;

// follows after even-aligned name of IZ_MENUItem
typedef struct {
    uint8_t icon;			// icon ID for menu item
    uint8_t key;			// keyboard shortcut
    uint8_t mark;			// checkmark for item
    uint8_t style;			// menu item style
} IZ_MENUItemExtra;

typedef struct {

	uint16_t	menuID;			// id of the menu
	uint16_t	width;			// used in memory
	uint16_t	height;			// used in memory
	uint16_t	resourceID;
	uint16_t	placeholder;
	uint32_t	enabled;		// bitmask for enabling menu items
	uint8_t		title_length;
	char		title[1];		// title of menu
	
	// IZ_MENUItem
	// zero or more QT_MenuItem's follow title
	
} IZ_MENU;

struct IZTabInfo {
	SInt16		iconSuiteID;
	Str255		tabName;			// This is descriptive, not accurate. String is not padded to 255 characters.
	SInt32		reserved1;
	SInt16		reserved2;
};

struct IZTabListRec {
	SInt16		version;
	SInt16		numTabs;
	IZTabInfo	tabs[1];
};

struct QDBitMap {
  Ptr			baseAddr;		// 32-bit read
  short			rowBytes;
  QDRect		bounds;
};
typedef struct QDBitMap			QDBitMap;

struct QDColorSpec {
  short               value;                  /*index or other value*/
  RGBColor            rgb;                    /*true color*/
};
typedef struct QDColorSpec			QDColorSpec;
typedef QDColorSpec*				QDColorSpecPtr;
typedef QDColorSpec					QDCSpecArray[1];

struct QDColorTable {
  SInt32		ctSeed;                 /*unique identifier for table*/
  SInt16		ctFlags;                /*high bit: 0 = PixMap; 1 = device*/
  SInt16		ctSize;                 /*number of entries in CTTable*/
  QDCSpecArray	ctTable;                /*array [0..0] of ColorSpec*/
};
typedef struct QDColorTable		QDColorTable;
typedef QDColorTable*			QDCTabPtr;
typedef QDCTabPtr*				QDCTabHandle;

struct QDPixMap {
  Ptr			baseAddr;               /* pointer to pixels */				// 32-bit read
  SInt16		rowBytes;               /* offset to next line */
  QDRect		bounds;                 /* encloses bitmap */
  short			pmVersion;              /* pixMap version number */
  short			packType;               /* defines packing format */
  SInt32		packSize;               /* length of pixel data */
  Fixed			hRes;                   /* horiz. resolution (ppi) */
  Fixed			vRes;                   /* vert. resolution (ppi) */
  SInt16		pixelType;              /* defines pixel type */
  SInt16		pixelSize;              /* # bits in pixel */
  SInt16		cmpCount;               /* # components in pixel */
  SInt16		cmpSize;                /* # bits per component */
  OSType		pixelFormat;            /* fourCharCode representation */
  QDCTabHandle	pmTable;                /* color map for this pixMap */		// 32-bit read
  void*			pmExt;                  /* Handle to pixMap extension */	// 32-bit read
};
typedef struct QDPixMap			QDPixMap;
typedef QDPixMap*				QDPixMapPtr;
typedef QDPixMapPtr*			QDPixMapHandle;

struct QDCIcon {
  QDPixMap		iconPMap;               /*the icon's pixMap*/
  QDBitMap		iconMask;               /*the icon's mask*/
  QDBitMap		iconBMap;               /*the icon's bitMap*/
  Handle		iconData;               /*the icon's data*/					// 32-bit read
  SInt16		iconMaskData[1];        /*icon's mask and BitMap data*/
};
typedef struct QDCIcon		QDCIcon;
typedef QDCIcon*			QDCIconPtr;
typedef QDCIconPtr*			QDCIconHandle;

#if PRAGMA_STRUCT_PACKPUSH
#  pragma pack(pop)
#else
#  pragma options align=reset
#endif

OSErr 
QTResError(void);

void
QTResEnableVerboseErrors(
	bool				inVerboseErrors);

ResFileRefNum
QTOpenResFile(
	const char*			inFullPathToFile,	// UTF-8 encoded, null terminated c-string
	SInt8				permsssion);

ResFileRefNum
QTFSpOpenResFile(
	const IzSpec*		spec,
	SignedByte			permission);

#if ALLOW_FS_REF
ResFileRefNum
QTFSOpenResourceFile(
	const FSRef*		ref,
	UniCharCount		forkNameLength,		// CAN BE ZERO
	const UniChar*		forkName,			// CAN BE NULL
	SInt8				permissions,
	ResFileRefNum*		refNum);
#endif

void
QTCloseResFile(
	ResFileRefNum		refNum);

ResFileRefNum 
QTCurResFile(void);

void 
QTUseResFile(
	ResFileRefNum		refNum);

SInt16 
QTCountResources(
	ResType				theType);

SInt16 
QTCount1Resources(
	ResType				theType);

Handle 
QTGetIndResource(
	ResType				theType,
	SInt16				itemIndex);

Handle 
QTGet1IndResource(
	ResType				theType,
	SInt16				itemIndex);

Handle 
QTGetResource(
	ResType				theType,
	ResID				theID);

Handle 
QTGet1Resource(
	ResType				theType,
	ResID				theID);

Handle 
QTGetNamedResource(
	ResType				theType,
	ConstStr255Param	name);

Handle 
QTGet1NamedResource(
	ResType				theType,
	ConstStr255Param	name);

void 
QTReleaseResource(
	Handle				theResource);

void 
QTDetachResource(
	Handle				theResource);

ResAttributes
QTGetResAttrs(
	Handle				theResource);

void 
QTGetResInfo(
	Handle				theResource,
	ResID*				theID,
	ResType*			theType,
	Str255				name);

StringHandle
QTGetString(
	short				stringID);

void
QTGetIndString(
	Str255				theString,
	short				strListID,
	short				itemIndex);

QDCIconHandle
QTGetCIcon(
	SInt16				iconID);

QDCIconHandle
QTGet1CIcon(
	SInt16				iconID);

void
QTDisposeCIcon(
	QDCIconHandle		theIcon);

PicHandle 
QTGetPicture(
	SInt16				pictureID);

PicHandle 
QTGet1Picture(
	SInt16				pictureID);

NativeMenu
QTGetMenu(
	SInt16				inMenuResID);

EXTERN_API_C( OSErr )	
QTRegisterResourceEndianFilter(
	ResType					theType,
	ResourceEndianFilterPtr	theFilterProc);

QT_ALRT_Handle
QT_Alert_GetAlert(
	int16_t				inAlertID);

QT_DITL_Handle
QT_Alert_GetDITL(
	QT_ALRT_Handle		inAlertHandle);

QT_DITLItem*
QT_DITL_CreateIndexedDialogItem(
	QT_DITL_Handle		inDITLHandle,
	size_t				inIndex0);


#else	// of #if FORCE_QT_RESOURCE_FUNCTIONS || ENABLE_QT_RESOURCE_FUNCTIONS

#define RESOURCE_FUNCTIONS_USE_QTML		TRUE

#define	QTResError				ResError
#define	QTFSpOpenResFile		FSpOpenResFile
#define	QTHOpenResFile			HOpenResFile
#define	QTFSOpenResourceFile	FSOpenResourceFile
#define QTCloseResFile			CloseResFile
#define QTCurResFile			CurResFile
#define QTUseResFile			UseResFile
#define QTCountResources		CountResources
#define QTCount1Resources		Count1Resources
#define QTGetIndResource		GetIndResource
#define QTGet1IndResource		Get1IndResource
#define QTGetResource			GetResource
#define QTGet1Resource			Get1Resource
#define QTGetNamedResource		GetNamedResource
#define QTGet1NamedResource		Get1NamedResource
#define QTReleaseResource		ReleaseResource
#define QTDetachResource		DetachResource
#define QTGetResAttrs			GetResAttrs
#define QTGetResInfo			GetResInfo
#define	QTGetString				GetString
#define QTGetIndString			GetIndString

// resource related functions...
#define	QTGetIcon				GetIcon
#define QTGetCIcon				GetCIcon
#define QTDisposeCIcon			DisposeCIcon
#define QTGetPicture			GetPicture
#define	QTGetMenu				MacGetMenu

#define	 QTRegisterResourceEndianFilter	RegisterResourceEndianFilter

#endif

#ifdef __cplusplus
std::string
QTGetIndCString(
	short				strListID,
	short				itemIndex);
#endif

#endif	// #ifndef _H_IZQTResources
