// ===========================================================================
//	IsadoraTypes				(c) 1999 Mark F. Coniglio. All rights reserved
// ===========================================================================
//
//	Defines and typedefs for Isadora

#ifndef _H_IsadoraTypes
#define _H_IsadoraTypes

#include "ArchitectureFlags.h"
#include "CPlusPlusMacros.h"
#include "CFloatXY.h"
#include "CFloatXYZ.h"
#include "NativeBitmap.h"
#include "ValueCommon.h"

#ifdef __ISADORA_APP__
#define	CONST_		
#else
#define	CONST_	const
#endif

// used to remove "unused variable warnings"

#ifndef UNUSED_VAR
#define	UNUSED_VAR(x)			{ (void) x; }
#endif

// ----------------------------------------------------------------------------------------
// UNICODE TEXT FLAGS
// ----------------------------------------------------------------------------------------

#define	ISADORA_UNICODE					TRUE

#define	ISADORA_MAC_OS_ROMAN_ONLY		(!ISADORA_UNICODE)
#define	ISADORA_LOCAL_CODE_PAGE_ONLY	(!ISADORA_UNICODE)

#include "wchar.h"

#if TARGET_OS_MAC
typedef	CFStringRef	IzzyUnicodeStr;
#elif TARGET_OS_WIN
typedef	WCHAR*		IzzyUnicodeStr;
#endif

enum IzzySubstrMode
{
	kIzzySubstr_FirstChar = 1,
	kIzzySubstr_LastChar,
	kIzzySubstr_InnerChar,
	kIzzySubstr_FirstLine,
	kIzzySubstr_LastLine,
	kIzzySubstr_InnerLine
};

// ----------------------------------------------------------------------------------------
// DATA TYPES
// Predefined data types for values whose type is kData:
// ----------------------------------------------------------------------------------------

#define	kImageBufferDataType	FOUR_CHAR_CODE('imgb')			// Bitmap Buffer: ImageBuffer*
#define	kCoreImageDataType		FOUR_CHAR_CODE('cimg')			// Core Image Buffer: CoreImageBuffer*
#define	kTextureBufferDataType	FOUR_CHAR_CODE('txtr')			// OpenGL Texture: TextureBuffer*
#define	kCoreAudioDataType		FOUR_CHAR_CODE('csnd')			// core audio
#define kDigitalAudioDataType	FOUR_CHAR_CODE('snds')			// digital audio
#define kTextDataType			FOUR_CHAR_CODE('text')			// text (ptr to c-string)

// ----------------------------------------------------------------------------------------
// HANDLES
//
// Modeled after the classic Handle (MacOS) and HANDLE (Windows) types.
//
// You can manage these with the callbacks
//
//		IzzyNewHandle_(size_t)				Create a New IzzyHandle of the specified size
//		IzzyDisposeHandle_(Handle)			Dispose an existing IzzyHandle
//		IzzyGetHandleSize_(Handle)			Get the Size of an IzzyHandle
//		IzzySetHandleSize_(Handle, Size)	Get the Size of an IzzyHandle
//		IzzyLockHandle_(Handle)				Lock the IzzyHandle in memory and get a Pointer to its data
//		IzzyUnlockHandle_(Handle)			Unlock an IzzyHandle locked with IzzyLockHandle_

typedef char*					IzzyPtr;
typedef void*					IzzyHandle;

// ----------------------------------------------------------------------------------------
// VIDEO BUFFER TYPES
// Identified one of the video buffer types supported by Isadora
// ----------------------------------------------------------------------------------------

enum
{
	kVideoBuffer_Invalid,
	kVideoBuffer_Bitmap,		// ImageBuffer (see ImageBufferCommon.h)
	kVideoBuffer_Texture,		// TextureBuffer (see TextureBufferCommon.h)
	kVideoBuffer_CoreImage		// CoreImageBuffer (see CoreImageBuffer.h) -- MAC OS X Only
};

typedef UInt8 VideoBufferType;

// ----------------------------------------------------------------------------------------
// MULTI IMAGE BUFFER
// ----------------------------------------------------------------------------------------

typedef struct
{
	VideoBufferType				mVideoBufferType;	// which type of video buffer
	
	union {
		struct ImageBuffer*		mImageBuffer;		// when mVideoBufferType is kVideoBuffer_Bitmap
		struct CoreImageBuffer*	mCoreImageBuffer;	// when mVideoBufferType is kVideoBuffer_Texture
		struct TextureBuffer*	mTextureBuffer;		// when mVideoBufferType is kVideoBuffer_CoreImage
		void*					mAnyImageBuffer;	// can be used when the caller simply wants to know if _any_ of the above pointers is valid 
	};
	
} MultiImageBuffer;

typedef MultiImageBuffer*	MultiImageBufferPtr;

// ----------------------------------------------------------------------------------------
// VideoBufferOrigin
// ----------------------------------------------------------------------------------------
//
// Specifies the location of the top-left corner of a video buffer.
//
// For CPU based ImageBuffers, the default is that the top-left pixel is at the base
// address given for the image buffer. It is possible to request that the image buffer
// is at the bottom left which is more efficient for Windows movies, since that is their
// native orientation in memory.
//
// When requesting an orientation using AcquireMediaInfo, you can choose to request
// that the origin is in its native position by specifying kVideoBufferOrigin_Native.
// Currently this option is only allowed in the mOrientation member of the AcquireMediaInfo
// struct. For Mac OS movies, the bitmapped image will have a top-left orientation. For
// Windows movies, it will be bottom left. One can determine the actual orientation by
// checking the mImageBufferOrientation member of the MediaPlayInfoV2 structure.
//
// For TextureBuffers in Isadora 2.0, this is always be kVideoBufferOrigin_BottomLeft
// 
// For Core image Buffers on Mac OS X, this is always kVideoBufferOrigin_TopLeft
//
// TOP-LEFT ORIGIN		BOTTOM-LEFT ORIGIN
//
// (0,0)----------		---------------
// |             |		|             |
// |             |		|             |
// |             |		|             |
// ---------------		(0,0)----------
//

enum
{
	kVideoBufferOrigin_TopLeft = 0,
	kVideoBufferOrigin_BottomLeft,
	kVideoBufferOrigin_Native,
	
	kVideoBufferOrigin_Unknown = 255
};

typedef UInt8	VideoBufferOrigin;

// ---------------------------------------------------------------------------
// TextureFlip Flags
// ---------------------------------------------------------------------------

typedef UInt8	IzFlipCode;
enum
{
	kIzFlip_None		= 0x00,
	kIzFlip_Horz		= 0x01,
	kIzFlip_Vert		= 0x02
};

// ----------------------------------------------------------------------------------------
// PLUGIN TYPES
// ----------------------------------------------------------------------------------------
typedef OSType							IsadoraPluginCategory;	// v2.6
#define kIsadoraPluginCategoryActor		FOUR_CHAR_CODE('actr')	// v2.6
#define kIsadoraPluginCategoryControl	FOUR_CHAR_CODE('cntl')	// v2.6

typedef	OSType			PluginClassIDT;				// Plugin class ID
typedef OSType			PluginIDT;					// Plugin ID

// ----------------------------------------------------------------------------------------
// STAGE INDEX
// ----------------------------------------------------------------------------------------

typedef UInt32					StageIndexT;

#define	kInvalidStageIndex		((StageIndexT)(-1))

// ----------------------------------------------------------------------------------------
// DISPLAY INDEX (ONE-BASED)
// ----------------------------------------------------------------------------------------

typedef UInt32					DisplayIndexT;

#define	kInvalidDisplayIndex	((DisplayIndexT)(0))

// ----------------------------------------------------------------------------------------
// RENDER TARGET (2.0)
// ----------------------------------------------------------------------------------------
typedef UInt32 RenderTarget;

enum
{
	kRenderTarget_Real = 0,		// Render to a Stage Window
	kRenderTarget_Virtual = 1	// Render to a Virtual Renderer (i.e., 3D Renderer actor)
};

#define	kRenderTarget_Invalid	((RenderTarget)(-1))

// ----------------------------------------------------------------------------------------
// RENDER CHANNEL (2.0)
// ----------------------------------------------------------------------------------------

typedef SInt32 RenderChannel;

#define	kRenderChannel_Invalid	((RenderChannel)(-1))

// ----------------------------------------------------------------------------------------
//	BLEND MODE
// ----------------------------------------------------------------------------------------

enum {
	kBlendMode_Additive = 0,
	kBlendMode_Transparent = 1,
	kBlendMode_Opaque = 2
};

typedef UInt32	RenderBlendMode;		// new in 2.0

#define	kBlendMode_Disabled		((RenderBlendMode)(-1))

// ----------------------------------------------------------------------------------------
// SCENE INDEX
// ----------------------------------------------------------------------------------------

typedef	UInt32							SceneIndexT;

#define kInvalidSceneIndex				((UInt32) -1)

// ----------------------------------------------------------------------------------------
// STREAMING PORT INDEX
// ----------------------------------------------------------------------------------------
typedef	UInt32							StreamingPortIndexT;
typedef	UInt32							StreamingProtocolCode;

// ----------------------------------------------------------------------------------------
// SERIAL PORT INDEX
// ----------------------------------------------------------------------------------------

#define kInvalidSerialStreamIndex		0
#define	kSerialStreamOffset				0x00000000
typedef	StreamingPortIndexT				SerialStreamIndexT;

// ----------------------------------------------------------------------------------------
// TCP PORT INDEX
// ----------------------------------------------------------------------------------------

#define kInvalidTCPStreamIndex			0
#define	kTCPStreamOffset				0x00000100
typedef	StreamingPortIndexT				TCPStreamIndexT;

enum TCPStreamStatus
{
	kTCPStreamStatus_NoConnection,
	kTCPStreamStatus_ConnectionAttemptInProgress,
	kTCPStreamStatus_ConnectionValid,
	kTCPStreamStatus_ConnectionFailed,
	kTCPStreamStatus_ConnectionLost
};

// ----------------------------------------------------------------------------------------
// MATRIX VALUE INDEX
// ----------------------------------------------------------------------------------------

#define kInvalidMatrixValuetreamIndex	0
#define	kMatrixValueStreamOffset		0x00000200
typedef	StreamingPortIndexT				MatrixValueIndexT;

// ----------------------------------------------------------------------------------------
// HID VALUE INDEX
// ----------------------------------------------------------------------------------------

#define kInvalidHIDValuetreamIndex		0
#define	kHIDValueStreamOffset			0x00000300
typedef	StreamingPortIndexT				HIDValueIndexT;

// ----------------------------------------------------------------------------------------
// STREAMING PORT INDEX
// ----------------------------------------------------------------------------------------
// to allow input streaming from various
// communications protocols, offset flags were added
// to the streaming protocols above (Serial, TCP/IP, etc.)
// This macro will extract the real port number, which 
// now has a limit of 256 ports per protocol

#define	kExtractStreamingProtocol(p)	( (StreamingProtocolCode) ((p) & 0xFFFFFF00) )
#define	kExtractStreamingPortIndex(p)	( (StreamingPortIndexT) ((p) & 0xFF) )
#define kMaskStreamingPortIndex(p)		( (p) & 0xFF )
#define kEncodeSerialStream(p)			( (SerialStreamIndexT) (kSerialStreamOffset | (p)) )
#define kEncodeTCPStream(p)				( (TCPStreamIndexT) (kTCPStreamOffset | (p)) )
#define kEncodeMatrixValueStream(p)		( (MatrixValueIndexT) (kMatrixValueStreamOffset | (p)) )
#define kEncodeHIDValueStream(p)		( (HIDValueIndexT) (kHIDValueStreamOffset | (p)) )

// ----------------------------------------------------------------------------------------
// GL BITMAAP INDEX
// Index into the bitmap rendering contexts used with the 3D actors and the
// the 3D Renderer actor
// ----------------------------------------------------------------------------------------

typedef	UInt32				RendererIndexT;

// ----------------------------------------------------------------------------------------
// RENDER GROUP (2.0)
// 
// The render group is passed to actor callbacks throught the IsadoraParameters struct.
//
// This is a unique ID that will be the same for all actors within the same scene.
// It allows the rendering engine to layer the multiple active scenes properly.
//
// Actors can depend on the fact that this value will not change from the moment of
// activation until the moment of deactivation.
//
// ----------------------------------------------------------------------------------------

typedef	UInt32				RenderGroupT;

#define kInvalidRenderGroup	((RenderGroupT) -1)

// ----------------------------------------------------------------------------------------
// MODEL INDEX
// Index used to select a 3D model from the list of models
// ----------------------------------------------------------------------------------------

typedef	UInt32				ModelIndexT;

// ----------------------------------------------------------------------------------------
// CALLBACK3D INDEX
// Index used to select a 3D callback from the list of callbacks
// ----------------------------------------------------------------------------------------

typedef	UInt32				Callback3DIndexT;

// ----------------------------------------------------------------------------------------
// TEXTURE3D INDEX
// Index used to select a texture associated with a 3D Model from a GLContext's
// active texture list
// ----------------------------------------------------------------------------------------

typedef	UInt32				Texture3DDIndexT;

// ----------------------------------------------------------------------------------------
// TEXTURE OVERRIDE
// ----------------------------------------------------------------------------------------

typedef struct
{
	MultiImageBuffer	mMIB;				// multi-image buffer
	UInt32				mTextureIndex0;		// which texture in the 3D model will this override	
} TextureOverwriteInfo;

typedef TextureOverwriteInfo*	TextureOverwriteInfoPtr;

// ----------------------------------------------------------------------------------------
// IzzyTime
//
// The finest granulation of time used by Isadora. Currently this is specified in 
// microseconds. But to ensure abstraction, this time value can be expressed in 
// as a struct to allow redefinition to other time scales. See CIzzyMasterTimer
// for functions that will convert this time to other granularities.
// ----------------------------------------------------------------------------------------

#define IZZY_TIME_STRUCT		FALSE

#if IZZY_TIME_STRUCT

	typedef struct {
		UInt64	t;
	} IzzyTime;

	#define IzzyTimeParam	const IzzyTime&

#else

	typedef	UInt64			IzzyTime;

	#define IzzyTimeParam	const UInt64
	
#endif

// ----------------------------------------------------------------------------------------
// CapDeviceIndexT
// Zero based index for a video or audio capture device
// ----------------------------------------------------------------------------------------

typedef	UInt16					CapDeviceIndexT;

#define	kInvalidCaptureDevice	((UInt16) -1)

// ----------------------------------------------------------------------------------------
// MediaCode defines the media that can be accessed via
// the QueueMediaCommand_, QueueMediaCommandReplace_
// and ExecuteMediaCommand_ callback functions.
// 
// The first four entries match the four media types that
// can be added into the Media Window. The remainder are
// other kinds of "media" that may be controlled, e.g., a
// DV Device.
enum
{
	kMediaTypeNone = 0,		// invalid media type
	kMediaTypeMovie,		// a quicktime movie
	kMediaTypeAudio,		// an AIFF file
	kMediaTypeMidi,			// a standard midi file
	kMediaTypePicture,		// a picture,
	
	kMediaTypeDVControl,	// an external DV camera

	kMediaType3D,			// a 3D object
	kMediaType3DRenderer,	// a 3D renderer
	kMediaTypeLiveVideo		// a Live Video Input Channel
};

typedef	UInt16				MediaCode;

typedef UInt32				MediaIndexT;

#define kInvalidMediaIndex	0
#define kMaxMediaTypes		9

// ----------------------------------------------------------------------------------------
// external dv camera sub commands
enum {
	kDVSubCommandPlay,							
	kDVSubCommandPause,
	kDVSubCommandStop,
	kDVSubCommandLocate
};

// ----------------------------------------------------------------------------------------
// ACTOR TYPES
// Isadora Actors are divided into three basic categories:
//
// Watchers wait for some event to occur in the outside world,
// and then report that event. If such a thing has inputs, they
// generally are specify a range of values that define the events
// that this watcher will see.
//
// Actors manipulate data arriving at their inputs and send
// the result out of their outputs.
//
// Generators send data that has been created algorithm-
// ically using parameters specified at their inputs
//
enum {
	kClassWatcher	= FOUR_CHAR_CODE('watc'),		// Watcher
	kClassActor		= FOUR_CHAR_CODE('acto'),		// Actor
	kClassGenerator	= FOUR_CHAR_CODE('gene')		// Generator
};
typedef UInt32			ActorClasses;

enum {
	kGroupVideo						= FOUR_CHAR_CODE('vide'),

	kSubGroupvideoSources			= FOUR_CHAR_CODE('vdpp'),
	kSubGroupVideoRenderers			= FOUR_CHAR_CODE('vdrn'),
	kSubGroupVideoGeneral			= FOUR_CHAR_CODE('vdst'),
	kSubGroupVideoCoreImage			= FOUR_CHAR_CODE('vdci'),
	kSubGroupVideoQuartzComposer	= FOUR_CHAR_CODE('vdqc'),
	kSubGroupVideoFreeFrame			= FOUR_CHAR_CODE('vdff'),
	kSubGroupVideoGLSLProgram		= FOUR_CHAR_CODE('glsl'),
	
	kGroupColor						= FOUR_CHAR_CODE('colr'),
	kGroupSound						= FOUR_CHAR_CODE('snd '),

	kSubGroupCoreAudioPlugin		= FOUR_CHAR_CODE('capl'),

	kGroupMidi						= FOUR_CHAR_CODE('midi'),
	kGroupGenerators				= FOUR_CHAR_CODE('gen '),
	kGroupMouseKeyboard				= FOUR_CHAR_CODE('mskb'),
	kGroupCalculation				= FOUR_CHAR_CODE('calc'),
	kGroupControl					= FOUR_CHAR_CODE('cntl'),
	kGroupCommunications			= FOUR_CHAR_CODE('comm'),
	kGroupText						= FOUR_CHAR_CODE('text'),
	kGroupSceneControl				= FOUR_CHAR_CODE('scne'),
	kGroupUser						= FOUR_CHAR_CODE('user'),

	// NOTE: this group is used internally by Isadora - don't
	// use this as a group for your custom actor or control
	kGroupFavorite					= FOUR_CHAR_CODE('fave')
	
};

typedef	PluginClassIDT		ActorClassIDT;				// Actor class ID
typedef PluginIDT			ActorIDT;					// Actor ID

typedef	UInt32				UATime;

// if we're compiling C++, make the UserActorIDT a C++ class
// otherwise -- it's just a struct.
#ifdef __cplusplus
class UserActorIDT
{
public:
	bool	operator==(
				const UserActorIDT&			inRhs) const
					{ return memcmp(this, &inRhs, sizeof(UserActorIDT)) == 0; }


	bool	operator<(
				const UserActorIDT&			inRhs) const
			{
				if (mPluginID != inRhs.mPluginID) {
					return (mPluginID < inRhs.mPluginID);
				} else if (mTime != inRhs.mTime) {
					return (mTime < inRhs.mTime);
				} else {
					return false;
				}
			}

#else
struct UserActorIDT
{
#endif

	PluginIDT	mPluginID;	// user actor ID
	UATime		mTime;		// gmtime at which created
};

#ifndef __cplusplus
typedef struct UserActorIDT UserActorIDT;
#endif

#define	IsUserActorPluginID(a) ( (((a).mPluginID >> 24) & 0xFF) == kGlobalUserActorIDPrefix\
 || (((a).mPluginID >> 24) & 0xFF) == kLocalUserActorIDPrefix\
 || (((a).mPluginID >> 24) & 0xFF) == kMacroIDPrefix )

#define	AreUserActorIDsEqual(a, b)	((a).mPluginID == (b).mPluginID && (a).mTime == (b).mTime)

// ----------------------------------------------------------------------------------------
// MATRIX VALUE struct
//
enum {
	MatrixValue_Action_Activate,
	MatrixValue_Action_Run,
	MatrixValue_Action_Deactivate
};

typedef struct {
	UInt32	mAction;			// MatrixValue_Action_Run or MatrixValue_Action_Deactivate
	void*	mUniqueID;			// unique identifier for the sender
	UInt32	mSceneIndex1;		// scene number from which this message originated
	UInt32	mPort;				// port id of this matrix value message 
	float	mSceneIntensity;	// intensity of this scene
	float	mValue;				// intensity of this value
	UInt32	mChannelCount;		// number of channels to follow
	UInt32	mChannels[1];		// channel numbers
} MatrixValue;

// ----------------------------------------------------------------------------------------
// CONTROL TYPES
// Isadora Actors are divided into three basic categories:
//
typedef PluginClassIDT		ControlClassIDT;			// Control ID
typedef PluginIDT			ControlIDT;					// Control ID
typedef void*				IzzyCtlPtr;					// abstract ptr to isadora CIzzyControl object
typedef UInt32				IzzyCtlNum;					// control number

#define kInvalidControlNum			0xFFFFFFFF
#define	kUnassignedControlNum		0
#define	kMinControlNum				1
#define kMaxControlNum				999

////////////////////////////////////////////////////////////////////////////
// IzzyControlMsg struct
////////////////////////////////////////////////////////////////////////////
//	This struct is passed to a ReceiveMessage function in the inData
//	parameter. The plugin should typecast inData to a type of IzzyControlMsg*
//	to make use of it.

struct FloatControlMsg
{
	IzValue_Flt	mMin;					// minimum value - ignored if mDirect is true
	IzValue_Flt	mMax;					// maximum value - ignored if mDirect is true
	IzValue_Flt	mValue;					// current value
	Boolean		mDirect;				// when true, ignore min & max and don't scale
	
	Boolean		mIgnoreOutOfRange;		// only meaningful when mDirect = true
};

////////////////////////////////////////////////////////////////////////////
// ControlMsgType enum
////////////////////////////////////////////////////////////////////////////
enum ControlMsgType {
	kIzzyCtl_FloatMessage,
	kIzzyCtl_StringMessage,
	kIzzyCtl_TimecodeMessage	// 3.0.3b9
};

////////////////////////////////////////////////////////////////////////////
// IzzyControlMsg struct
////////////////////////////////////////////////////////////////////////////

struct IzzyControlMsg
{
	ControlMsgType	mType;					// message type -- see above
	IzzyCtlNum		mCtlNum;				// the control ID from which the message was received
	
	union {
		FloatControlMsg		mFloat;			// float message
		const char*			mString;		// string message
		IzValue_Timecode	mTimecode;		// timecode message
	} u;
};

typedef struct IzzyControlMsg IzzyControlMsg;

////////////////////////////////////////////////////////////////////////////
// IzzyStatus enum
////////////////////////////////////////////////////////////////////////////
//	This struct is passed to Message Receivers who want the IzzyStatus
//	message. The comments to the right of each IzzyStatusMessage indicate
//	the meaning of the member. Check these comments for the meaning of the
//	mData parameter if it is used.

enum IzzyStatusMessage {
	
	kActivateScene			= (1 << 0),			// sent when the scene is activated and fading in
	// kFadeInComplete		= (1 << 1),			// sent when the fade of a scene in is complete
	// kStartFadeOut		= (1 << 2),			// sent when a scene starts to fade out
	// kDeactivateScene		= (1 << 3),			// sent just immediately prior to deactivating a scene
	kMediaListAboutToChange	= (1 << 4),			// sent prior to the the media list being changed due to import or reordering
	kMediaListJustChanged	= (1 << 5),			// sent after to the the media list has been changed due to import or reordering
	kMediaListAddedMedia	= (1 << 6),			// media was just added to the media window. mData = MediaListChangeInfo*
	
	// ADDED AS OF V2.6
	
	// N.B. at the time of this writing (Oct 27 2018)
	// the kSceneCompleteFadeInAsPrimary message is only
	// received by the scene related controls
	
	kSceneStartFadeInAsPrimary		= (1 << 7),	// sent when a scene is activated to become primary and starts its fade in
	kSceneCompleteFadeInAsPrimary	= (1 << 8),	// sent when a scene has completed its fade in and becomes the primary scene
	kSceneListContentChanged		= (1 << 9),	// sent when the scene formatting has changed, e.g., cue numbers were shown or hidden

	// ADDED AS OF V3

	kStageListAboutToChange	= (1 << 10),		// sent before to the the stage list has been changed due to editing
	kStageListJustChanged	= (1 << 11),		// sent after to the the stage list has been changed due to editing
	
	// ADDED AS OF V3.0.8b47
	
	kSceneNameChanged		= (1 << 12)			// sent whenver the user changes a scene name
};

////////////////////////////////////////////////////////////////////////////
// MediaListChangeItem enum
////////////////////////////////////////////////////////////////////////////

enum MediaChangeCode {
	kMediaNoChange = 0,	// the media is not changing at all -- in this case mNewMediaIndex is the same as mOldMediaIndex
	kMediaDeleted = 1,	// the medicqa will be deleted -- in this case mNewMediaIndex is 0
	kMediaAdded = 2,	// the media was added to the list - in this case mOldMediaIndex is 0
	kMediaMoved = 3,	// the media is will move -- in this case mOldMediaIndex is the old index, mNewMediaIndex is the new index
	kMediaChanged = 4	// the media has changed, i.e., a new file is associated with this media
};

////////////////////////////////////////////////////////////////////////////
// MediaChangeItem struct
////////////////////////////////////////////////////////////////////////////

struct MediaChangeItem
{
	MediaChangeCode	mChangeCode;		// how the media is changing: 
	MediaCode		mMediaType;			// the kind of media
	MediaIndexT		mOldMediaIndex1;	// the old one based index in the media list
	MediaIndexT		mNewMediaIndex1;	// the new one based index in the media list
};
typedef struct MediaChangeItem MediaChangeItem;

////////////////////////////////////////////////////////////////////////////
// MediaChangeInfo struct
////////////////////////////////////////////////////////////////////////////

struct MediaChangeInfo
{
	UInt32			mMediaChangeInfoCount;
	MediaChangeItem	mMediaChangeInfoItems;
};
typedef struct MediaChangeInfo MediaChangeInfo;

////////////////////////////////////////////////////////////////////////////
// IzzyStatus struct
////////////////////////////////////////////////////////////////////////////
// Sent to actors that handle the Izzy Status message with the appropriate
// information

struct IzzyStatus
{
	IzzyStatusMessage	mStatus;		// see definitions above
	void*				mData;			// currently unused data pointer
};

typedef struct IzzyStatus IzzyStatus;

////////////////////////////////////////////////////////////////////////////
// IzzyMediaPlayHints enum
////////////////////////////////////////////////////////////////////////////
//	This struct is passed back via the GetMediaObjectProperties
//	callback. For plugins that play media, these flags provide useful
//	hints about the properties of the media that may influence the
//	choice of playback mechanism

enum {

	kIsQuickTimeCompatible		= (1 << 1),		// set for movies that can be played by 
	kIsAVFoundationCompatible	= (1 << 2),		// set for movies that can be played by Apple's AVFoundation technology
	kIsWindowsNativeFormat		= (1 << 3),		// set for movies that are in a Windows native format (e.g., AVI, WMV)
	kUsesHAPCodec				= (1 << 4),		// set for movies that use the one of the HAP codecs
												// see http://github.com/Vidvox/hap
	kIsAudioOnlyMovie			= (1 << 5),		// set for movies that have no video tracks
	kIsQuickTimeVRMovie			= (1 << 6)		// set for QuickTime VR movies
};

typedef	UInt32	IzzyMediaPlayHints;

////////////////////////////////////////////////////////////////////////////
// MPOptimizeMode enum
////////////////////////////////////////////////////////////////////////////
// This function is passed to the AcquireIzzyMoviePlayerTask function to
// specify which kind of player will be returned.

enum MPOptimizeMode
{
	kInvalidOptimizationMode = -1,
	kOptimizeForPerformance = 0,		// optimize playback for performance
	kOptimizeForInteraction = 1			// optimize playback for interaction
};

////////////////////////////////////////////////////////////////////////////
// PlaybackEngine enum
////////////////////////////////////////////////////////////////////////////
// Returned with the MediaPlayInfoV2 struct to indicate the system used
// to play the current Movie

typedef UInt32	 PlaybackEngineT;

enum {
	kPlaybackEngine_None,
	kPlaybackEngine_AVFoundation,
	kPlaybackEngine_QuickTime,
	kPlaybackEngine_DirectShow,
	kPlaybackEngine_WindowsMediaFoundation
};

typedef	UInt32		MovieEngineSelect;

enum
{
	kMovieEngineSelect_Unknown = 0xFFFFFFFF,

	kMovieEngineSelect_DS_WMF = 0,		// try Direct Show first then Windows Media Foundation
	kMovieEngineSelect_WMF_DS,			// try Windows Media Foundation first then Direct Show
	kMovieEngineSelect_DS_Only,			// try Direct Show only
	kMovieEngineSelect_WMF_Only,		// try Windows Media Foundation Only

	kMovieEngineSelect_AVF_QT,			// on windows we always try AVFoundation First
	
	kMovieEngineSelect_WindowsFirst = kMovieEngineSelect_DS_WMF,
	kMovieEngineSelect_WindowsLast = kMovieEngineSelect_WMF_Only,
	kMovieEngineSelect_WindowsDefault = kMovieEngineSelect_WMF_DS,

	kMovieEngineSelect_MacFirst = kMovieEngineSelect_AVF_QT,
	kMovieEngineSelect_MacLast = kMovieEngineSelect_AVF_QT,
	kMovieEngineSelect_MacDefault = kMovieEngineSelect_AVF_QT
};

#define	IsMovieEngineSelectWindows(a)	((a) >= kMovieEngineSelect_WindowsFirst && (a) <= kMovieEngineSelect_WindowsLast)
#define	IsMovieEngineSelectMac(a)		((a) >= kMovieEngineSelect_MacFirst && (a) <= kMovieEngineSelect_MacLast)

////////////////////////////////////////////////////////////////////////////
// IzzyMoviePlayerTask 
////////////////////////////////////////////////////////////////////////////
// Opaque pointer used by AcquireIzzyMoviePlayerTask and
// RelinquishIzzyMoviePlayerTask

typedef struct OpaqueIzzyMoviePlayerTask*   IzzyMoviePlayerTask;

// ----------------------------------------------------------------------------------------
// THINGS

typedef	void*			ThingPtr;					// abstract ptr to an isadora CThing object
typedef UInt32			ThingID;					// unique ID for every CThing object	

enum {
	kJumpRelative = 1,
	kJumpAbsolute = 2
};
typedef	UInt16			JumpMode;					// 1 = relative, 2 = absolute

// ----------------------------------------------------------------------------------------
// PROPERTIES

enum {										// a property is either input or output
	kPropertyTypeInvalid = 0,
	kInputProperty,
	kOutputProperty
};
typedef UInt8			PropertyType;		// the type of an input property

typedef UInt16			PropertyIndex;		// index used to access a property

#define	kInvalidPropertyIndex	((PropertyIndex) -1)

typedef UInt16			DestinationIndex;	// one-based index used to access a link
typedef	UInt32			PropIDT;			// property ID code

typedef void*			PropertyPtr;		// abstract ptr to an Isadora CProperty object

// This structure is passed to properties whose type
// is changing to match another input or output whose
// value display is property defined. This allows the
// linked property to use that properties functions
// to convert from a value to a string and vice-versa
typedef struct {
	PluginIDT		mActorID;			// unique ID of the source actor
	PropertyType	mPropertyType;		// either kInputProperty or kOutputProperty
	PropIDT			mPropertyID;		// unique ID of the source actor's property
} PropertyDefinedValueSource;

typedef struct PropDefinedValueSource	PropDefinedValueSource;

// PROPERTY FORMAT NUMBERS
// used to create the property display type bit mask below

enum {
	// 0 - 7
	kDisplayFormatNumberBit	= 0,			// displays number as integer or float
	kDisplayFormatOnOffBit,					// displays number as off or on (0-1)
	kDisplayFormatHexBit,					// displays number in hexadecimal format
	kDisplayFormatMIDINoteBit,				// displays number as a MIDI Note Name (0-127)
	kDisplayFormatCharacterBit,				// a alphanumeric character, from (0-255)
	kDisplayFormatTimecodeBit,				// displays a time percentage as timecode
	kDisplayFormatSecondsBit,				// display milliseconds value as integer
	kDisplayFormatMilliSecsBit,				// display milliseconds as milliseconds

	// 8 - 15
	kDisplayFormatHertzBit,					// display miillseconds as hertz (or cycles per second)
	kDisplayFormatTriggerBit,				// displays 0 as '-' and 1 as '*'
	kDisplayFormatColorBit,					// displays float as color
	kDisplayFormatVideoBit,					// displays as "video"
	kDisplayFormatSoundBit,					// displays as "sound"
	kDisplayFormatTextBit,					// displays as "text"
	kDisplayFormatFontBit,					// displays "text" as font
	kDisplayFormatMidiTicksBit,				// displays a MIDI Ticks			
	
	// 16-23
	kDisplayFormatMovieFileBit,				// displays movie file names		
	kDisplayFormatSoundFileBit,				// displays sound file names		
	kDisplayFormatPictFileBit,				// displays picture names		
	kDisplayFormatMidiFileBit,				// displays midi file names
	kDisplayFormat3DFileBit,				// displays 3D file names	
	kDisplayFormatCoreImageBit,				// displays as "image"
	kDisplayFormatCoreAudioBit,				// displays as "audio"
	kDisplayFormatTextureBit,				// display as "texture"
	
	// 24-31
	kDisplayFormatColorAlphaBit = 24,		// displays as a color, and offers the possibility to adjust alpha

	// 25-31 -- special display format flags
	kDisplayFormatGrpConformIDBit = 25,		// these properties will be automatically grouped with the matching conform IDs
	kDisplayFormatPopupMenuBit = 26,		// when user clicks on this item, call generate popup menu proc
	kDisplayFormatCustomDataBit = 27,		// DataValueInfo struct is actuall CustomDataValueInfo with encoded name
	kDisplayFormatNoConformBit = 28,		// prevents this value from conforming
	kDisplayFormatDontSnapBit = 29,			// prevents this property from getting snapped by snapshot
	kDisplayFormatPropertyDefinedBit = 30,	// display using actor supplied function
	kDisplayFormatMutableBit = 31			// property type changes to match it's input
};

// PROPERTY FORMAT FLAGS
// determines how a properties value will be displayed
enum {

	// bits 0 - 7
	kDisplayFormatInvalid			= 0,
	kDisplayFormatNumber			= (1 << kDisplayFormatNumberBit),			// 0000 0001
	kDisplayFormatOnOff				= (1 << kDisplayFormatOnOffBit),			// 0000 0002
	kDisplayFormatHex				= (1 << kDisplayFormatHexBit),				// 0000 0004
	kDisplayFormatMIDINote			= (1 << kDisplayFormatMIDINoteBit),			// 0000 0008
	kDisplayFormatCharacter 		= (1 << kDisplayFormatCharacterBit),		// 0000 0010
	kDisplayFormatTimecode			= (1 << kDisplayFormatTimecodeBit),			// 0000 0020
	kDisplayFormatSeconds			= (1 << kDisplayFormatSecondsBit),			// 0000 0040
	kDisplayFormatMilliSecs			= (1 << kDisplayFormatMilliSecsBit),		// 0000 0080
	
	// bits 8 - 15
	kDisplayFormatHertz				= (1 << kDisplayFormatHertzBit),			// 0000 0100
	kDisplayFormatTrigger			= (1 << kDisplayFormatTriggerBit),			// 0000 0200
	kDisplayFormatColor				= (1 << kDisplayFormatColorBit),			// 0000 0400
	kDisplayFormatVideo				= (1 << kDisplayFormatVideoBit),			// 0000 0800
	kDisplayFormatSound				= (1 << kDisplayFormatSoundBit),			// 0000 1000
	kDisplayFormatText				= (1 << kDisplayFormatTextBit),				// 0000 2000
	kDisplayFormatFont				= (1 << kDisplayFormatFontBit),				// 0000 4000
	kDisplayFormatMidiTicks			= (1 << kDisplayFormatMidiTicksBit),		// 0000 8000
	
	// bits 16 - 23
	/* media formats */
	kDisplayFormatMovieFile			= (1 << kDisplayFormatMovieFileBit),		// 0001	0000
	kDisplayFormatSoundFile			= (1 << kDisplayFormatSoundFileBit),		// 0002	0000
	kDisplayFormatPictFile			= (1 << kDisplayFormatPictFileBit),			// 0004	0000
	kDisplayFormatMidiFile			= (1 << kDisplayFormatMidiFileBit),			// 0008	0000
	kDisplayFormat3DFile			= (1 << kDisplayFormat3DFileBit),			// 0010	0000
	
	/* stream format */
	kDisplayFormatCoreImage			= (1 << kDisplayFormatCoreImageBit),		// 0020	0000
	kDisplayFormatCoreAudio			= (1 << kDisplayFormatCoreAudioBit),		// 0040	0000
	kDisplayFormatTexture			= (1 << kDisplayFormatTextureBit),			// 0080	0000
																			
	// bits 24 - 31
	
	/* alpha color */
	kDisplayFormatColorAlpha		= (1 << kDisplayFormatColorAlphaBit),		// 0100 0000

	/* property control flags */
	kDisplayFormatGrpConformID		= (1 << kDisplayFormatGrpConformIDBit),		// 0200 0000
	kDisplayFormatPopupMenu			= (1 << kDisplayFormatPopupMenuBit),		// 0400 0000
	kDisplayFormatCustomData		= (1 << kDisplayFormatCustomDataBit),		// 0800 0000
	kDisplayFormatNoConform			= (1 << kDisplayFormatNoConformBit),		// 1000 0000
	kDisplayFormatDontSnap			= (1 << kDisplayFormatDontSnapBit),			// 2000 0000
	kDisplayFormatPropertyDefined	= (1 << kDisplayFormatPropertyDefinedBit),	// 4000 0000
	kDisplayFormatMutable			= (1 << kDisplayFormatMutableBit),			// 8000 0000
	
	// all media formats in one
	kDisplayFormatMedia				= (kDisplayFormatMovieFile | kDisplayFormatSoundFile | kDisplayFormatPictFile | kDisplayFormatMidiFile | kDisplayFormat3DFile),

	// all media formats in one
	kDisplayFormatColorAllFormats	= (kDisplayFormatColorAlpha | kDisplayFormatColor),

	kDisplayFormatAllData			= (
										// bits 0 - 7
										kDisplayFormatNumber
										| kDisplayFormatOnOff
										| kDisplayFormatHex
										| kDisplayFormatMIDINote
										| kDisplayFormatCharacter
										| kDisplayFormatTimecode
										| kDisplayFormatSeconds
										| kDisplayFormatMilliSecs
										// bits 8 - 15
										| kDisplayFormatHertz
										| kDisplayFormatTrigger
										| kDisplayFormatColor
										| kDisplayFormatVideo
										| kDisplayFormatSound
										| kDisplayFormatText
										| kDisplayFormatFont
										| kDisplayFormatMidiTicks
										// bits 16 - 23
										| kDisplayFormatMovieFile
										| kDisplayFormatSoundFile
										| kDisplayFormatPictFile
										| kDisplayFormatMidiFile
										| kDisplayFormat3DFile
										| kDisplayFormatCoreImage
										| kDisplayFormatCoreAudio
										| kDisplayFormatTexture
										// bits 24 - 31
										| kDisplayFormatColorAlpha				// bit 24
										| kDisplayFormatPopupMenu				// bit 26
										| kDisplayFormatPropertyDefined),		// bit 30

	kDisplayFormatAllScalar				= (
										// bits 0 - 7
										kDisplayFormatNumber
										| kDisplayFormatOnOff
										| kDisplayFormatHex
										| kDisplayFormatMIDINote
										| kDisplayFormatCharacter
										| kDisplayFormatTimecode
										| kDisplayFormatSeconds
										| kDisplayFormatMilliSecs
										// bits 8 - 15
										| kDisplayFormatHertz
										| kDisplayFormatTrigger
										| kDisplayFormatColor
										// | kDisplayFormatVideo
										// | kDisplayFormatSound
										| kDisplayFormatText
										| kDisplayFormatFont
										// | kDisplayFormatMidiTicks
										// bits 16 - 23
										| kDisplayFormatMovieFile
										| kDisplayFormatSoundFile
										| kDisplayFormatPictFile
										| kDisplayFormatMidiFile
										| kDisplayFormat3DFile
										// | kDisplayFormatCoreImage
										// | kDisplayFormatCoreAudio
										// | kDisplayFormatTexture
										// bits 24 - 31
										| kDisplayFormatColorAlpha				// bit 24
										// | kDisplayFormatPopupMenu			// bit 26
										| kDisplayFormatPropertyDefined),		// bit 30

	kDisplayFormatFloatIntCharTextColor	= (
										// bits 0 - 7
										kDisplayFormatNumber
										| kDisplayFormatOnOff
										| kDisplayFormatHex
										| kDisplayFormatMIDINote
										| kDisplayFormatCharacter
										// | kDisplayFormatTimecode
										// | kDisplayFormatSeconds
										// | kDisplayFormatMilliSecs
										// bits 8 - 15
										// | kDisplayFormatHertz
										// | kDisplayFormatTrigger
										| kDisplayFormatColor
										// | kDisplayFormatVideo
										// | kDisplayFormatSound
										| kDisplayFormatText
										// | kDisplayFormatFont
										// | kDisplayFormatMidiTicks
										// bits 16 - 23
										// | kDisplayFormatMovieFile
										// | kDisplayFormatSoundFile
										// | kDisplayFormatPictFile
										// | kDisplayFormatMidiFile
										// | kDisplayFormat3DFile
										// | kDisplayFormatCoreImage
										// | kDisplayFormatCoreAudio
										// | kDisplayFormatTexture
										// bits 24 - 31
										| kDisplayFormatColorAlpha				// bit 24
										// | kDisplayFormatPopupMenu			// bit 26
										| kDisplayFormatPropertyDefined),		// bit 30

	kDisplayFormatControlFlags		= (kDisplayFormatGrpConformID
										| kDisplayFormatCustomData
										| kDisplayFormatNoConform
										| kDisplayFormatDontSnap
										| kDisplayFormatPropertyDefined
										| kDisplayFormatMutable)

};

// allowed types: all scalar values -- only data is not allowed
#define kAllowedTypes_AllScalar "number,onoff,hex,pitch,char,timecode,seconds,millisecs,hertz,trig,color,text,font,moviefile,soundfile,pictfile,midifile,3dfile,acolor,propdef"

#define kAllowedTypes_FloatIntCharTextColor "number,onoff,hex,pitch,char,color,text,acolor,propdef"

// ISADORA 2.0 SDK
// RENAMED PROPERTY FORMAT FLAG CONSTANTS
//
// The following three constants were renamed for Isadora 2.0
// Their values, however, did not changed. Existing, compiled
// plugins should run fine if they use these constants
//
// If your code contains these constants, the defines below
// will generate to compilation errors, reminding you to 
// change the constants as shown below

#define	kDisplayPropertyNoConform	RENAMED_IN_ISAODRA_20	// change to kDisplayFormatNoConform
#define	kDisplayPropertyDontSnap	RENAMED_IN_ISAODRA_20	// change to kDisplayFormatDontSnap
#define	kDisplayPropertyDefined		RENAMED_IN_ISAODRA_20	// change to kDisplayFormatPropertyDefined

// the display format for a property
typedef	UInt32				PropertyDispFormat;		
typedef UInt32				PropertyOptions;
typedef UInt8				MutableGroup;

#define ExtractPropertyDisplayFormatBit(a, b)		( ((PropertyDispFormat) (a)) & ((PropertyDispFormat) (b)) )
#define MaskPropertyDisplayFormatBit(a, b)			( ((PropertyDispFormat) (a)) & ~((PropertyDispFormat) (b)) )
#define IsPropertyDisplayFormatBitSet(a, b)			( ExtractPropertyDisplayFormatBit(a, b) != 0 )

#define ExtractPropertyDisplayFormatMutableBit(a)	ExtractPropertyDisplayFormatBit(a, kDisplayFormatMutable)
#define MaskPropertyDisplayFormatMutableBit(a)		MaskPropertyDisplayFormatBit(a, kDisplayFormatMutable)
#define IsPropertyDisplayFormatMutableBitSet(a)		IsPropertyDisplayFormatBitSet(a, kDisplayFormatMutable)

#define	kIndMutableGroup	255

enum {
	kPropertyOption_DrawValueGraphically = (1 << 0)
};

#define	kTriggerIdleString		"-"
#define	kTriggerActiveString	"X"

// ----------------------------------------------------------------------------------------
// USER ACTOR CONSTANTS

#define	kGlobalUserActorIDPrefix		'$'
#define	kLocalUserActorIDPrefix			'#'
#define	kMacroIDPrefix					'@'
#define	kUserInPropIDPrefix				'i'
#define kUserOutPropIDPrefix			'o'

// ----------------------------------------------------------------------------------------
// MOD KEY CONSTANTS
// bit mask indicates which modifier key(s) are pressed

enum {
#if TARGET_OS_MAC
	kModKey_MacCmd_WinCtl_Mask		= (1 << 0),
	kModKey_MacCtl_WinCmd_Mask		= (1 << 1),
	kModKey_MacOpt_WinAlt_Mask		= (1 << 2),
	kModKey_MacShf_WinShf_Mask		= (1 << 3),
	kModKey_ShiftLockMask			= (1 << 4)
#elif TARGET_OS_WIN
	kModKey_MacCmd_WinCtl_Mask		= (1 << 0),
	kModKey_MacCtl_WinCmd_Mask		= (1 << 2),
	kModKey_MacOpt_WinAlt_Mask		= (1 << 1),
	kModKey_MacShf_WinShf_Mask		= (1 << 3),
	kModKey_ShiftLockMask			= (1 << 4)
#endif
};

// ----------------------------------------------------------------------------------------
// Measure/Beat/Tick conversion struct

typedef struct {
	UInt16	mMeasure;	// set on input
	UInt16	mBeat;		// set on input
	UInt16	mTick;		// set on input
} MBT;

// ----------------------------------------------------------------------------------------
// MIDI timecode Types (See MTCReader class)

enum MTCType
{
	kMTCType_24,
	kMTCType_25,
	kMTCType_30DF,
	kMTCType_30,
	kMTCType_Invalid = -1
};

CLASSHDR TimecodeHMSF {
PUBLIC
#ifdef __cplusplus

	TimecodeHMSF() : h(0), m(0), s(0), f(0) { }
	
	bool	operator==(const TimecodeHMSF& t) { return (h == t.h) && (m == t.m) && (s == t.s) && (f == t.f); }
	bool	operator!=(const TimecodeHMSF& t) { return (h != t.h) || (m != t.m) || (s != t.s) || (f != t.f); }
	
	UInt32	ToUInt32() { return *((UInt32*) this); }
	
#endif

	UInt8	h;
	UInt8	m;
	UInt8	s;
	UInt8	f;
};

#define ZeroTimecodeHMSF(a)	{ memset(&(a), 0, sizeof(a)); }

// ----------------------------------------------------------------------------------------
// MTCGeneratorRef - used with timecode Generator Functions

typedef struct OpaqueMTCGeneratorRef*   MTCGeneratorRef;

// ----------------------------------------------------------------------------------------
// MTCGenMode - run mode used with timecode Generator Functions

enum MTCGenMode
{
	kMTCGenMode_InternalClock,
	kMTCGenMode_ExternalClock
};
	
// ----------------------------------------------------------------------------------------
// LoopStepInfo

typedef struct {
	UInt16					mLoopStepIndex1;	// current loop step index - 0 if none
	UInt16					mRepeatIndex1;		// current loop repeat index - 0 if non
	UInt16					mLoopMode;			// 0 = forward, 1 = backward, 2 = palindrome
	float					mLoopRate;			// loop rate; +1.0 = forward, normal, -1.0 = backwards normal; +2.0 = forward twice normal, etc.
	UInt32					mLoopStart;			// current loop start point
	UInt32					mLoopDuration;		// current loop duration in milliseconds
} LoopStepInfo;

// ----------------------------------------------------------------------------------------
// MediaPlayInfoV1
//
// DEPRECATED: this struct has been replaced by the MediaPlayInfo structure
// this version is maintained to allow older plugins to still function
// propertly, but should not be used for new plugins

typedef struct {
	
	struct ImageBuffer*		mImageBuffer;		// returns pointer to media's ImageBuffer (if there is one)
	struct CoreImageBuffer*	mCoreImageBuffer;	// returns pointer to media's CoreImageBuffer (if there is one)
	float					mPlayPosition;		// returns pointer to current play position
	Boolean					mIsPlaying;			// returns true if media is playing now

	// only valid when using movie looper -- outputs giving loop status
	LoopStepInfo			mLoopStepInfo;
	
	// audio metering
#define kMaxAudioVolumeMeteringChannels	8
	Boolean					mAudioVolumeMeteringInfoValid;									// if true then the next two fields are valid
	UInt32					mNumAudioVolumeMeteringChannels;								// number of valid channels in the mAudioVolumeMeteringLevels array
	Float32					mAudioVolumeMeteringLevels[kMaxAudioVolumeMeteringChannels];	// levels for the audio metering
	
} MediaPlayInfoV1;

// ----------------------------------------------------------------------------------------
// MediaPlayInfo

enum {
	kMediaPlayinfoV2_Version_Isadora20 = 1,									// initial version as of Isadora 2.0
	kMediaPlayinfoV2_Version_Isadora30 = 2,									// initial version as of Isadora 3.0
	kMediaPlayinfoV2_Version_Isadora308 = 3,								// Isadora v3.0.8
	
	kMediaPlayInfo_CurrentVersion = kMediaPlayinfoV2_Version_Isadora308		// current version for this SDK
};

enum PanRespect {
	kPanRespect_Unknown,
	kPanRespect_No,
	kPanRespect_Yes
};

typedef struct {

// VERSION NUMBER
// IMPORTANT!!!! MUST BE SET BY CALLERS BEFORE REQUESTING MEDIA PLAY INFO

	UInt32					mVersion;						// Isadrora 2.0: version of this struct - see enums above

// ------------------------------------------
// kMediaPlayinfoV2_Version_Isadora20 MEMBERS
// ------------------------------------------

// VISUAL MEDIA
	struct ImageBuffer*		mImageBuffer;					// returns pointer to media's ImageBuffer (if there is one)
	struct CoreImageBuffer*	mCoreImageBuffer;				// returns pointer to media's CoreImageBuffer (if there is one)
	struct TextureBuffer*	mTextureBuffer;					// returns pointer to media's TextureBuffer (if there is one)
	
// LOCATBLE MEDIA
	float					mPlayPositionPercent;			// returns the current play position as a percentage of the total media duration
															// (previously called mPlayPosition)
// PLAYABLE MEDIA
	Boolean					mIsPlaying;						// returns true if media is playing now

// VISUAL MEDIA
	VideoBufferOrigin		mVideoBufferOrigin;				// Isadora 2.0: the origin of the video buffer. (Currently only valid for Movies)

// MOVIES ONLY
	PlaybackEngineT			mPlaybackEngine;				// Isadora 2.0: playback engine used. (Valid for Movies only)
															//              Will be either kVideoBufferOrigin_TopLeft or kVideoBufferOrigin_BottomLeft
// MOVIES - PRIVATE
	LoopStepInfo			mLoopStepInfo;					// only when used with MoviePlayer++; not publicly available

// IMAGE BUFFER FLAGS
	Boolean					mImageBufferIsPremultiplied;	// Isadora 2.0: set to TRUE if image buffer contains premultiplied data

// ------------------------------------------
// kMediaPlayinfoV2_Version_Isadora30 MEMBERS
// ------------------------------------------

// LOCATBLE MEDIA
	float					mPlayPositionSeconds;			// returns the current play position in seconds

// AURAL MEDIA
	UInt32					mAudioChannels;					// returns number of audio channels for the all audio tracks/streams
	
// ------------------------------------------
// kMediaPlayinfoV2_Version_Isadora308 MEMBERS
// ------------------------------------------

// AURAL MEDIA
	PanRespect				mPanRespect;					// see PanRespect enum above

} MediaPlayInfo;

#define	MediaPlayInfo_Size_For_Version_1		(sizeof(MediaPlayInfo))

// ----------------------------------------------------------------------------------------
// MediaFoundTextInfo

typedef struct {
	bool	mTextFound;				// true if text was found
	float	mTextStartPercent;		// start point of text as a percentage
	float	mTextDurationPercent;	// duration of text as a percentage
	bool	mTimecodeValid;			// true if timecode points are valid
	UInt32	mTextStartTimecode;		// start point of text as timecode
	UInt32	mTextEndTimecode;		// end point of text as timecode
} MediaFoundTextInfo;

// ----------------------------------------------------------------------------------------
// Acquire ID

// Except in the case of a Preload actor, when media is loaded
// or preloaded, the caller will provide a non-zerp Acquire ID 
// thatuniquely identifies the owner of the loaded media. Preload
// actors use the special value kGenericPreloadAcquireID when preloading
// media to show that the media can be acquired by any module

#if IZ_64BIT
typedef	int64_t	AcquireIDT;
#else
typedef	SInt32	AcquireIDT;
#endif

#define kGenericPreloadAcquireID	0

// ----------------------------------------------------------------------------------------
// OutputGeometry

typedef struct
{
	bool		mValid;							// when true -- data is valid
	float		mDirectOutputCenterH;			// direct output - image h center : 0.0 = centered; +/- 1.0 = full stage width
	float		mDirectOutputCenterV;			// direct output - image v center : 0.0 = centered; +/- 1.0 = full stage height
	float		mDirectOutputWidth;				// direct output - image width : 1.0 = 100%
	float		mDirectOutputHeight;			// direct output - image height : 1.0 = 100%
	float		mDirectOutputZoom;				// direct output - image zoom : 1.0 = 100%
	bool		mDirectOutputPreserveAspect;	// direct output - preserve aspect ratio if true

} OutputGeometry;

#define	InitOutputGeometry(og) { og.mValid = true; og.mDirectOutputCenterH = 0.0f; og.mDirectOutputCenterV = 0.0f; og.mDirectOutputWidth = 1.0f; og.mDirectOutputHeight = 1.0f; og.mDirectOutputZoom = 1.0f; og.mDirectOutputPreserveAspect = false; } 

// ----------------------------------------------------------------------------------------
// MediaIdleMode  ** DEPRECATED **
// These flags are no longer used and are ignored by the kMediaIdle command 

enum {
	kMediaIdleNone = 0,
	kMediaIdleWaitBackgroundService,
	kMediaIdleWaitNewFrame
};

typedef	SInt32	MediaIdleMode;	// ** DEPRECATED **

// ----------------------------------------------------------------------------------------
// ShowMediaInfo

typedef struct
{
	double			mOutMovieDurationSeconds;		// output: duration of the movie
	double			mOutFrameRate;					// output: frame rate as reported by the movie
	UInt32			mOutVideoTrackCount;			// output: number of video tracks
	UInt32			mOutAudioTrackCount;			// output: number of audio tracks
} ShowMovieInfo;

typedef struct {
	union {
		ShowMovieInfo	movie;
	} u;
} ShowMediaInfo;

// ----------------------------------------------------------------------------------------
// PreloadMode

enum PreloadMode {

	kKeepUntilUnloadedMask	= (1 << 0),
	kKeepUntilNextPlayFlag	= 0,
	kKeepUntilUnloadedFlag	= kKeepUntilUnloadedMask,
	
	kKeepInRAMMask			= (1 << 1),
	kDontKeepInRAMFlag		= 0,
	kKeepInRAMMFlag			= kKeepInRAMMask,

	kKeepUntilNextPlay		= kDontKeepInRAMFlag | kKeepUntilNextPlayFlag,
	kKeepUntilUnloaded		= kDontKeepInRAMFlag | kKeepUntilUnloadedFlag,
	kKeepUntilNextPlayInRAM = kKeepInRAMMFlag	 | kKeepUntilNextPlayFlag,
	kKeepUntilUnloadedInRAM = kKeepInRAMMFlag	 | kKeepUntilUnloadedFlag
};

// ----------------------------------------------------------------------------------------
// PreloadStatus

enum PreloadStatus {
	kPreloadStatus_Incomplete,
	kPreloadStatus_Complete,
	kPreloadStatus_NotFound
};

// ----------------------------------------------------------------------------------------
// PostPreloadPlayOptions

enum {
	kPostPreloadPlayFlags_None				= 0,
	kPostPreloadPlayFlags_MasterVolumeValid	= (1 << 0)
};

typedef uint32_t PostPreloadPlayFlags;

struct PostPreloadPlayOptions {
	PostPreloadPlayFlags	mFlags;
	float					mMasterVolume;
};

typedef struct PostPreloadPlayOptions	PostPreloadPlayOptions;

////////////////////////////////////////////////////////////////////////////
// ActorPreloadCommand
////////////////////////////////////////////////////////////////////////////
//
enum ActorPreloadCommand {
	kActorPreloadCommand_Service,
	kActorPreloadCommand_Cancel
};

////////////////////////////////////////////////////////////////////////////
// ActorPreloadStatus
////////////////////////////////////////////////////////////////////////////
//
enum ActorPreloadStatus
{
	kActorPreloadStatus_Idle = 0,
	kActorPreloadStatus_PreloadInitiated,			// true if preload action is in progress
	kActorPreloadStatus_PreloadComplete,			// true when preload action is complete
	kActorPreloadStatus_FirstFrameDelivered			// true after the first frame of playback has been delivered
};

////////////////////////////////////////////////////////////////////////////
// ActorPreloadInfo
////////////////////////////////////////////////////////////////////////////
//

typedef const void* MediaPlayerUniqueID;

CLASSHDR ActorPreloadInfo
{
PUBLIC
#ifdef __cplusplus
								ActorPreloadInfo() :
									mRefCount(1),
									mActorPreloadStatus(kActorPreloadStatus_Idle),
									mMediaType(kMediaTypeNone),
									mMediaIndex1(kInvalidMediaIndex),
									mPreloadInitTime_nS(0)
								{
								}
								
	virtual						~ActorPreloadInfo()
								{
								}

	void						Retain() const
									{ mRefCount += 1; }
									
	bool						Release() const
								{
									mRefCount -= 1;
									
									if (mRefCount == 0) {
										delete this;
										return true;
									}
									
									return false;
								}
									
	virtual bool				InitiatePreload() = 0;
	
	virtual ActorPreloadStatus	Service() = 0;
	
	virtual	bool				PostPreloadPlay() = 0;
	
	virtual	bool				PostPreloadPlayAtTime(
									uint64_t		inSystemTimeNanoseconds) = 0;
	
	virtual	void				PreloadComplete() = 0;
	
#endif

	mutable int			mRefCount;
	ActorPreloadStatus	mActorPreloadStatus;		// see ActorPreloadStatus above
	MediaCode			mMediaType;					// media type for preloaded item
	MediaIndexT			mMediaIndex1;				// media index for preloaded item
	uint64_t			mPreloadInitTime_nS;		// system time in microseconds
};

// ----------------------------------------------------------------------------------------
// HSBCAdjust struct

//	Specifies adjustmens to the hue, saturation, brightness, and contrast

struct HSBCAdjust
{
	float					mHue;
	float					mSaturation;
	float					mBrightness;
	float					mContrast;
};

typedef struct HSBCAdjust HSBCAdjust;

// ----------------------------------------------------------------------------------------
// MONITOR WINDOW
//

// MWDirection identifies the "direction" of a message to be shown
// in the Monitor window.
//
// note: if the order of input and/or output is changed, then
// you must also re-order these items in sDirectionToWantBits
// in CMonitorWindow.cp

enum MWDirection {
	kMWDirection_Invalid = 0,
	kMWDirection_Input,				// message came from an input
	kMWDirection_Output,			// message came from an output
	kMWDirection_MaxDirections
};

// MWMsgType identifies the type of a message to be shown
// in the the Monitor Window. Each of these will be related
//	with a MWDirection to indicate if the message was 
//	received or transmitted by Isadora.
//
// note: if the order of the message types is changed, then
// you must also re-order these items in sMsgTypeToWantBits
// in CMonitorWindow.cp

enum MWMsgType {
	kMWMsgType_Invalid = 0,
	kMWMsgType_MIDI,
	kMWMsgType_OSC_Summary,
	kMWMsgType_OSC_Detail,
	kMWMsgType_Serial,
	kMWMsgType_TCPIP,
	kMWMsgType_JavascriptOutput,	// used only with kMWDirection_Output
	kMWMsgType_JavascriptError,		// used only with kMWDirection_Output
	kMWMsgType_OSC_Error,			// v2.6
	kMWMsgType_MaxMsgTypes
};

// ----------------------------------------------------------------------------------------
// Movie Playback Callbacks

#ifdef __cplusplus
extern "C" {
#endif

	typedef void (*MediaHitLoopEndProc)(void* inRefCon, void* inInfo1, void* inInfo2);

	typedef void (*MediaTextOutputProc)(void* inRefCon, char* inText, SInt32 inTextLen);

#ifdef __cplusplus
}
#endif

// ----------------------------------------------------------------------------------------
// Text Track Setup Info

typedef struct {
	SInt32	mTextTrackIndex1;
	void*	mRefCon;
} TextOutputInfo;

// ----------------------------------------------------------------------------------------
// Timecode Related

// N.B. The TimecodeType values do NOT match the MTCType values.
//
// Take note that 23.976 is not a drop frame timecode rate.
// (In fact, there is no such thing as "24 fps drop frame.) As
// is true for 29.97 (30 DF) video, a TC value of 00:00:01:00
// at 23.976 fps represents slightly more than a second. But
// _unlike_ 29.97 (30 DF), no timecode values are skipped for
// material playing back at 23.976.
//
// It's all wonderfully confusing. For more, see
//
// https://en.wikipedia.org/wiki/24p#23.976p
//

enum TimecodeType
{
	kTimecodeType_Default = 0,	// 0 = default frame rate for document
	kTimecodeType_23976,		// 1 = 23.976 fps
	kTimecodeType_24,			// 2 = 24.000 fps
	kTimecodeType_25,			// 3 = 25.000 fps
	kTimecodeType_2997,			// 4 = 29.970 fps
	kTimecodeType_30,			// 5 = 30.000 fps
	kTimecodeType_50,			// 6 = 50.000 fps
	kTimecodeType_5994,			// 7 = 59.940 fps
	kTimecodeType_60,			// 8 = 60.000 fps
	kTimecodeType_Max			// 9 = number of rates in the list
};

typedef int32_t	TimecodeFrames;

typedef uint8_t	TimecodeFPS;

typedef bool	TCDropFrameFlag;

// the bitwise layout of an IzValue_Timecode is as follows
//
// | upper 32 bits                           | lower 32 bits                           |
// | ---- ---- ---- ---- ---- ---- ---- TTTT | ssss ssss ssss ssss ssss ssss ssss ssss |
//
// There TTTT is a four bit TimecodeType, and the lower 32 bits are a 32-bit floating
// point value representing the number of seconds.
//
// The following macros allow you to extrct the type and seconds, or to make a new timecode
// value from the type and seconds values.

#define GetTimecodeType_(tc) 						((TimecodeType)(((tc) >> 32) & 0x0F))
#define GetTimecodeSeconds_(tc, floatOut)			(*((uint32_t*)&(floatOut)) = ((tc) & 0xFFFFFFFF))

#define TimecodeValueMakeWithTypeAndSeconds_(t, s)	(((uint64_t) (t)) << 32 | (uint64_t)(*((uint32_t*) &(s))))

typedef struct
{
	uint8_t		mEncTimecodeType : 3;		// see constants above
	uint8_t		mHours : 5;
	uint8_t		mMinutes;
	uint8_t		mSeconds;
	uint8_t		mFrames;
	
} EncodedTCValue;

// ----------------------------------------------------------------------------------------
// AcquireMediaInfo

#define	AMI_MAX_AUDIO_TRACKS	64

// this constant is depcrecated... see the equivalent
// values in VideoBufferOrigin
enum {
	DEPRECATED_kOrientation_BaseAddressIsTopLeft = 0,
	DEPRECATED_kOrientation_BaseAddressIsBottomLeft
};

// this general version of the struct gives basic information that
// any visual media might require to be loaded into memory. Movies
// and Sounds have their own version of this struct which is defined
// below

struct AcquireMediaInfo
{
	PreloadMode			mPreloadMode;					// how to keep the media preloaded
	bool				mLoadIntoRAM;					// if true, attempt to load media into RAM

	StageIndexT			mTargetStageIndex0;				// desired stage window index (ignored if not direct output or core image)

	VideoBufferType		mVideoBufferType;				// desired output buffer type for visual media

	// note: the following three flags are mutually exclusive
	// only one of them should be set true
	bool				mCoreImageOutput;				// set to true to request core image output for visual media
	bool				mDirectToStageOutput;			// set to true to request that the movie be rendered direct to the stage
	bool				mForceGWorldOutput;				// set to true to force visual media to be rendered to a GWorld
	
	// NOTE: in previous versions of the SDK, this flag should
	// have been set to kOrientation_BaseAddressIsTopLeft
	// or kOrientation_BaseAddressIsBottomLeft which are now
	// deprecated.
	//
	// Older implementations will work because the values
	// or kVideoBufferOrigin_TopLeft and kVideoBufferOrigin_BottomLeft
	// match the old values.
	
	// As of In Isadora 2.0 you may now additionally request that
	// the visual media keep its native orientation.
	//
	// In Isadora 3.0, this setting only implemented for Movies and
	// will result in top-left orientation on all platforms and 
	// for all playback engines.
	//
	// In Isadora 2.0, this setting was only impelemented for Movies
	// and will result in top-left orientation for Mac OS playback
	// (i.e., AVFoundation/Quicktime) and bottom-left orientation
	// on for Windows (i.e., DirectShow) movies
	
	VideoBufferOrigin	mRequestedOrientation;			// either kVideoBufferOrigin_TopLeft, kVideoBufferOrigin_BottomLeft or kVideoBufferOrigin_Native
};	

typedef struct AcquireMediaInfo AcquireMediaInfo;

// video output device ref gives the name and mode for a video output device
#define Isadora_VideoOutDeviceInfo_NameLength		256
#define Isadora_VideoOutDeviceInfo_ModeLength		256

struct VideoOutDeviceInfo
{
	char	mVideoOutputDeviceName[Isadora_VideoOutDeviceInfo_NameLength];
	char	mVideoOutputDeviceVideoMode[Isadora_VideoOutDeviceInfo_ModeLength];
	char	mVideoOutputDeviceAudioMode[Isadora_VideoOutDeviceInfo_ModeLength];
};

typedef struct VideoOutDeviceInfo VideoOutDeviceInfo;

#define Isadora_AudioOutDeviceInfo_NameLength		256

// audio output device ref gives name of audio output device
struct AudioOutDeviceInfo
{
	char	mAudioOutputDeviceName[Isadora_AudioOutDeviceInfo_NameLength];
};

typedef struct AudioOutDeviceInfo AudioOutDeviceInfo;

// this version is specific to movies
struct AcquireMovieInfo
{
	AcquireMediaInfo	mInfo;
	
	bool				mLinkToDisplay;					// when true, use display link to drive output
	
	OutputGeometry		mOutputGeometry;				// only used if mDirectToStageOutput is true

	// playback control
	Fixed				mRate;							// desired playback speed; 65536 = 1.0
	bool				mLoopEnable;					// set to true if movie will loop
	bool				mPalindromeEnable;				// set to true if movie will palindrome loop; ignored if mLoop is false
	
	float				mLoopStartPercent;				// the loop start position as a percentage of total duration
	float				mLoopDurationPercent;			// the loop duration position as a percentage of total duration
	float				mLoopPosition;					// the play start position as a percentage of the loop duration
	
	bool				mUseSecondsLoopPoints;			// true if timecode loop points are valid; otherwise use percentages
	float				mLoopStartSeconds;				// loop start position in seconds
	float				mLoopEndSeconds;				// loop duration in seconds
	float				mLoopPositionSeconds;			// play start position in seconds
	
	const char*			mUseMarkerLoopPoints;			// when true, gives a text track marker to search for in the text track
	
	MediaHitLoopEndProc	mHitLoopEndProc;				// callback called when the loop endpoint is reached
	void*				mHitLoopEndRefCon;				// hit loop end callback reference constant
	void*				mHitLoopEndInfo1;				// hit loop end callback info data 1
	void*				mHitLoopEndInfo2;				// hit loop end callback info data 1

	// video output
	VideoOutDeviceInfo*	mVideoOutputDeviceInfo;			// NULL means output to stage; non-null gives information about the desired video output device
	
	// video
	HSBCAdjust			mHSBCAdjust;					// hue/saturation/brightness/contrast adjustments; core image video movies only

	// sound
	float				mMasterVolume;					// the master volume setting: 100% = unity
	float				mMasterPan;						// the master pan setting: 0% = left, 50% = center, 100% = right
	SInt32				mMasterSoundOutputRouting;		// the mast sound output routing

	// multi-channel sound (64 tracks max)
	UInt32				mIndTrackCount;									// 0 = no individual track routing; > 0 = ind volume/pan/routing arrays are valid
	float				mIndTrackVolume[AMI_MAX_AUDIO_TRACKS];			// array of volumes for individual tracks
	float				mIndTrackPan[AMI_MAX_AUDIO_TRACKS];				// array of pans for invididual tracks
	SInt32				mIndSoundOutputRouting[AMI_MAX_AUDIO_TRACKS];	// array of sound output routings for invidividual tracks

	// audio frequency monitor
	UInt32				mAudioMonitorFrequencyBands;	// audio frequency monitor: 0 = off; > 0 = number of bands

	// text tracks
	MediaTextOutputProc	mTextOutputProc;				// text track output callback
	SInt32				mTextOutputTrackIndex1;			// track to be output
	void*				mTextOutputRefCon;				// text track callback reference constant
	void*				mTextOutputInfo1;				// text track callback info data 1
	void*				mTextOutputInfo2;				// text track callback info data 2
	
	// quicktime vr
	float				mVRPanPercent;
	float				mVRTiltPercent;
	float				mVRZoomPercent;
	
	// acquisition output
	ShowMovieInfo		mShowMovieInfo;					// on successful acquire, these fields are filled in

	AudioOutDeviceInfo*	mAudioOutDeviceInfo;			// v2.5.3
};

typedef struct AcquireMovieInfo AcquireMovieInfo;

struct AcquireSoundInfo {
	
	AcquireMediaInfo	mInfo;
	
	// channel
	SInt32				mSoundChannel;					// channel on which the sound will play
	
	// volume/pan
	float				mMasterVolume;					// the master volume setting: 100% = unity
	float				mMasterPan;						// the master pan setting: 0% = left, 50% = center, 100% = right
	
	// speed
	Fixed				mRate;							// desired playback speed; 65536 = 1.0
	
	// looping
	bool				mLoopEnable;					// set to true if movie will loop
	float				mLoopStartPercent;				// the loop start position as a percentage of total duration
	float				mLoopDurationPercent;			// the loop duration position as a percentage of total duration
	SInt32				mCrossfadeFrames;				// number of frames for crossfade
	
	// chunk
	bool				mChunkEnable;
	SInt32				mChunkSizeFrames;
	SInt32				mChunkSkipFrames;
	
	// hit loop end (added 3.0.8b1)
	MediaHitLoopEndProc	mHitLoopEndProc;				// callback called when the loop endpoint is reached
	void*				mHitLoopEndRefCon;				// hit loop end callback reference constant
	void*				mHitLoopEndInfo1;				// hit loop end callback info data 1
	void*				mHitLoopEndInfo2;				// hit loop end callback info data 1
};

typedef struct AcquireSoundInfo AcquireSoundInfo;

// currently, we only have information for movies
// later we may add types for other media

struct AcquireInfo {
	union {
		AcquireMediaInfo	media;
		AcquireSoundInfo	sound;
		AcquireMovieInfo	movie;
	} u;

};

typedef struct AcquireInfo	AcquireInfo;

// ------------------------------------------------------------
// IZMEDIAINFO struct

// this version is specific to movies
struct IzMediaInfo
{
	MediaCode	mMediaCode;			// See MediaCode enum list of types.
	MediaIndexT	mMediaIndex1;		// the index of the media that used by this actor
	double		mFrameRate;			// frame rate in frames per second. (0.0 = invalid)
	double		mDuration;			// duration in seconds. (0.0 = invalid)
	MTCType		mTimecodeType;		// valid for the specified media if not kMTCType_Invalid
	bool		mCanInteract;		// true if media performs well under interactive control
	bool		mReversePlayback;	// true if media supports reverse playback
	UInt32		mAudioChannels;		// if the media contains audio, the number of channels in the audio stream
};

typedef struct IzMediaInfo	IzMediaInfo;

// ------------------------------------------------------------
// IZAUDIODEVICE struct

typedef UInt32 IzAudioBitDepth;

enum
{
	kIzAudioDeviceInfo_BasicInfoOnly		= 0,			// mix channelsa and sample rate
	kIzAudioDeviceInfo_ActualDeviceInfo		= (1 << 0),		// actual device name and channels
	kIzAudioDeviceInfo_RequestedDeviceInfo	= (1 << 1)		// requested device name and channels
};

typedef UInt32 IzAudioDeviceInfoFlags;

// this version is specific to files with audio. the fields
// that will be filled in when requesting this information are
// dependent on the IzAudioDeviceInfoFlags passed to the
// media command queue via kMediaGetCurrentAudioDeviceInfo

struct IzAudioDeviceInfo
{
// these fields are always filled in regardless of the flags
	UInt32			mMixChannels;				// channels available in the backend mixer
	UInt32			mSampleRate;				// sample rate in Hz

// only filled in if kIzAudioDeviceInfo_ActualDeviceInfo is set
	char			mActualDeviceName[128];		// name of current physical output device
	UInt32			mActualDeviceChannels;		// channels on current physical output device
	
// only filled in if kIzAudioDeviceInfo_RequestedDeviceInfo is set
	char			mRequestedDeviceName[128];	// name of current physical output device
	UInt32			mRequestedDeviceChannels;	// channels on current physical output device
};

typedef struct IzAudioDeviceInfo	IzAudioDeviceInfo;

CLASSHDR PostPreloadPlayInfo
{
PUBLIC

#ifdef __cplusplus
	PostPreloadPlayInfo() :
		mRequiresPostPreloadPlay(FALSE),
		mMinPrerollCompleteUntilPlayTime_mS(0),
		mPostPlayMinTimeUntilSceneChange_mS(0)
	{
	}
#endif

	Boolean		mRequiresPostPreloadPlay;
	UInt32		mMinPrerollCompleteUntilPlayTime_mS;
	UInt32		mPostPlayMinTimeUntilSceneChange_mS;
};

#define InitAcquireInfo(a) {\
	memset(&(a), 0, sizeof(AcquireInfo));\
	(a).u.media.mTargetStageIndex0 = kInvalidStageIndex;\
}

#define InitAcquireMediaInfo(a) {\
	memset(&(a), 0, sizeof(AcquireMediaInfo));\
	(a).mTargetStageIndex0 = kInvalidStageIndex;\
}

#define InitAcquireSoundInfo(a) {\
	memset(&(a), 0, sizeof(AcquireSoundInfo));\
	(a).mRate = 65536;\
	(a).mLoopDurationPercent = 100.0;\
	(a).mMasterVolume = 100.0;\
	(a).mMasterPan = 50.0;\
}

#define InitAcquireMovieInfo(a) {\
	memset(&(a), 0, sizeof(AcquireMovieInfo));\
	(a).mInfo.mTargetStageIndex0 = kInvalidStageIndex;\
	(a).mRate = 65536;\
	(a).mLoopEnable = true;\
	(a).mLoopDurationPercent = 100.0;\
	(a).mHSBCAdjust.mSaturation = 1.0;\
	(a).mHSBCAdjust.mContrast = 1.0;\
	(a).mMasterVolume = 100.0;\
	(a).mMasterPan = 50.0;\
}

typedef struct OpaqueVideoBufferRef*   IzzyVideoBufferRef;

struct AcquireLiveVideoStreamRequest {
	
	VideoBufferType		mVideoBufferType;			// type of video buffer requested
	
	StageIndexT			mTargetStageIndex0;			// zero based target stage index when mVideoBufferType is
													// kVideoBuffer_Texture && kVideoBuffer_CoreImage;
													// ignored when mVideoBufferType is kVideoBuffer_Bitmap
	
	IzzyVideoBufferRef	mVideoBufferRef;			// non null when successfully acquired
	
};

typedef struct AcquireLiveVideoStreamRequest AcquireLiveVideoStreamRequest;

struct LiveVideoStreamInfo {
	
	struct TextureBuffer*	mTextureBuffer;
	struct ImageBuffer*		mImageBuffer;
	struct CoreImageBuffer*	mCoreImageBuffer;
};

typedef struct LiveVideoStreamInfo LiveVideoStreamInfo;

// IS-1163: Switching From Modern to Legacy Doesnt Always Update the Video Stream Inside Isadora
// for the Video In Watcher, we need to know if we are currently capturing or not. This structure
// adds a flag to LiveVideoStreamInfo that gives us this information. This structure is used in
// coordination with the kMediaGetLiveVideoStream command in the media command queue.

struct LiveCaptureVideoStreamInfo {

	bool				mIsCapturing;
	LiveVideoStreamInfo	mLiveVideoStreamInfo;
};

typedef struct LiveCaptureVideoStreamInfo LiveCaptureVideoStreamInfo;


// ----------------------------------------------------------------------------------------
// MEDIA COMMANDS
//
// Media Commands are passed via the callback mechanism (one of
// QueueMediaCommand_, QueueMediaCommandReplace_, or
// ExecuteMediaCommand_) to tell Isadora to manipulate some
// piece of media. DoQueueMediaCommand and  QueueMediaCommandReplace_
// can be safely called at interrupt/thread time, regardless of the type of
// media. ExecuteMediaCommand_ should be called for functions that are
// interrupt/thread safe only.
//

enum
{
	// do nothing command
	kMediaNullCmd = 0,						// param1 = n/a									param2 = n/a
	
	// get play info for a time-based media object
	DEPRECATED_kMediaGetMediaPlayInfoV1,	// param1 = pointer to MediaPlayInfo			param2 = n/a
	
	// get the output image buffer for a visual media object
	kMediaGetImageBuffer,					// param1 = pointer to CImageBuffer*			param2 = n/a
	
	// retrieve name of a media object
	kMediaGetName,							// param1 = pointer to char buffer				param2 = size of buffer in bytes
	
	// note that show media will also start playing the media
	kMediaShowMedia,						// param1 = ShowMediaInfo*						param2 = non-zero to load into RAM
	kMediaShowMediaDirect,					// param1 = ShowMediaInfo*						param2 = non-zero to load into RAM
	kMediaHideMedia,						// param1 = n/a									param2 = n/a
	
	// commands to acquire and release a media object
	kMediaAcquire,							// param1 = Boolean* = successfully acquired	param2 = for movies, AcquireMediaInfo*
	kMediaRelease,							// param1 = n/a									param2 = n/a
	
	// commands to preload and unload preloaded media
	kMediaPreload,							// param1 = AcquireMediaInfo*					param2 = fade in time in milliseconds
	DEPRECATED_kMediaUnload,				// param1 = n/a									param2 = n/a
	
		// time based playback control
	kMediaPlay,								// param1 = fixed rate with 0x10000 = 1.0		param2 = sound channel for audio media, 0 for others
	kMediaStop,								// param1 = n/a									param2 = n/a
	kMediaRestart,							// param1 = n/a									param2 = n/a
	kMediaGetPlaying,						// param1 = Boolean* = true if playing			param2 = n/a
	kMediaGoToTime,							// param1 = millisecond location				param2 = n/a
	kMediaSetSpeed,							// param1 = UnsignedFixed speed: 0x10000 = 1.0	param2 = n/a
	
		// location
	kMediaSetPosition_Seconds,				// param1 = IzReal = seconds					param2 = n/a
	kMediaGetPosition_Seconds,				// param1 = IzReal* = seconds					param2 = n/a
	kMediaSetLoopStartEnd_Seconds,		// param1 = IzReal = loop start					param2 = IzReal = loop duration
	
	kMediaSetPosition_Percent,				// param1 = locate position (float 0-100%)		param2 = n/a
	kMediaGetPosition_Percent,				// param1 = float* = the position				param2 = n/a
	kMediaSetPositionWithinLoop_Percent,	// param1 = locate position (float 0-100%)		param2 = n/a
	
		// looping
	kMediaGetLoopStartDurationPercent,		// param1 = UInt32* = loop start percent		param2 = UInt32* = loop duration percent
	kMediaSetLoopStart_Percent,				// param1 = loop start as % of total dur		param2 = n/a
	kMediaFindTextInTextTrack,				// param1 = const char* = the text				param2 = MediaFoundTextInfo
	kMediaSetLoopDuration_Percent,			// param1 = loop dur as % of total dur			param2 = n/a
	kMediaSetLoopStartDuration_Percent,		// param1 = loop start as % of total dur		param2 = n/a
											// param2 = loop duration as % of total dur		param2 = n/a
	kMediaSetLoopFlag,						// param1 = looping on or off (0 or 1)			param2 = palindromic looping on or off (0 or 1)
	
		// set crossfade frames
	kMediaSetCrossfadeFrames,				// param1 = number of frames					param2 = n/a
	
		// position of visual media on the stage
	kMediaSetHorizPosition,					// param1 = horizontal position in pixels		param2 = n/a
	kMediaSetVertPosition,					// param1 = vertical position in pixels			param2 = n/a
	kMediaSetWidth,							// param1 = width position in pixels			param2 = n/a
	kMediaSetHeight,						// param1 = height position in pixels			param2 = n/a

		// sound files
	kMediaSetVolume_Percent,				// param1 = float* = volume, from 0 to 100%		param2 = n/a
	kMediaSetPan_Percent,					// param1 = float* = panning, from 0 to 100%	param2 = n/a

		// external dv devices
	kMediaDVControlCommand,					// param1 = dv sub command						param2 = timecode location for locate
	
		// updating the stage
	kMediaUpdateStage,						// param1 = stage index (one-based)				param2 = n/a
	
		// audio chunk features
	kMediaSetChunkEnable,					// param1 = boolean 1 or 0						param2 = n/a
	kMediaSetChunkSizeFrames,				// param1 = chunk size in frames				param2 = n/a
	kMediaSetChunkSkipFrames,				// param1 = chunk skip in frames				param2 = n/a
	
		// Quicktime VR
	kMediaSetVRPan_Percent,					// param 1 = float* = pan in degrees			param2 = n/a
	kMediaSetVRTilt_Percent,				// param 1 = float* = tilt in degrees			param2 = n/a
	kMediaSetVRZoom_Percent,				// param 1 = float* = field of zoom in percent	param2 = n/a
	
		// access to object count
	kMediaGetObjectCount,					// param1 = UInt32* to receive count			param2 = n/a
	
		// set tempo for a time based media with BPM measurement
	kMediaSetTempo,							// param1 = float* = tempo						param2 = n/a

		// convert measures, beats, and ticks into a percentage
	kMediaMBTToPercent,						// param1 = MBT*								param2 = float*
	
		// force the output midi channel to the specified channel
	kMediaForceMidiChannel,					// param1 = 0 for assignments in MIDI file, 1-16 to force to that channel
	
		// convert milliseconds to a percentage for the specified media object
	kMediaMillisecondsToPercent,			// param1 = milliseconds						param2 = ptr to float

		// set the hit loop end proc for the specified time based media
	kMediaSetHitLoopEndProc,				// param1 = MediaHitLoopEndProc pointer			param2 = callback refcon
	
		// set sound output routing codes for media that can output sound
	kMediaSetExtSoundOutputRouting,			// param1 = 0 for built-in, 1 for outputs 1 & 2, 2 for outputs 3 & 4, etc.
	
	kMediaPlayAudioStream,					// param1 = CSoundProvider* - left chan			param2 = CSoundProvider* - right chan
	kMediaStopAudioStream,					// param1 = n/a									param2 = n/a
	
		// Sound Movie Settings
	kMediaSetTrackVolume_Percent,			// param1 = Track Index (one-based				param2 = float* = with volume, from 0 to 100%
	kMediaSetTrackPan_Percent,				// param1 = Track Index (one-based)				param2 = float* = panning, from 0 to 100%
	kMediaSetTrackExtSoundOutputRouting,	// param1 = Track Index (one-based				param2 = 0 for built-in, 1 for outputs 1 & 2, 2 for outputs 3 & 4, etc.
	
		// 3D Models
	kMedia3DAttachModel,					// param1 = ModelInfo*							param2 = n/a
	kMedia3DDetachModel,					// param1 = RenderTarget						param2 = One Based Render Target Channel

		// 3D Stage Orientation
	kMedia3DRotation,						// param1 = CFloatXYZ* = delta rotation			param2 = n/a
	kMedia3DTranslate,						// param1 = CFloatXYZ* = delta translation		param2 = n/a
	kMedia3DScale,							// param1 = CFloatXYZ* = delta translation		param2 = n/a
	
	kMedia3DOverwriteTextureMap,			// param1 = see below							param2 = NULL or TextureOverwriteInfo*
											// if param2 is 0, then param1 = ImageBufferPtr
											// if param2 is not 0, then param1 = TextureOverwriteInfoPtr
											
	kMedia3DSetTextureMapScaling,			// param1 = ptr to CFloatXYZ = scaling values	param2 = n/a
	kMedia3DSetUseAlpha_Deprecated,			// this command is deprecated as of v1.3.0f14
	kMedia3DSetTransparency,				// param1 = float* - transparency from 0 to 1	param2 = n/a

		// 3D Renderes
	kMedia3DAcquireBitmapRenderer,			// param1 = width								param2 = height
	kMedia3DDisposeRenderer,				// param1 = n/a									param2 = n/a
	kMedia3DRenderDraw,						// param1 = n/a									param2 = 1 for stage / 0 for bitmap renderer
	kMedia3DCameraRotation,					// param1 = delta rotation - ptr to CFloatXYZ	param2 = 1 for stage / 0 for bitmap renderer
	kMedia3DCameraTranslate,				// param1 = delta rotation - ptr to CFloatXYZ	param2 = 1 for stage / 0 for bitmap renderer
	kMedia3DGetImageBuffer,					// param1 = CImageBuffer**
	
		// 3D Drawing Callbacks
	kMedia3DAttachDrawCallback,				// param1 = one based renderer channel			param2 = ptr to Render3DInfo
	kMedia3DDetachDrawCallback,				// param1 = one based renderer channel			param2 = ptr to Render3DInfo
	kMedia3DContextNeedsDraw,				// param1 = RenderTarget						param2 = One Based Render Target Channel
	kMedia3DCreateTextureMap,				// param1 = one based renderer channel			param2 = ptr to TextureMapInfo
	kMedia3DDisposeTextureMap,				// param1 = ptr to TextureMapInfo				param2 = n/a
	kMedia3DStoreTextureMapImage,			// param1 = ptr to TextureMapInfo				param2 = n/a
	kMedia3DPrepareTextureMapForRender,		// param1 = ptr to TextureMapInfo				param2 = ptr to PluginContextInfo
	kMedia3DSetLightPositionAndColors,		// param1 = ptr to LightingInfo	  				param2 = 1 for stage / 0 for bitmap renderer
	
		// hue-saturation-brightness-contrast control (CoreImage Movies Only!)
	kMediaAdjustHSBC,						// param1 = ptr to HSBCAdjust struct			param2 = n/a
	
		// stage options
	kMediaSetStageScalingOptions,			// param1 = ptr to StageScalingOptions  		param2 = 1 for stage / 0 for bitmap renderer
	kMediaSetStageBackgroundOptions,		// param1 = ptr to StageBackgroundOptions		param2 = 1 for stage / 0 for bitmap renderer
	kMediaClearStageBackgroundOptions,		// param1 = ptr to StageBackgroundOptions 		param2 = 1 for stage / 0 for bitmap renderer
	kMediaUpdateStageBackgroundOptions,		// param1 = ptr to StageBackgroundOptions		param2 = 1 for stage / 0 for bitmap renderer
	
		// get core image buffer for a visual media object
	kMediaGetCoreImageBuffer,				// param1 = pointer to CoreImageBuffer*			param2 = n/a

		// get core image buffer for a visual media object
	kMediaSetLoopList,						// param1 = pointer to CLoopStepList			param2 = n/a
											// use NULL ptr to clear the list


	kMediaIdle,								// param1 = n/a									param2 = n/a
	
	kMediaEnableAudioFrequencyMonitor,		// param1 = 1 to enable, 0 to disable
	kMediaGetAudioFrequencyLevels,			// param1 = UInt32* - channels in level array	param2 = Float32* - array of levels
	
	kMediaCommandGroupStart,				// param1 = n/a									param2 = n/a
	kMediaCommandGroupEnd,					// param1 = n/a									param2 = n/a
	
	kMediaUpdateOutputGeometry,				// param1 = OutputGeometry*						param2 = n/a
	
	kMediaSetGLRenderOptions,				// param1 = ptr to GLRenderOptions				param2 = 1 for stage / 0 for bitmap renderer
	kMediaClearGLRenderOptions,				// param1 = ptr to GLRenderOptions				param2 = 1 for stage / 0 for bitmap renderer

	kMedia3DSetRenderOptions,				// param1 = ptr to Media3DRenderOptions			param2 = n/a

	kMediaActivate,							// param1 = n/a									param2 = n/a

	kMediaSetGlobalStageKeystoneOptions,	// param1 = ptr to StageKeystoneOptions			param2 = 1 for stage / 0 for bitmap renderer
	kMediaGetGlobalStageKeystoneOptions,	// param1 = ptr to StageKeystoneOptions			param2 = 1 for stage / 0 for bitmap renderer
	
	kMediaSetGlobalEdgeBlendOptions,		// param1 = ptr to StageEdgeBlendOptions		param2 = 1 for stage / 0 for bitmap renderer
	kMediaGetGlobalEdgeBlendOptions,		// param1 = ptr to StageEdgeBlendOptions		param2 = 1 for stage / 0 for bitmap renderer
	
		// set the hit loop end proc for the specified time based media
	kMediaSetTextOutputProc,				// param1 = MediaTextOutputProc pointer			param2 = TextOutputInfo*
	
		// returns true only if the AcquireMediaInfo for the specified
		// object matches that given by param1
	kIsMediaPreloaded,						// param1 = AcquireMediaInfo*					param2 = PreloadStatus*

	kMediaGetDirectVideoOutputDevice_DEPRECATED,		// param1 = CExtVideoOutput*					param2 = n/a

	kMediaExists,							// param1 = Boolean* = true if media exists		param2 = Boolean* = true if can be preloaded

	kMediaGetTextureBuffer,					// param1 = pointer to TextureBuffer*			param2 = n/a

	kMedia3DRenderModel,					// param1 = one based renderer channel			param2 = Media3DDrawingOverrides

	kMedia3DOverrideTexture,				// param1 = ptr to Media3DTextureOverride		param2 = n/a

	kMedia3DSet3DModulationOptions,			// param1 = ptr to Media3DModOptions			param2 = n/a
	
	kMedia3DSetBackgroundLighting,			// param1 = ptr to BackgroundLightingInfo		param2 = 1 for stage / 0 for bitmap renderer

	kMediaGetLiveVideoImageBuffer,			// param1 = ptr to ImageBufferPtr				param2 = n/a

	kMediaRequiresPostPreloadPlay,			// param1 = PostPreloadPlayInfo*				param2 = n/a
	
	kMediaPostPreloadPlay,					// param 1 = PostPreloadPlayOptions*			param2 = n/a

	kMediaReadyAfterPostPreloadPlay,		// param 1 = Boolean* = true if ready			param2 = n/a

	kMediaGetPreloadStatus,					// param1 = AcquireMediaInfo*					param 2 = PreloadStatus*

	kMedia3DGetCameraTranslateAndRotate,	// param1 = CFloatXYZ[2] - trans + rot			param2 = 1 for stage / 0 for bitmap renderer
	
	kMedia3DAttachDrawWithContextCallback,	// param1 = one based renderer channel			param2 = ptr to Render3DWithContextInfo
	kMedia3DDetachDrawWithContextCallback,	// param1 = one based renderer channel			param2 = ptr to Render3DWithContextInfo

	kMedia3DAcquireTextureRenderer,			// param1 = stage index (one-based)				param2 = SInt32[2] = width/height
	kMedia3DGetTextureBuffer,				// param1 = TextureBufferPtr*
	
	kMediaAcquireLiveVideoStream,			// param1 = AcquireLiveVideoStreamRequest*		param2 = n/a
	kMediaReleaseLiveVideoStream,			// param1 = IzzyVideoBufferRef					param2 = n/a
	kMediaGetLiveVideoStream,				// param1 = IzzyVideoBufferRef					param2 = LiveCaptureVideoStreamInfo*	// IS-1163
	
	// get play info for a time-based media object
	// NOTE: on entry the mVersion member must be set by the caller
	kMediaGetMediaPlayInfo,					// param1 = pointer to MediaPlayInfoV2			param2 = n/a

	kMedia3DCanAcquireRenderer,				// param1 = n/a									param2 = Boolean* = output, yes or no

	kMedia3DCameraFieldOfView,				// param1 = float* = field of view				param2 = n/a

	kMediaSetAudioOutputDevice,				// param1 = const char* = name of device		param2 = n/a
	
	kMediaGetMediaInfo,						// param1 = IzMediaInfo*						param2 = n/a

	// if a movie is already loaded and playing, you  can use this
	// call to change the origin of the bitmap image returned by
	// the movie

	kMediaUpdateVideoOrigin,				// param1 = VideoBufferOrigin*					param2 = n/a
	
	// 3.0.8b1 -- added
	// the first parameter is a valid sound output routing string
	// in the form "s1:d1, [s2:d2,] ..." where each "s:d" pairing
	// specifies a routing from the source sound channel in the
	// media to the destination sound channel on the hardware device
	
	kMediaSetSoundChannelOutputRouting,		// param1 = const char*							param2 = n/a

	// 3.0.8b9 - added
	// returns information about the current audio device including
	// it's name and the number of output channels it offers. For this
	// call the media object number is the device number.
	kMediaGetCurrentAudioDeviceInfo,		// param1 = IzAudioDeviceInfoFlags				param2 = IzAudioDeviceInfo*
	
	// marker for the last command
	kMediaLastCommand
};
typedef	UInt16				MediaCommand;

// ----------------------------------------------------------------------------------------
//	MediaCommandComplete Callback
//
//	Passed as a parameter to QueueMediaCommand_ or QueueMediaCommandReplace_ functions, this
//	callback will be called when the media command has been executed.

#ifdef __cplusplus
extern "C" {
#endif

typedef	void	(*MediaCommandCompleteProc)(UInt32	inQueuedCommandID);

#ifdef __cplusplus
}
#endif				

////////////////////////////////////////////////////////////////////////////
// FORWARD DECLARATIONS
////////////////////////////////////////////////////////////////////////////

CLASSHDR IsadoraTexture;		// forward declaration for IsadoraTexture
CLASSHDR IsadoraFrameBuffer;	// forward declaration for IsadoraFrameBuffer;

////////////////////////////////////////////////////////////////////////////
// CAPTURE MODE DEFINES
////////////////////////////////////////////////////////////////////////////

typedef	UInt8	CaptureModeFlags;

enum {
	kIzzyCaptureNone = 0,
	kIzzyCaptureVideo = 1,
	kIzzyCaptureAudio = 2
};

typedef	UInt8	VideoCaptureInputType;
enum {
	kIzzyVidCaptureDefaultInput,
	kIzzyVidCaptureComposite,
	kIzzyVidCaptureSVideo,
	kIzzyVidCaptureComponent,
	kIzzyVidCaptureTV
};

typedef	UInt8	AudioCaptureInputType;
enum {
	kIzzyAudCaptureDefaultInput,
	kIzzyAudCaptureLineIn,
	kIzzyAudCaptureInternalMicrophone,
	kIzzyAudCaptureDeviceInput1,
	kIzzyAudCaptureDeviceInput2,
	kIzzyAudCaptureDeviceInput3,
	kIzzyAudCaptureDeviceInput4,
	kIzzyAudCaptureDeviceInput5,
	kIzzyAudCaptureDeviceInput6
};

typedef	UInt16	CaptureChannelIndex;

// ---------------------------------------------------------------
// User Image Processing Preferences
// 
// The user sets these preferences to determine what to do when
// two or more images are being combined in some way (i.e.,
// crossfades, keying, etc.)
//
// kImageProcessLargerSlower indicates that the user prefers that you
// scale the smaller image up to match the largest one, thus preserving
// quality at the expense of speed
//
// kImageProcessSmallerFaster indicates that the user wishes to scale
// larger images down to match the smallest image size, thus reducing
// quality but increasing processing speed
//
// kImageProcessFixedSize indicates that the user has set a fixed
// height and width at which all images should be processed. If this
// option is chosen, video output for an actor should always be scaled
// to the user specified size.
//
// kImageProcessMatchFormatOnly indicates that an intermediary buffer
// should only be created if the formats don't match -- but the images
// will never be scaled
//
enum {
	kImageProcessLargerSlower,					// scale the smaller up to match the larger
	kImageProcessSmallerFaster,					// scale the larger down to match the smaller
	kImageProcessFixedSize,						// process at user specified fixed size
	kImageProcessUseDefault,					// parameter passed to IsIntermediaryImageBufferRequired/CreateComplimentaryImageBuffer
												// to indicate that default should be used
	kImageProcessMatchFormatOnly				// created buffer to match formats, but don't scale
};
typedef SInt16	ImageProcessingMode;

#define	kMinimumImageProcessingDepthRGB		32
#define	kMinimumImageProcessingDepthYUV		16

struct ImageProcessingPrefs
{
	ImageProcessingMode			mImgProcMode;			// see Image Processing Preferences above...
	SInt16						mFixedSizeWidth;		// internal video processing width if mImgProcPref == kImageProcessFixedSize
	SInt16						mFixedSizeHeight;		// internal video processing height if mImgProcPref == kImageProcessFixedSize
	SInt16						mFixedBitDepth;			// internal video processing height if mImgProcPref == kImageProcessFixedSize
};
typedef struct ImageProcessingPrefs ImageProcessingPrefs;

// ----------------------------------------------------------------------------------------
// Media3DDrawMode determine how the models are drawn

enum {
	k3DDrawMode_Normal = 0,
	k3DDrawMode_Wireframe = 1,
	k3DDrawMode_Points = 2
};

typedef UInt8	Media3DDrawMode;

// ----------------------------------------------------------------------------------------
// Media3DRenderOptions provides flags used when rendering a 3D Media, i.e., a 3D Model
// 
struct Media3DRenderOptions
{
	Boolean				mRenderBackfaces;
	Boolean				mDepthTest;
	Boolean				mEnableLighting;
	RenderBlendMode		mRenderBlendMode;
	Media3DDrawMode		mDrawMode;
};
typedef struct Media3DRenderOptions Media3DRenderOptions;
typedef const Media3DRenderOptions* Media3DRenderOptionsPtr;

// ----------------------------------------------------------------------------------------
// Media3DTextureOverride provides information used to override how the texture is
// applied to a 3D Model
// 
struct Media3DTextureOverride
{
	Boolean			mEnableExtTexture;		// if true, use the texture ID specified below for the first texture
	
	IsadoraTexture*	mGLTexture;				// if non-null, use this texture for the override, otherwise user target/id below
	int				mClampMode;				// if non-zero, force this clamp mode when rendering
	Boolean 		mFlipIfOriginBotLeft;	// if TRUE, flip the texture matrix if the texture has a bottom left origin
	
	Boolean			mEnableSubImage;		// if true, use the four fields below to use a portio of the texture map
	
	float			mOffsetH;				// offset to the left edge, from 0.0 to 1.0
	float			mOffsetV;				// offset to the top edge, from 0.0 to 1.0
	float			mSizeH;					// width from 0.0 to 1.0
	float			mSizeV;					// height from 0.0 to 1.0
};
typedef struct Media3DTextureOverride Media3DTextureOverride;
typedef const Media3DTextureOverride* Media3DTextureOptionsPtr;

// ----------------------------------------------------------------------------------------
// Media3DDrawingOverrides allows the caller to override the transparency, color and 
// shininess of the 3D object, as well as the color of the ambient, diffuse and
// specular light sources
//

	struct Media3DGroupOverride
	{
		UInt8	mOverrideEnable;	// enables the override
		UInt32	mGroupMinIndex0;	// lowest group number to render
		UInt32	mGroupMaxIndex0;	// highest group number to render
	};
	typedef Media3DGroupOverride	Media3DGroupOverride;

	struct Media3DValueOverride
	{
		UInt8	mOverrideEnable;	// enables the override
		UInt8	mModulate;			// when false, replace the existing value; when true, modulate the current value by mValue
		float	mValue;				// the value, from 0.0f to 1.0f
	};
	typedef Media3DValueOverride	Media3DValueOverride;

	struct Media3DColorOverride
	{
		UInt8	mOverrideEnable;	// enables the override
		UInt8	mModulate;			// when false, replace the existing color; when true, modulate it
		float	mColor[4];			// the color values, from 0.0f to 1.0f
	};
	typedef Media3DColorOverride	Media3DColorOverride;


struct Media3DDrawingOverrides
{
	// object overrides
	Media3DColorOverride	mObjectColorOverride;		// overrides the color of the object
														// default object color = all white when lighting enabled
														// default object color = diffuse color when lighting disabled
														
	Media3DValueOverride	mTransparencyOverride;		// overrides the transparency of the object; applied after mObjectColorOverride
	
	Media3DValueOverride	mShininessOverride;			// overrides shininess of the object

	// lighting overrides
	Media3DColorOverride	mAmbientLightOverride;		// overrides the color of the ambient light source	
	Media3DColorOverride	mDiffuseLightOverride;		// overrides the color of the diffuse light source
	Media3DColorOverride	mSpecularLightOverride;		// overrides the color of the specular light source
	
	// selected group rendering overrides
	Media3DGroupOverride	mMediaGroupOverride;
};

typedef struct Media3DDrawingOverrides Media3DDrawingOverrides;
typedef const Media3DDrawingOverrides* Media3DDrawingOverridesPtr;

// ----------------------------------------------------------------------------------------
// Media3DPositionMod sets parameters that affect the way the frequency content
// will modulate the size of a specified 3D Model
// 

enum
{
	kModMode3D_None,
	
	kModMode3D_Position_X		= (1 << 0),		// modulation affects the x position of the object
	kModMode3D_Position_Y		= (1 << 1),		// modulation affects the y position of the object
	kModMode3D_Position_Z		= (1 << 2),		// modulation affects the z position of the object
	
	kModMode3D_Rotation_X		= (1 << 3),		// modulation affects rotation of the object about the x-axis
	kModMode3D_Rotation_Y		= (1 << 4),		// modulation affects rotation of the object about the y-axis
	kModMode3D_Rotation_Z		= (1 << 5),		// modulation affects rotation of the object about the z-axis
	
	kModMode3D_Scale_X			= (1 << 6),		// modulation affects scaling of the object along the x-axis
	kModMode3D_Scale_Y			= (1 << 7),		// modulation affects scaling of the object along the y-axis
	kModMode3D_Scale_Z			= (1 << 8),		// modulation affects scaling of the object along the z-axis
	
	// convenience constant to scale along all axes
	kModMode3D_AllScale			= (kModMode3D_Scale_X | kModMode3D_Scale_Y | kModMode3D_Scale_Z),

	kModMode3D_ColorRandom		= (1 << 9)		// modulation affects the color of the object

};
typedef	UInt16				ModMode3D;

// ----------------------------------------------------------------------------------------
// Media3DSoundMod specifies how sound input will modifiy the individual objects
// within a 3D Model.
// 

struct Media3DSoundMod
{
	CaptureChannelIndex	mCaptureChannelIndex;		// one based channel index specifying the audio capture channel
	
	ModMode3D			mModMode;					// what will be modulated; see ModMode3D
	
	float				mModDepth;					// scaling factor specifies the amount of influence
		
	float				mPositionMultiplier;		// an additional scaling factor when influencing the x/y/z position of the object
	float				mRotationMultiplier;		// an additional scaling factor when influencing the x/y/z rotation of the object
	float				mScalingMultiplier;			// an additional scaling factor when influencing the x/y/z scaling of the object
	float				mColorMultiplier;			// an additional scaling factor when influencing the color of the object
	
	float				mModMinimum;				// for freq channels, the min frequency; for volume channels, the min amplitude
	float				mModMaximum;				// for freq channels, the max frequency; for volume channels, the max amplitude
	
	UInt32				mAffectedObjIndex;			// if the zero based object index is < this value, it is affected by sound modulation
};

typedef struct Media3DSoundMod Media3DSoundMod;

// ----------------------------------------------------------------------------------------
// Media3DPositionMod allows a position input to further modify the how
// sound parameters manipulate the individual objects within a 3D Model.
// 

struct Media3DPositionMod
{
	Boolean				mEnabled;				// true if this modulation is active
	
	CFloatXYZ			mPos;					// the center of influence for the modulation
	float				mRangeOfInfluence;		// range of influence for the modulation from mPos
	float				mScaleFactor;			// the overall scaling factor for the pos and rot depth factors
	
	CFloatXYZ			mPosDepth;				// the amount of influence over the position, from +1 to +1
	CFloatXYZ			mRotDepth;				// the amount of influence over the position, from +1 to +1
	CFloatXYZ			mScaleDepth;			// the amount of influence over the scaling, from +1 to +1
	float				mPositionMultiplier;	// an additional scaling factor when influencing the x/y/z position of the object
	float				mRotationMultiplier;	// an additional scaling factor when influencing the x/y/z rotation of the object
	float				mScalingMultipler;		// an additional scaling factor when influencing the x/y/z scaling of the object

	float				mColorDepth;			// the amount of influence over the color, from 0 to 1.
	float				mColorMultiplier;		// an additional scaling factor when influencing the color of the object
	
	float				mRiseFactor;			// response factor when new value is higher than old factor
	float				mFallFactor;			// response factor when new value is lower than old factor
};

typedef struct Media3DPositionMod Media3DPositionMod;

// ----------------------------------------------------------------------------------------
// Media3DFlyinMod allows one to selectively "fly in" the objects within a single
// 3D Model.
// 

struct Media3DFlyinMod
{
	UInt32		mFlyInCount;		// objects with indexes < this value are flown in; others are suspended
	UInt32		mFlyInAxis;			// axis along which objects will fly. 0 = x, 1, = y, 2 = z
	CFloatXYZ	mFlyInSource;		// the source point from which objects will fly
	float		mFlyInCondense;		// the amount of "condensation" around the source point
	float		mFlyinMaxSpeed;		// the flyin maximum speed in units per second
	float		mFlyinSlowDownDist;	// distance at which the object starts slowing down
};

typedef struct Media3DFlyinMod Media3DFlyinMod;

// ----------------------------------------------------------------------------------------
// Media3DModOptions allows 3D Models with multiple objects to be modulated in 
// various ways. See the Media3DSoundMod & Media3DPositionMod structs for
// more information.
// 

#define MAX_POSITION_MODULATORS		4

struct Media3DModOptions
{
	// SOUND MOD OPTIONS
	float					mSoundRiseFactor;			// response factor when new value is higher than old factor
	float					mSoundFallFactor;			// response factor when new value is lower than old factor
	
	UInt32					mMinVisibleObject;			// v2.6 - zero based index of the first object to draw
	UInt32					mMaxVisibleObject;			// v2.6 - zero based index of the last object to draw
	
	Media3DSoundMod			mVolumeMod;
	Media3DSoundMod			mFrequencyMod;

	CFloatXYZ				mRandomPosMin;				// mimimum value for the random position values generated when modulatiing by position
	CFloatXYZ				mRandomPosMax;				// mimimum value for the random position values generated when modulatiing by position

	UInt32					mRandomPosMode;				// when non-zero, randomly set the position of each object using the ranges
														// given by mRandomPosMin and mRandomPosMax
	UInt32					mRandomPosTextureColumns;	// when > 1, divide the assocaited texture map into the specified
														// number of columns and randomly assign a column to each object
	UInt32					mRandomPosTextureRows;		// when > 1, divide the assocaited texture map into the specified
														// number of rows and randomly assign a row to each object
	
	// POSITION MOD OPTIONS
	Media3DPositionMod		mPositionModulation[MAX_POSITION_MODULATORS];
	
	// FLYIN MOD OPTIONS
	Media3DFlyinMod			mFlyinMod;
};

typedef struct Media3DModOptions Media3DModOptions;
typedef const Media3DModOptions* Media3DModOptionsPtr;

// ----------------------------------------------------------------------------------------
//	CGLContextFlags (class for C++, opaque struct otherwise)
//
#if __cplusplus
	class			CGLContextFlags;
#else
	typedef struct	CGLContextFlags	CGLContextFlags;
#endif

// -----------------------------------------------------------------`-----------------------
//	CGLContext (class for C++, opaque struct otherwise)
//
#if __cplusplus
	class			CGLContext;
#else
	typedef struct	CGLContext		CGLContext;
#endif

// ----------------------------------------------------------------------------------------
//	PluginContextInfo
//
// Plugins that work with OpenGL are can be passed this structure in their
// Render3D callback. This is the parameter that should be passed back to
// Isadora callbacks need a reference to the flags or the context.
//
// In addition, in v3, the mIgnoreFade member was added. If this value is
// true then you should not modulate the intensity of your 3D rendering
// with GetSceneIntensity_() call.

typedef struct {
	CGLContext*			mContext;
	CGLContextFlags*	mFlags;
	Boolean				mIgnoreFade;
} PluginContextInfo;


// ----------------------------------------------------------------------------------------
//	Render3DCallback Callback
//
// Passed as a parameter to inside of the Render3DInfo structure (see below),
// this callback allows plugins to render in 3D using OpenGL. Once attached to
// a specific stage using the kMedia3DAttachDrawCallback, this function will be
// called each time the stage needs to be rendered. At the time of the call, the
// OpenGL context is current, the camera has been positioned (as specified by the
// 3D Stage Orientation actor), the lighting has been positioned, and the scene
// is ready for rendering commands (i.e., a call to glBegin(...), glVertex, etc.)
//
// GL CONTEXT DEFAULT SETTINGS:
// this is the state of the OpenGL flags/settings used by Isadora at the time
// the 3D Render callback will be called - it is up to you to restore these
// before returning from the callbak
//
//  set with glEnable/Disable
//	GL_COLOR_MATERIAL = FALSE
//	GL_RESCALE_NORMAL = FALSE
//	GL_NORMALIZE = FALSE
//	GL_LIGHTING = FALSE
//	GL_LIGHT0 = FALSE
//	GL_LIGHT1 = FALSE
//	GL_LIGHT2 = FALSE
//	GL_LIGHT3 = FALSE
//	GL_BLEND = FALSE
//	GL_DEPTH_TEST = TRUE
//	GL_CULL_FACE = FALSE
//	GL_TEXTURE_2D = FALSE
//	GL_TEXTURE_RECTANGLE_ARB = FALSE
//
//  depth mask parameters: set with glDepthMask()
//	glDepthMask(GL_DEPTH_WRITE_MASK)
//
//  blend function parameters:
//	glBlendFunc(GL_SRC_ALPHA, GL_ZERO)
//
//  texture environment mode: set with glTexEnvi():
//	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE)
//
//  light model color control: set with glLightModeli():
//	glLightModeli(GL_LIGHT_MODEL_COLOR_CONTROL, GL_SINGLE_COLOR)
//

#if __cplusplus
extern "C" {
#endif

// 3D render callback function prototype: use the CPlugin3DRender object
// to manage 3D rendering inside your plugin. It allows you to control the
// target stage, layering, and visibility of your OpenGL rendering.
// 
// When calling the SetRenderInfo() method of CPlugin3DRender, you may
// pass two "reference constants" that will be passed back to you when
// your drawing function is called. These are the inRefCon1 and inRefCon2
// parameters seen below.

typedef void	(*Render3DCallback)(struct IsadoraParameters* ip, void* inRefCon1, void* inRefCon2);

// this version of the function provides a  pointer to a CGContextFlags
// C++ object. This object will manage the setting and restoring of the OpenGL
// parameters used by Isadora. Your plugin needs to be compiled with C++ and
// to make use of this functionality, and must include "CGLContextFlags.cp"
// and "CGLContextFlags.h" in your plugin project.
//
// IMPORTANT CHANGE FOR ISADORA 3:
//
// Note that he final parameter to this function has changed from a CGLContextFlags
// pointer to a PluginContextInfo pointer. This was necessary because CGLContextFlags
// was a superclass to CGLContext, allowing Isadora to derive the CGLContext from
// CGLContextFlags. These classes needed to be separated in Isadora 3, so there
// needed to be a way to pass both pointers to the callbacks. Calls that previously
// accepted the CGLContextFlags parameter will now need to accept a PluginContextInfo
// pointer, and then use the mFlags parameter with the flag related functions.

typedef void	(*Render3DCallbackWithContextFlags)(struct IsadoraParameters* ip, void* inParam1, void* inParam2, PluginContextInfo* inContextInfo);

#if __cplusplus
}
#endif

// ----------------------------------------------------------------------------------------
// Render3DInfo provides information necessary to call a 3D Rendering function
// that exists inside of an Isadora Plugin.
// 
// This info is passed to Isadora using the ExecuteMediaCommand_ function with a command
// of kMedia3DAttachDrawCallback. Once this is call is made, the callback will be called
// each time the stage needs to be rendered.
//
// When the plugin is done rendering, it should call ExecuteMediaCommand_ with a command of
// kMedia3DDetachDrawCallback
// 
// If possible, use the Render3DWithContextInfo structure defined below. This will allow
// to efficiently set/get the pertinent OpenGL flags
//
struct Render3DInfo
{
	Render3DCallback					mCallback;
	SInt32								mLayer;
	ThingPtr							mThing;
	void*								mRefCon1;
	void*								mRefCon2;
};
typedef struct Render3DInfo Render3DInfo;
typedef const Render3DInfo* ConstRender3DInfoPtr;

// ----------------------------------------------------------------------------------------
// Render3DWithContextInfo provides information necessary to call a 3D Rendering function
// that exists inside of an Isadora Plugin.
// 
// This info is passed to Isadora using the ExecuteMediaCommand_ function with a command
// of kMedia3DAttachDrawWithContextCallback. Once this is call is made, the callback will
// be called each time the stage needs to be rendered.
//
// When the plugin is done rendering, it should call ExecuteMediaCommand_ with a command of
// kMedia3DDetachDrawWithContextCallback
// 
struct Render3DWithContextInfo
{
	Render3DCallbackWithContextFlags	mCallback;
	SInt32								mLayer;
	ThingPtr							mThing;
	void*								mRefCon1;
	void*								mRefCon2;
};
typedef struct Render3DWithContextInfo Render3DWithContextInfo;
typedef const Render3DWithContextInfo* ConstRender3DWithContextInfo;

// ----------------------------------------------------------------------------------------
// StageScalingOptions provides information necessary to set the scaling of imagery
// when rendering to the stage

struct StageScalingOptions
{
	CFloatXY		mViewportOffset;		// viewport offset, 1.0 = one window width/height
	CFloatXY		mViewportSize;			// viewport scale, 1.0 = one window width/height
};
typedef struct StageScalingOptions StageScalingOptions;
typedef const StageScalingOptions* ConstStageScalingOptionsPtr;

// ----------------------------------------------------------------------------------------
// StageBackgroundOptions sets the background color of the stage and whether or not
// the stage will be erased before each rendering cycle
struct StageBackgroundOptions
{
	UInt32		mBkgColor;				// background color to use when rendering
	Boolean		mErase;					// if true, erase before each frame; if false, don't erase
	
	float		mIntensity;				// intensity level of these options, from 0.0 to 1.0; no effect when set to 0.0
										
	float		mFeedbackAmt;			// feedback intensity from 0.0 to 1.0; effectively "off" when 0.0
	float		mScaleH;				// 1.0 = no scaling; < 1.0 = smaller, > 1.0 = bigger
	float		mScaleV;				// ditto
	float		mOffsetH;				// < 0.0 = left, > 0.0 = right, 1.0 = unity
	float		mOffsetV;				// < 0.0 = down, > 0.0 = up, 1.0 = unity
	float		mRotation;				// rotation: -360 to +360
	float		mRotCenterH;			// rotation center point within the image
	float		mRotCenterV;
};
typedef struct StageBackgroundOptions StageBackgroundOptions;
typedef const StageBackgroundOptions* ConstStageBackgroundOptionsPtr;

// ----------------------------------------------------------------------------------------
// LStream - forward declaration for C++
#ifdef __cplusplus
class LStream;
#endif

// ----------------------------------------------------------------------------------------
// StageKeystoneOptions determines the keystoning of the image on the stage

#define MAX_STAGE_KEYSTONE_OFFSETS		4

CLASSHDR StageKeystoneOptions
{
PUBLIC
#ifdef __cplusplus
				StageKeystoneOptions()
				{
					mKeystoneFlipHorz = FALSE;
					mKeystoneFlipVert = FALSE;
					mKeystoneOffsets[0] = mKeystoneOffsets[1] = mKeystoneOffsets[2] = mKeystoneOffsets[3] = CFloatXY(0.0f, 0.0f);
				}
	
	UInt32		Read(LStream* inStream, UInt32 inFileVersion);
	UInt32		Write(LStream* inStream);

#endif

	Boolean		mKeystoneFlipHorz;
	Boolean		mKeystoneFlipVert;
	CFloatXY	mKeystoneOffsets[MAX_STAGE_KEYSTONE_OFFSETS];
};

// ----------------------------------------------------------------------------------------
// StageEdgeBlendOptions determines the masking of the image on the stage

#define MAX_EDGE_BLEND_EDGES	4

#include "CCurveSpec.h"

CLASSHDR EdgeSpec
{
PUBLIC
#ifdef __cplusplus
				EdgeSpec()
				{
					mPos = 0.0f;
					mRotation = 0.0f;
					mWidth = 0.0f;
				}
	
	UInt32		Read(LStream* inStream, UInt32 inFileVersion);
	UInt32		Write(LStream* inStream) const;
	
	bool		operator==(
					const EdgeSpec&		inRhs) const
				{
					return (mPos == inRhs.mPos && mRotation == inRhs.mRotation && mWidth == inRhs.mWidth && mCurve == inRhs.mCurve);
				}
	bool		operator!=(
					const EdgeSpec&		inRhs) const
				{
					return (mPos != inRhs.mPos || mRotation != inRhs.mRotation || mWidth != inRhs.mWidth || mCurve != inRhs.mCurve);
				}
#endif
	
	// edge position, rotation, width
	float		mPos;
	float		mRotation;
	float		mWidth;
	
	// edge curve specification
	CCurveSpec	mCurve;
};

typedef UInt32 StageEdgeBlendFlags;

enum {
	StageEdgeBlendOptions_UseExplicitCurveSettings = (1 << 0),
	StageEdgeBlendOptions_UseSharpnessGammaCurve = (1 << 1)
};

CLASSHDR StageEdgeBlendOptions
{
PUBLIC
#ifdef __cplusplus
	
				// IS-339 reversed kBottom and kTop; renamed with kEdge... for clarity
				enum Edge
				{
					kInvalid = -1,
					
					kEdgeLeft = 0,
					kEdgeRight,
					kEdgeBottom,
					kEdgeTop
				};
	
				StageEdgeBlendOptions()
				{
					mEBWidth = 1.0f;
					mEBHeight = 1.0f;
					mEBOffsetH = 0.0f;
					mEBOffsetV = 0.0f;
					
					mFlags = StageEdgeBlendOptions_UseSharpnessGammaCurve;
					
					for (UInt32 i=0; i<MAX_EDGE_BLEND_EDGES; i++) {
						mEdges[i].mPos = 0.0f;
						mEdges[i].mRotation = 0.0f;
						mEdges[i].mWidth = 0.0f;
					}
					
//					mEdges[kEdgeLeft].mCurve.Init(TRUE,  0.0f, 1.0f, 0.0f, 1.0f, 0.0f, 0.3f, 0.7f, 1.0f, DEFAULT_CURVE_SHARPNESS, DEFAULT_GAMMA, DEFAULT_KNEE);
//					mEdges[kEdgeRight].mCurve.Init(TRUE,  0.0f, 1.0f, 1.0f, 0.0f, 0.3f, 1.0f, 1.0f, 0.3f, DEFAULT_CURVE_SHARPNESS, DEFAULT_GAMMA, DEFAULT_KNEE);
//					mEdges[kEdgeBottom].mCurve.Init(FALSE, 0.0f, 1.0f, 1.0f, 0.0f, 0.3f, 1.0f, 1.0f, 0.3f, DEFAULT_CURVE_SHARPNESS, DEFAULT_GAMMA, DEFAULT_KNEE);
//					mEdges[kEdgeTop].mCurve.Init(FALSE, 1.0f, 0.0f, 1.0f, 0.0f, 1.0f, 0.7f, 0.3f, 0.0f, DEFAULT_CURVE_SHARPNESS, DEFAULT_GAMMA, DEFAULT_KNEE);

					mEdges[kEdgeLeft].mCurve.InitLeft();
					mEdges[kEdgeRight].mCurve.InitRight();
					mEdges[kEdgeTop].mCurve.InitTop();
					mEdges[kEdgeBottom].mCurve.InitBottom();
				}
	
	UInt32			Read(LStream* inStream, UInt32 inFileVersion);
	UInt32			Write(LStream* inStream) const;
	
	bool		operator==(
					const StageEdgeBlendOptions&		inRhs)
				{
					return (
						mFlags == inRhs.mFlags
						&& mEBWidth == inRhs.mEBWidth
						&& mEBHeight == inRhs.mEBHeight
						&& mEBOffsetH == inRhs.mEBOffsetH
						&& mEBOffsetV == inRhs.mEBOffsetV
						&& mEdges[0] == inRhs.mEdges[0]
						&& mEdges[1] == inRhs.mEdges[1]
						&& mEdges[2] == inRhs.mEdges[2]
						&& mEdges[3] == inRhs.mEdges[3]
					);
				}
	
	bool		operator!=(
					const StageEdgeBlendOptions&		inRhs)
				{
					return (
						mFlags != inRhs.mFlags
						|| mEBWidth != inRhs.mEBWidth
						|| mEBHeight != inRhs.mEBHeight
						|| mEBOffsetH != inRhs.mEBOffsetH
						|| mEBOffsetV != inRhs.mEBOffsetV
						|| mEdges[0] != inRhs.mEdges[0]
						|| mEdges[1] != inRhs.mEdges[1]
						|| mEdges[2] != inRhs.mEdges[2]
						|| mEdges[3] != inRhs.mEdges[3]
					);
				}
	
	const EdgeSpec&	GetEdgeSpec(
						Edge				inEdge) const
					{
						#ifdef DebugAssert_
						DebugAssert_((int) inEdge >= 0 && (int) inEdge <= MAX_EDGE_BLEND_EDGES);
						#endif
						return mEdges[inEdge];
					}
	
	void			SetEdgeSpec(
						Edge				inEdge,
						const EdgeSpec&		inEdgeSpec)
					{
						#ifdef DebugAssert_
						DebugAssert_((int) inEdge >= 0 && (int) inEdge <= MAX_EDGE_BLEND_EDGES);
						#endif
						mEdges[inEdge] = inEdgeSpec;
					}
	
	void			SetEdgeSpecCurve(
						Edge				inEdge,
						const CCurveSpec&	inCurveSpec)
					{
						#ifdef DebugAssert_
						DebugAssert_((int) inEdge >= 0 && (int) inEdge <= MAX_EDGE_BLEND_EDGES);
						#endif
						mEdges[inEdge].mCurve = inCurveSpec;
					}
	
	void			GetEdgeSpecCurve(
						Edge				inEdge,
						CCurveSpec&			outCurveSpec)
					{
						#ifdef DebugAssert_
						DebugAssert_((int) inEdge >= 0 && (int) inEdge <= MAX_EDGE_BLEND_EDGES);
						#endif
						outCurveSpec = mEdges[inEdge].mCurve;
					}
	
	Boolean			UseSharpnessGammaCurve() const;
	
	void			SetUseSharpnessGammaCurve(
						Boolean		inUseSharpnessGammaCurve);
#endif

	float				mEBWidth;			// width of mask, as percentage of stage (0.0 - 1.0)
	float				mEBHeight;			// height of mask, as percentage of stage (0.0 - 1.0)
	float				mEBOffsetH;			// horizontal center of mask, as percentage of stage (0.0 - 1.0)
	float				mEBOffsetV;			// vertical center of mask, as percentage of stage (0.0 - 1.0)
	
	StageEdgeBlendFlags	mFlags;				// usage flags - see above
	
	EdgeSpec			mEdges[MAX_EDGE_BLEND_EDGES];
};

// ----------------------------------------------------------------------------------------
// GLRenderOptions sets sets options for how 3D objects will be rendered

CLASSHDR GLRenderOptions
{
PUBLIC
#ifdef __cplusplus
	GLRenderOptions() :
		mZClipEnable(FALSE),
		mZClipNear(0.0f),
		mZClipFar(0.0f)
	{
	}
#endif

	Boolean	mZClipEnable;
	float	mZClipNear;
	float	mZClipFar;
};

// ----------------------------------------------------------------------------------------
// ModelInfo provides the information necessary to render a 3D Model. This struct is passed
// to as the first parameter to the kMedia3DAttachModel command.
//
struct ModelInfo
{
	RenderTarget				mRenderTarget;			// render target - real or virtual
	RenderChannel				mRenderChannelIndex1;	// one-based stage index or virtual renderer index
	SInt32						mRenderLayer;			// render layer controls order of model rendering
	ThingPtr					mOwningThing;			// the thing that owns the model; acquired from ActorInfo struct
};
typedef struct ModelInfo ModelInfo;
typedef const ModelInfo* ModelInfoPtr;

// ----------------------------------------------------------------------------------------
// TextureMapInfo provides the information necessary to create an OpenGL texture map from
// an ImageBuffer. Optionally it may create a framebuffer for offscreen rendering
//
// The caller creates the texture map by calling ExecuteMediaCommand_ with a command of
// kMedia3DCreateTextureMap. When calling this function, the user need only set
// mImageBuffer or mCoreImageBuffer (one or the other, but not both) and the mRepeat
// input -- Isadora will fill in the remaining fields upon successful
// complettion of the call.
// 
// From this point on, you can use the TextureMapInfo with the
// kMedia3DStoreTextureMapImage and/or kMedia3DPrepareTextureMapForRender commands
// to the ExecuteMediaCommand_. This allows you to bind the texture map to the object
// being rendered.
//
// Once the plugin is done with the texture map, it must destroy it by calling
// ExecuteMediaCommand_ with a command of kMedia3DDisposeTextureMap

struct TextureMapInfo
{
// INPUTS


	MultiImageBuffer			mMIB;						// INPUT: video buffer type and associated buffer pointer

	Boolean						mRepeat;					// INPUT: mRepeat affects GL_TEXTURE_WRAP_S/GL_TEXTURE_WRAP_T
															//		  if true, use GL_REPEAT and force the texture target
															//		  to GL_TEXTURE_2D; if false, use GL_CLAMP_TO_BORDER.
													
	Boolean						mClampToBorder;				// INPUT: set this flag to use GL_CLAMP_TO_BORDER instead of 
															//		  GL_CLAMP_TO_EDGE, which is the default.
															
	Boolean						mDisableAppleDMA;			// INPUT: on Mac OS X, prevents using Apple Client Storage optimization
															//		  when creating a texture. Normally this should be enabled as
															//		  transfers from the CPU to GPU are faster. However,
															//        when Apple Client Storage is enabled (i.e., this flag is
															//        false) the actual size of the texture memory must be a
															//        multiple of 32 bytes. The extra texels to the right will
															//        be filled with black. If your texture coordinates go beyond
															//        the right edge, and your texture is set to render with
															//		  GL_CLAMP_TO_EDGE, you will see black instead of the edge 
															//        pixels being extended.
// OUTPUTS: filled in by Isadora

	UInt32						mRenderChannelIndex0;		// OUTPUT: zero-based render channel or stage index; 0xFFFFFFFF = invalid
	Boolean						mGLTextureAttached;			// OUTPUT: when TRUE, a texture is attached to a rendering context
	IsadoraTexture*				mGLTexture;					// OUTPUT: pointer to IsadoraTexture associated with this map
	Boolean						mOwnsGLTexture;				// OUTPUT: if TRUE, mGLTexture is owned by this object
	Boolean						mRequiresHAPQShader;		// OUTPUT: will be set to true if texture requires HAP shader
};
typedef struct TextureMapInfo TextureMapInfo;
typedef const TextureMapInfo* TextureMapInfoPtr;

// ----------------------------------------------------------------------------------------
// LightingInfo - Used to specify the global lighting for a specific stage or render channel.
//
// The caller sets the lighting info by calling ExecuteMediaCommand_ with a command of
// kMedia3DLightPositionAndColors. 
//
struct LightingInfo
{
	UInt16			mLightIndex0;		// specified which light, from 0 to 3
	Boolean			mLightEnable;		// when true, enable this light. otherwise disable it.
	CFloatXYZ		mRotation;			// light rotation
	float			mDistance;			// distance component of the light
	UInt32			mAmbientColor;		// color ref: AARRGGBB
	UInt32			mDiffuseColor;		// color ref: AARRGGBB
	UInt32			mSpecularColor;		// color ref: AARRGGBB
};
typedef struct LightingInfo LightingInfo;
typedef const LightingInfo* LightingInfoPtr;

// ----------------------------------------------------------------------------------------
// BackgroundLightingInfo - Used to specify the background ambient light color for
// a specific stage or render channel.
//
// The caller sets the background ambient lighting color by calling ExecuteMediaCommand_
// with a command of kMedia3DBackgroundLightingInfoColor. 
//
struct BackgroundLightingInfo
{
	UInt32			mAmbientBackgroundColor;
};
typedef struct BackgroundLightingInfo BackgroundLightingInfo;
typedef const BackgroundLightingInfo* BackgroundLightingInfoPtr;

// ----------------------------------------------------------------------------------------
// Constant passed to an actor's DrawActorDefinedArea proc
// to indicate which part to draw.
enum
{
	kActorDefinedAreaTop		= 1,
	kActorDefinedAreaBottom		= 2
};
typedef	UInt16				ActorDefinedAreaPart;

// ----------------------------------------------------------------------------------------
// Constant passed to an actor's DrawActorDefinedAreaPropertyIndicator
// that determines the type of indicator to draw
enum
{
	kActorIndicatorType_Bullet = 1,	// either inInputPropertyIndex1 or inOutputPropertyIndex1 can be valid
	kActorIndicatorType_Link		// both inInputPropertyIndex1 and inOutputPropertyIndex1 must be valid
};
typedef	UInt16				ActorIndicatorType;

////////////////////////////////////////////////////////////////////////////
// CONTROL/ACTOR DEFINED AREA MOUSE MESSAGES DEFINES
////////////////////////////////////////////////////////////////////////////
// These messages are passed to the mouse message proc of either a control
// or of an actor with an interface element in the Actor Defined Area.
//
// To receive these messages in a Control, you must set the
// mControlMouseMessageProc member of the ControlInfo param when the
// Control's GetControlInfo function is called.
//
// To receive these messages in an actor, you must set the
// mMouseTrackInActorDefinedAreaProc member of the ActorInfo
// param when the Actor's GetActorInfo function is called.
//

enum {
	kIzzyMouseNone,		// inactive mouse state - not sent to the mouse message
						// proc but useful to indicate a non-tracking state
	
	kIzzyMouseEnter,	// mouse has entered the control/actor defined area
	kIzzyMouseMove,		// mouse is moving within the control/actor defined area
	kIzzyMouseLeave,	// mouse is leaving the control/actor defined area
	
	kIzzyMouseDown,		// when mouse is clicked in the control/actor defined area
	kIzzyMouseTrack,	// after click in control/actor defined area, reports movements of the mouse
	kIzzyMouseUp,		// after click in control/actor defined area, reports mouse up
	
	kIzzyDoubleClick,	// double click (sent upon mouse down after down/track/up)
	
	kIzzyMouseHit,		// mouse function should return true if mouse point is within user area
	
	// v2.6 ->
	kIzzyMouseWheelX,	// horizontal mouse wheel was moved while over a control/actor
	kIzzyMouseWheelY	// vertical mouse wheel was moved while over a control/actor
	// v2.6 <-
};
typedef	UInt8	IzzyMouseMessage;
	
////////////////////////////////////////////////////////////////////////////
// ACTOR MESSAGE DEFINES
////////////////////////////////////////////////////////////////////////////

enum {
	kIzzyActorHandleUndo,
	kIzzyActorHandleRedo,
	kIzzyActorDisposeUndoRedo
};
typedef	UInt32	IzzyActorMessage;
	
////////////////////////////////////////////////////////////////////////////
// CONTROL MESSAGE DEFINES
////////////////////////////////////////////////////////////////////////////

enum {
	kIzzyControlHandleUndo,
	kIzzyControlHandleRedo,
	kIzzyControlDisposeUndoRedo,
	kIzzyControlMediaListChanged
};
typedef	UInt32	IzzyControlMessage;

////////////////////////////////////////////////////////////////////////////
// PROPERTY FILE PATH MODE ENUMERATON
////////////////////////////////////////////////////////////////////////////
// To be used with the callback function SetPropertyFilePathModeProc_
// added in 3.0.8b39

typedef	UInt32	PropFilePathMode;

enum {
	kPropertyFilePath_None,
	
	kPropertyFilePath_FileRead,
	kPropertyFilePath_FileWrite,
	kPropertyFilePath_FileReadWrite,

	kPropertyFilePath_PathRead,
	kPropertyFilePath_PathWrite,
	kPropertyFilePath_PathReadWrite
};
	
#endif	// #ifndef _H_IsadoraTypes
