// ===========================================================================
//	MacNativeImageUtil			(c) 2015 Mark F. Coniglio. All rights reserved
// ===========================================================================
#ifndef _H_MacNativeImageUtil
#define _H_MacNativeImageUtil

#if TARGET_OS_MAC

#include <AGL/AGL.h>

#include "CoreImageCompatibility.h"
#include "MacColorSpaces.h"
#include "CPixelSize.h"

CGContextRef
CreateBitmapContextWithBitmap_RGB_NoAlpha(
	const PixelSize&	inSize,
	const UInt32		inRowBytes,
	void*				inImageData);
	
CGContextRef
CreateBitmapContext_Grey(
	const PixelSize&	inSize,
	const UInt32		inRowBytes);
	
CGContextRef
CreateBitmapContext_PremultipliedARGB(
	const PixelSize&	inSize,
	const UInt32		inRowBytes);
	
CGContextRef
CreateBitmapContext_RGB_NoAlpha(
	const PixelSize&	inSize,
	const UInt32		inRowBytes);

enum CGImageAlphaOption
{
	CGImageAlphaOption_NoAlpha,
	CGImageAlphaOption_StraightAlpha,
	CGImageAlphaOption_PremultipliedAlpha
};

CGImageRef
Create_CGImageRef_ARGB(
	const PixelSize&		inSize,
	const UInt32			inRowBytes,
	const void*				inImageData,
	CGImageAlphaOption		inAlphaOption,
	const IzRect*			inSrcRect);			// CAN BE NULL

CGImageRef
Create_CGImageRef_ARGB_WithCrop(
	const PixelSize&		inSize,
	const UInt32			inRowBytes,
	const void*				inImageData,
	CGImageAlphaOption		inAlphaOption,
	const PixelPos&			inCropPos,
	const PixelSize&		inCropSize);

#if COMPILE_CORE_VIDEO
enum CIImageFlipOption
{
	kCIImageFlipOption_NotFlipped,
	kCIImageFlipOption_Flipped
};

// NOTE: the returned CIImage has not been retained
CI_Image
CIImageWithGLTexture(
	GLint					inTextureName,
	float					inWidth,
	float					inHeight,
	CIImageFlipOption		inFlipped);

// NOTE: the returned CIImage has not been retained
CI_Image
CIImageWithImageARGBBitmap(
	const PixelSize&		inSize,
	const UInt32			inRowBytes,
	void*					inImageData);

#endif // #if COMPILE_CORE_VIDEO

#if APP_COMPILE_CORE_VIDEO

CI_Context
CreateCIContextWithCGLContext(
	CGLContextObj			inCGLContext,			// CGL Context 
	CGLPixelFormatObj		inCGLPixelFormat,		// CGL Pixel Format for specified context
	NativeWindow			inWindowRef);			// CAN BE NULL FOR GENERIC COLOR SPACE CI CONTEXT

void
DisposeCIContextForCGLContext(
	CI_Context				inCIContext,			// CI Context
	CGLContextObj			inCGLContext);			// CGL Context

#endif	// #if APP_COMPILE_CORE_VIDEO

#endif	// #if TARGET_OS_MAC

#endif // #ifndef _H_MacNativeImageUtil

