// ===========================================================================
//	NativeBitmap			 (C) 2016 Mark F. Coniglio. All rights reserved.
// ===========================================================================

#ifndef _H_NativeBitmap
#define _H_NativeBitmap

#include "IZDefinitions.h"

// ----------------------------------------------------------------------------------------
// NATIVE BITMAP TYPE
//
// Isadora v2.6 and After:	CGContextRef of Mac OS, HBITMAP on Windows
// Isadora v2.5 and Before:	GWorldPtr
// ----------------------------------------------------------------------------------------

#define NATIVE_BITMAP_OBJECTS		(TRUE || !IZ_ALLOW_QUICKTIME)

#if NATIVE_BITMAP_OBJECTS

	#define	NATIVE_BITMAP_IS_GWORLD		FALSE

	OSType		NativeBitmapGetFormat(
					NativeBitmap	inNativeBitmap);

	IzInt32		NativeBitmapGetWidth(
					NativeBitmap	inNativeBitmap);

	IzInt32		NativeBitmapGetHeight(
					NativeBitmap	inNativeBitmap);

	void		NativeBitmapGetBounds(
					NativeBitmap	inNativeBitmap,
					IzRect*			outRect);

	IzInt32		NativeBitmapGetRowBytes(
					NativeBitmap	inNativeBitmap);

	void*		NativeBitmapGetBaseAddress(
					NativeBitmap	inNativeBitmap);

	IzUInt32	NativeBitmapGetBitsPerPixel(
					NativeBitmap	inNativeBitmap);

	void		NativeBitmapGetInfo(
					NativeBitmap	inNativeBitmap,
					OSType*			outFormat,
					IzInt32*		outWidth,
					IzInt32*		outHeight,
					IzInt32*		outRowBytes,
					IzInt32*		outBitDepth,
					void**			outBaseAddress,
					Boolean*		outHasAlpha,
					Boolean*		outIsPremultiplied);
				
#else

	#define	NATIVE_BITMAP_IS_GWORLD		TRUE

	// pre Isadora 2.5
	typedef CGrafPtr		NativeBitmap;
	typedef PixMap			NativePixMap;
	typedef PixMapPtr		NativePixMapPtr;
	typedef PixMapHandle	NativePixMapHandle;

	#if TARGET_OS_WIN

		#define NativeBitmapGetHBITMAP(nb)	static_cast<HBITMAP>(GetPortHBITMAP((GrafPtr) (nb)))
		#define NativeBitmapGetHDC(nb)		static_cast<HDC>(GetPortHDC((GrafPtr) (nb)))

	#endif
	
	#if TARGET_OS_MAC

		#define	NativeBitmapGetPixMap(nb)			(::GetPortPixMap(nb))
		#define NativeBitmapGetBounds(nb, r)		::GetPortBounds(nb, r)

		#define NativeBitmapGetPixBaseAddr(pm)		(::GetPixBaseAddr(pm))
		#define NativeBitmapGetPixRowBytes(pm)		(::GetPixRowBytes(pm))
		#define NativeBitmapGetPixBounds(pm, r)		::GetPixBounds((pm), (r))
		#define NativeBitmapGetPixDepth(pm)			(::GetPixDepth(pm))

	#else

		#define	NativeBitmapGetPixMap(nb)			((nb)->portPixMap)
		#define NativeBitmapGetBounds(nb, r)		{ *(r) = (nb)->portRect; }

		#define NativeBitmapGetPixBaseAddr(pm)		((**(pm)).baseAddr)
		#define NativeBitmapGetPixRowBytes(pm)		(::GetPixRowBytes(pm))
		#define NativeBitmapGetPixBounds(pm, r)		(*(r)) = (**(pm)).bounds
		#define NativeBitmapGetPixDepth(pm)			((**(pm)).pixelSize)

	#endif

	#define NativeBitmapGetPixFormat(pm)			((**pm).pixelFormat)

	#define NativeBitmapLockPixels(pm)				::LockPixels(pm)
	#define NativeBitmapUnlockPixels(pm)			::UnlockPixels(pm)
	#define NativeBitmapGetPixelsState(pm)			::GetPixelsState(pm)

	#define NativeBitmapGetBitsPerPixel(nb)			NativeBitmapGetPixDepth(NativeBitmapGetPixMap(nb))
	#define NativeBitmapGetFormat(nb)				NativeBitmapGetPixFormat(NativeBitmapGetPixMap(nb))
	#define NativeBitmapGetBaseAddress(nb)			NativeBitmapGetPixBaseAddr(NativeBitmapGetPixMap(nb))
	#define NativeBitmapGetRowBytes(nb)				NativeBitmapGetPixRowBytes(NativeBitmapGetPixMap(nb))

#endif

#if IZ_ALLOW_QUICKTIME

	#if TARGET_OS_MAC

		#define	GWorldGetPixMap(nb)				(::GetPortPixMap(nb))
		#define GWorldGetBounds(nb, r)			::GetPortBounds(nb, r)

		#define GWorldGetPixBaseAddr(pm)		(::GetPixBaseAddr(pm))
		#define GWorldGetPixRowBytes(pm)		(::GetPixRowBytes(pm))
		#define GWorldGetPixBounds(pm, r)		::GetPixBounds((pm), (r))
		#define GWorldGetPixDepth(pm)			(::GetPixDepth(pm))

	#else

		#define	GWorldGetPixMap(nb)				((nb)->portPixMap)
		#define GWorldGetBounds(nb, r)			{ *(r) = (nb)->portRect; }

		#define GWorldGetPixBaseAddr(pm)		((**(pm)).baseAddr)
		#define GWorldGetPixRowBytes(pm)		(::GetPixRowBytes(pm))
		#define GWorldGetPixBounds(pm, r)		(*(r)) = (**(pm)).bounds
		#define GWorldGetPixDepth(pm)			((**(pm)).pixelSize)

	#endif

	#define GWorldGetPixFormat(pm)				((**pm).pixelFormat)

	#define GWorldLockPixels(pm)				::LockPixels(pm)
	#define GWorldUnlockPixels(pm)				::UnlockPixels(pm)
	#define GWorldGetPixelsState(pm)			::GetPixelsState(pm)

	#define GWorldGetBitsPerPixel(nb)			GWorldGetPixDepth(GWorldGetPixMap(nb))
	#define GWorldGetFormat(nb)					GWorldGetPixFormat(GWorldGetPixMap(nb))
	#define GWorldGetBaseAddress(nb)			GWorldGetPixBaseAddr(GWorldGetPixMap(nb))
	#define GWorldGetRowBytes(nb)				GWorldGetPixRowBytes(GWorldGetPixMap(nb))

#endif

#define ALLOW_QT_EXTERNAL_OUTPUT		(IZ_ALLOW_QUICKTIME && NATIVE_BITMAP_IS_GWORLD)
#define ALLOW_QT_SOUND_SETUP			(IZ_ALLOW_QUICKTIME)

#endif	// #ifndef _H_NativeBitmap
