// ===========================================================================
//	StGLContextFlags		 (c) 2013 Mark F. Coniglio	  All rights reserved.
// ===========================================================================

#ifndef _H_StGLContextFlags
#define _H_StGLContextFlags

#include "CGLContextFlags.h"

#if defined(__ISADORA_APP__) && DEBUG
	#include "COpenGLUtil.h"
	#define	StGLContextReportErr()		CheckOpenGLError()
#else
	#define	StGLContextReportErr()		{ }
#endif

// --------------------------------------------------------------------------------

#define	StGLContextFlag(name, ctx, glFlag, flagValue)		__StGLContextFlag name(ctx, glFlag, kGLFlag_##glFlag, flagValue)

class __StGLContextFlag
{
public:

	__StGLContextFlag(
		CGLContextFlags*	inContextInfo,
		GLenum				inGLFlag,
		GLenum				inGLFlagIndex0,
		GLboolean			inGLFlagValue) :
			mGLContextFlags(inContextInfo),
			mGLFlag(inGLFlag),
			mGLFlagIndex0(inGLFlagIndex0),
			mSavedEnableFlag(inContextInfo->__GLIsEnabled(inGLFlag, inGLFlagIndex0))
	{
		mGLContextFlags->__GLSetEnableFlag(mGLFlag, mGLFlagIndex0, inGLFlagValue);
		StGLContextReportErr();
	}
	
	~__StGLContextFlag()
	{
		mGLContextFlags->__GLSetEnableFlag(mGLFlag, mGLFlagIndex0, mSavedEnableFlag);
		StGLContextReportErr();
	}

private:
	CGLContextFlags*	mGLContextFlags;
	GLenum				mGLFlag;
	GLenum				mGLFlagIndex0;
	GLboolean			mSavedEnableFlag;
};

// --------------------------------------------------------------------------------

class StGLContextDepthWriteMask
{
public:

	StGLContextDepthWriteMask(
		CGLContextFlags*	inContextInfo,
		GLboolean			inEnable) :
			mGLContextFlags(inContextInfo),
			mSavedDepthWriteMask(inContextInfo->GLGetDepthWriteMask())
	{
		// change the depth mask setting
		mGLContextFlags->GLSetDepthWriteMask(inEnable);
		StGLContextReportErr();
	}
	
	~StGLContextDepthWriteMask()
	{
		// restore
		mGLContextFlags->GLSetDepthWriteMask(mSavedDepthWriteMask);
		StGLContextReportErr();
	}

private:
	CGLContextFlags*	mGLContextFlags;
	GLboolean			mSavedDepthWriteMask;
};

// --------------------------------------------------------------------------------

class StGLContextTexEnvMode
{
public:
	StGLContextTexEnvMode(
		CGLContextFlags*	inContextInfo) :
			mGLContextFlags(inContextInfo),
			mSavedTexEnvMode(inContextInfo->GLGetTexEnvMode())
	{
		StGLContextReportErr();
	}

	StGLContextTexEnvMode(
		CGLContextFlags*	inContextInfo,
		GLint				inTexEnvMode) :
			mGLContextFlags(inContextInfo),
			mSavedTexEnvMode(inContextInfo->GLGetTexEnvMode())
	{
		mGLContextFlags->GLSetTexEnvMode(inTexEnvMode);
		StGLContextReportErr();
	}
	
	~StGLContextTexEnvMode()
	{
		// restore old value
		mGLContextFlags->GLSetTexEnvMode(mSavedTexEnvMode);
		StGLContextReportErr();
	}

private:
	CGLContextFlags*	mGLContextFlags;
	GLint				mSavedTexEnvMode;
};

class StGLContextTextureTarget
{
public:

	StGLContextTextureTarget(
		CGLContextFlags*	inContextInfo,
		GLenum				inTextureTarget) :
			mGLContextFlags(inContextInfo),
			mSavedTextureTarget(inContextInfo->GetActiveTextureTarget())
	{
		mGLContextFlags->EnableTextureMapping(inTextureTarget);
		StGLContextReportErr();
	}
			
	~StGLContextTextureTarget()
	{
		mGLContextFlags->EnableTextureMapping(mSavedTextureTarget);
		StGLContextReportErr();
	}
	
private:
	CGLContextFlags*	mGLContextFlags;
	GLenum				mSavedTextureTarget;
};



#endif
