//============================================================================
//	TextureBufferBase			(c) 2015 Mark F. Coniglio. All rights reserved
// ===========================================================================

#ifndef _H_TextureBufferBase
#define	_H_TextureBufferBase
#pragma once

// system includes
#include "OpenGLIncludes.h"

// isadora includes
#include "IsadoraTypes.h"
#include "IsadoraDebugging.h"

// other includes
#include "IsadoraTypes.h"
#include "ValueCommon.h"
#include "IzzyTextureBase.h"

// ===========================================================================
// TextureBuffer struct

struct TextureBuffer
{
	DataValueInfo	mInfo;					// our data value info struct - required
	UInt32			mMagic;					// magic value
	IzzyTexture		mTexture;				// pointer to an IsadoraTexture struct -- see IzzyTextureBase.h
	StageIndexT		mTargetStageIndex0;		// destination stage for the buffer
};

typedef struct TextureBuffer TextureBuffer, *TextureBufferPtr;

// ===========================================================================
// Texture Buffer Magic Value

// used to mark a Texture Buffer as valid

#define kTextureBufferMagic					0x12345678

// ===========================================================================
// Handy function for safely getting the CGLTexture from a TextureBuffer

// if the Texture Buffer is not NULL
// and if the magic value is valid
// and if the mTexture member is not NULL
// and the mTexture's GetCGLTexture() does not return NULL
// then this macro will return a valid CGLTexture*

#define GetCGLTextureFromTextureBuffer(a) \
	((a) != NULL && (a)->mMagic == kTextureBufferMagic && (a)->mTexture != NULL && (a)->mTexture->IsCGLTextureValid()) \
	? ((a)->mTexture->GetCGLTexture()) : NULL

// same as above, but return's a pointer
// to the CGLTexture's CGLContext owner

#define GetCGLContextFromTextureBuffer(a) \
	((a) != NULL && (a)->mMagic == kTextureBufferMagic && (a)->mTexture != NULL && (a)->mTexture->IsCGLTextureValid()) \
	? ((a)->mTexture->GetCGLTexture()->GetOwningGLContext()) : NULL


#endif	// #ifndef _H_TextureBufferBase
