// ===========================================================================
//	TextureBufferCommon			 (C)2010 Mark F. Coniglio. All rights reserved.
// ===========================================================================
//
// TextureBuffers define an OpenGL texture map buffer

#ifndef _H_TextureBufferCommon
#define _H_TextureBufferCommon

// isadora includes
#include "IsadoraTypes.h"
#include "IsadoraDebugging.h"

// texture buffer includes
#include "IzzyTextureBase.h"
#include "TextureBufferBase.h"

// ===========================================================================
// DEBUGGING

#define DEBUG_TEXTURE_BUFFER_COMMON			(FALSE && DEBUG)

#if DEBUG_TEXTURE_BUFFER_COMMON
	#define	tbc_log_info					IzzyLogInfo
#else
	#define	tbc_log_info(...)				{ }
#endif

// ===========================================================================
// GLOBALS

extern TextureBuffer		sTextureBuffer_CLEAR;

// ===========================================================================
// DEFINES

#define	DebugCheckTextureBufferClear(a)	{ IzzyDebugAssert_((a)->mMagic == 0 && (a)->mTexture == NULL); }

#ifndef CHECK_TEXTURE_BUFFER_MAGIC
#define CHECK_TEXTURE_BUFFER_MAGIC		TRUE
#endif

#if (DEBUG || CHECK_TEXTURE_BUFFER_MAGIC)
	#define TextureBuffer_CheckMagic(tb)	IzzyAssert_(tb == NULL || (tb)->mMagic == kTextureBufferMagic)
#else
	#define TextureBuffer_CheckMagic(tb)	{ }
#endif

// ===========================================================================
// FUNCTIONS

Boolean
TextureBuffer_IsValid(
	const TextureBufferPtr	inTextureBuffer);

Boolean
TextureBuffer_IsInitialized(
	const TextureBufferPtr	inTextureBuffer);

void
TextureBuffer_Init(
	TextureBufferPtr		inTextureBuffer);

void
TextureBuffer_Clear(
	TextureBufferPtr		inTextureBuffer);

void
TextureBuffer_SetTextureBufferInfo(
	TextureBufferPtr		inTextureBuffer,
	IzzyTexture				inTexture,
	StageIndexT				inTargetStageIndex0);
	
#ifdef __ISADORA_APP__

void
TextureBuffer_Uninit(
	TextureBufferPtr		inTextureBuffer);

void
TextureBuffer_Dispose(
	TextureBufferPtr		inTextureBuffer);

void
TextureBuffer_GetFullDimensions(
	TextureBufferPtr		inTextureBuffer,
	UInt32*					outWidth,
	UInt32*					outHeight);

void
TextureBuffer_GetFullDimensions(
	TextureBufferPtr		inTextureBuffer,
	float*					outWidth,
	float*					outHeight);

void
TextureBuffer_GetUsedDimensions(
	TextureBufferPtr		inTextureBuffer,
	UInt32*					outWidth,
	UInt32*					outHeight);

void
TextureBuffer_GetUsedDimensions(
	TextureBufferPtr		inTextureBuffer,
	float*					outWidth,
	float*					outHeight);
#endif

#if DEBUG_TEXTURE_BUFFER_COMMON
	void
	TextureBuffer_IncrementFrameCount(
		TextureBufferPtr	inTextureBuffer);
#else

	#define TextureBuffer_IncrementFrameCount(inTextureBuffer)	{ if (inTextureBuffer != NULL) ((inTextureBuffer)->mInfo.mDataChangeCount) +=1; }

#endif
	
#endif	// #if _H_TextureBufferCommon
