// ===========================================================================
//	VidColorMacros				  (C)2007 Mark F. Coniglio. All rights reserved.
// ===========================================================================
// These macros allow you to pull the individual components (A/R/G/B or
// Cb/Y0/Cr/Y1) from a color. They are conditionally compiled depending
// on the platform endian settings in VidFormats.h

#ifndef _H_VIDCOLORMACROS
#define _H_VIDCOLORMACROS

#include "VidFormats.h"

// PIXEL ORDER MSB -> LSB = ARGB
#if kGWorldPixelComponentOrder_ARGB == 1
	#define	RGB_(r, g, b)				( (((UInt32)((r) & 0xFF) << 16)) | (((UInt32)((g) & 0xFF) << 8)) | ((UInt32)((b) & 0xFF)) )
	#define	ARGB_(a, r, g, b)			( (((UInt32)((a) & 0xFF) << 24)) | (((UInt32)((r) & 0xFF) << 16)) | (((UInt32)((g) & 0xFF) << 8)) | ((UInt32)((b) & 0xFF)) )

	#define MAX_ALPHA					0xFF000000
	#define MASK_ONLY_ALPHA				0xFF000000
	#define MASK_NO_ALPHA				0x00FFFFFF
	
	#define REMOVE_ALPHA(rgb)			( MASK_NO_ALPHA & (rgb) )
	#define	SET_ALPHA(rgb)				( MAX_ALPHA | (rgb) )
	#define	SET_ALPHA_VALUE(a, rgb)		( (((UInt32)((a) & 0xFF) << 24)) | (rgb) )
	#define	ALPHA_MASK_(argb)			( ((UInt32) argb) & MASK_ONLY_ALPHA)
	#define HAS_MAX_ALPHA(rgb)			( ((rgb) & MASK_ONLY_ALPHA) == MAX_ALPHA )

	#define	ALPHA_(argb)				((UInt8)(((argb) >> 24) & 0xFF))
	#define	RED_(argb)					((UInt8)(((argb) >> 16) & 0xFF))
	#define	GREEN_(argb)				((UInt8)(((argb) >> 8) & 0xFF))
	#define	BLUE_(argb)					((UInt8)(((argb) >> 0) & 0xFF))

// PIXEL ORDER MSB -> LSB = BGRA
#else

	#define	RGB_(r, g, b)				((((UInt32)((b) & 0xFF) << 24)) | (((UInt32)((g) & 0xFF) << 16)) | (((UInt32)((r) & 0xFF) << 8)))
	#define	ARGB_(a, r, g, b)			((((UInt32)((b) & 0xFF) << 24)) | (((UInt32)((g) & 0xFF) << 16)) | (((UInt32)((r) & 0xFF) << 8)) | (((UInt32)((a) & 0xFF) << 0)))

	#define MAX_ALPHA					0x000000FF
	#define MASK_ONLY_ALPHA				0x000000FF
	#define MASK_NO_ALPHA				0xFFFFFF00

	#define REMOVE_ALPHA(rgb)			( MASK_NO_ALPHA & (rgb) )
	#define	SET_ALPHA(rgb)				( MAX_ALPHA | (rgb) )
	#define	SET_ALPHA_VALUE(a, rgb)		( (((UInt32)((a) & 0xFF) << 0)) | (rgb) )
	#define	ALPHA_MASK_(bgra)			( ((UInt32) bgra) & MASK_ONLY_ALPHA )
	#define HAS_MAX_ALPHA(rgb)			( ((rgb) & MASK_ONLY_ALPHA) == MAX_ALPHA )
	
	#define	ALPHA_(bgra)				((UInt8)((bgra) & 0xFF))
	#define	RED_(bgra)					((UInt8)(((bgra) >> 8) & 0xFF))
	#define	GREEN_(bgra)				((UInt8)(((bgra) >> 16) & 0xFF))
	#define	BLUE_(bgra)					((UInt8)(((bgra) >> 24) & 0xFF))

#endif

// useful notes about YUV and YCbCr
// U = Cb
// V = Cr

// --------------------------------------
// 2vuy / UYVY
// --------------------------------------
// -- Apple k2vuyPixelFormat
// -- Windows FourCC UYVY
// -- Memory Layout
// -- Big:    CbY0CrY1
// -- Little: Y1CrY0Cb

// --------------------------------------
// yuvs / YUY2
// --------------------------------------
// -- Apple kYUVSPixelFormat
// -- Windows FourCC YUY2
// -- Memory Layout = CbY0CrY1
// -- Big:    CbY0CrY1
// -- Little: Y1CrY0Cb

#if TARGET_RT_BIG_ENDIAN

	// CbY0CrY1
	
	#define kCbShift					24
	#define kY0Shift					16
	#define kCrShift					8
	#define kY1Shift					0

	#define	kCbMask						0xFF000000
	#define	kY0Mask						0x00FF0000
	#define	kCrMask						0x0000FF00
	#define	kY1Mask						0x000000FF
	
	#define kYUVColorMask				0xFF00FF00
	#define kYUVLumaMask				0x00FF00FF
	
	#define kYMin						16
	#define kYMax						235
	#define kYOffset					16
	#define kYDelta						219
	
	#define kRawCbCrMin					16
	#define kRawCbCrMax					240
	#define kCbCrOffset					16
	#define kSignedCbCrMin				-112
	#define kSignedCbCrMax				112
	
#elif TARGET_RT_LITTLE_ENDIAN

	// Y1CrY0Cb
	
	#define kY1Shift					24
	#define kCrShift					16
	#define kY0Shift					8
	#define kCbShift					0

	#define	kY1Mask						0xFF000000
	#define	kCrMask						0x00FF0000
	#define	kY0Mask						0x0000FF00
	#define	kCbMask						0x000000FF
	
	#define kYUVColorMask				0x00FF00FF
	#define kYUVLumaMask				0xFF00FF00
	
	#define kYMin						16
	#define kYMax						235
	#define kYOffset					16
	#define kYDelta						219
	
	#define kRawCbCrMin					16
	#define kRawCbCrMax					240
	#define kCbCrOffset					16
	#define kSignedCbCrMin				-112
	#define kSignedCbCrMax				112
	
#endif

#define	Y0_(c)						((UInt32) (((c) >> kY0Shift) & 0xFF))
#define	Cb_(c)						((UInt32) (((c) >> kCbShift) & 0xFF))
#define	Y1_(c)						((UInt32) (((c) >> kY1Shift) & 0xFF))
#define	Cr_(c)						((UInt32) (((c) >> kCrShift) & 0xFF))

#define	YUV_(y0, Cb, y1, Cr)		(((UInt32)((y0) & 0xFF)) << kY0Shift) | ( (((UInt32)((Cb) & 0xFF)) >> kCbShift) | (((UInt32)((y1) & 0xFF)) << kY1Shift) ) | (((UInt32)((Cr) & 0xFF)) << kCrShift)

#endif
