// ===========================================================================
//	izzy_math				 (C)2012 Mark F. Coniglio. All rights reserved.
// ===========================================================================

#ifndef _H_izzy_math
#define _H_izzy_math

// ensure that M_PI is defined... windows does not define this
// unless you set a special switch
#ifndef M_PI
#define M_PI        3.14159265358979323846264338327950288   /* pi */
#endif

// convenience function to convert degrees to radians
#ifndef DEG2RAD
#define	DEG2RAD(a)	( (a) * (M_PI  / 180.0) )
#endif

// convenience function to convert radians to degrees
#ifndef RAD2DEG
#define	RAD2DEG(a)	( (a) * (180.0  / M_PI) )
#endif

// special definitions for cross platform compatibility
#ifdef __cplusplus

#include <cmath>

#if defined(_MSC_VER)
	#include <float.h>
	#include <limits>
//#elif defined(__MWERKS__)
//	#include <cmath>
//#else
//	#include <cmath>
#endif

	// --------------------------------------------------
	// endian_swap_float - "flips" a float from big to
	// little, or from little to big endian
	
	inline float	endian_swap_float(
						float	value)
					{
						unsigned long ivalue = *((unsigned long*) &value);
							
						ivalue
							= ((ivalue & 0xff000000) >> 24)
							| ((ivalue & 0x00ff0000) >>  8)
							| ((ivalue & 0x0000ff00) <<  8)
							| ((ivalue & 0x000000ff) << 24);
		
						return *((float*) &ivalue);
					}


	// --------------------------------------------------
	// NAN - "not a number" testing
	
	#ifdef is_nan
	#undef is_nan
	#endif

	// WIN32 - VC++
	#if defined(_MSC_VER)

		#define	is_nan(a)		_isnan(a)
	
		#ifdef INFINITY
		#undef INFINITY
		#endif
		
		#define	INFINITY		std::numeric_limits<float>::infinity()

	// WIN32 - METROWERKS
	#elif defined(__MWERKS__)

		#define	is_nan(a)		isnan(a)
	
	// MAC OS
	#else

		#define	is_nan(a)		std::isnan(a)
	
	#endif

#endif	// #ifdef __cplusplus

#endif	// #ifndef _H_win32_math
