# ANALYZE IMAGE DEMO PLUGIN - READ ME

## DESCRIPTION

(Please don't skip the section below "ABOUT ANALYZING OPENGL TEXTURE DATA"
as this information is very important!)

This plugin does a simple analysis of the color of the incoming video stream
to illustrate an important general technique: how to use C++ code to analyze
a video frame that typically resides on the graphics card (GPU) of your
computer. This technique is required if you wish to analyze a video stream
with a CPU based analysis library such as OpenCV.

## CUSTOM USER INTERFACE DEMONSTRATION

Instead of an icon, some actors offer a custom, dynamic display that gives
the user an indication of their function. This plugin illustrates one such
interface element by displaying an indicator that shows the portion of the
image that is being analyzed. See the GetActorDefinedArea and
DrawActorDefinedArea functions for more information.

## ABOUT VIDEO TYPES

In Isadora, video data comes in three "flavors":

- **vid-gpu**: Image is stored as an OpenGL texture on your graphics card.
Image data is processed using OpenGL shaders

- **vid-ci**: Image is stored as
a CIImage on your graphics card (macOS only). Image data is processed using
OpenGL shaders

- **vid-cpu**: Image is stored in the main memory of your
computer. image data is processed using standard C++.  format is only used by
legacy Isadora plugins, usually marked by the word classic" in their name.

This plugin offers a mutable input that will "mutate" to whatever video type
is received at the input. Then a conversion function supplied by the SDK
ensures that, regardless of the video type you receive, the image ends
up as a bitmap in the main memory of the computer that can be analyzed using
C++.

## **VERY IMPORTANT**: ABOUT ANALYZING OPENGL TEXTURE DATA

The main processor (CPU) cannot directly read the individual pixels of a
texture that resides within the video memory of the GPU. To gain access to
those pixels, one must transfer the image from the GPU's video memory to the
computer's main memory (RAM) so that the computer's processor can access it.

This plugin illustrates how to use a function offered by the SDK called
`ConvertVideoStreamToBitmap_()` to pull texture data back into the computer's
main memory. This function is designed to be as efficient as possible, but
pulling texture data back to the the main processor's memory is,
unfortunately, inherently inefficient.

Why? Because doing so causes a "stall", where all pending parallel operations
pending on the GPU must be completed before the pixels can be read back to
the CPU. This can significantly reduce the throughput of the GPU, which will
in turn slow down all video processing in the currently executing Isadora
Scene.

If this is done by many actors in the same processing cycle, Isadora's
LOAD indicator may go up significantly as all video processing is slowed
down because of multiple stalls. This may be especially noticeable this
is done with higher resolution images such as 2K or (worse yet) 4K video.
That is why this plugin offers the opportunity to sample only a portion of
the image, because pulling large images can introduce even more significant
delays.

To always analyze the entire image, set the `ANALAYZE_PARTIAL_IMAGE` switch
to 0. This will remove all code that has to do with selecting a portion of
the image for analysis.

## CUSTOMIZING THIS SOURCE CODE

To customize this file, search for the text ###. All of the places where you
will need to customize the file are marked with this pattern of characters.

## LICENSE

IMPORTANT: This source code ("the software") is supplied to you in
consideration of your agreement to the following terms. If you do not agree
to the terms, do not install, use, modify or redistribute the software.

Mark Coniglio (aka TroikaTronix) grants you a personal, non exclusive license
to use, reproduce, modify this software with and to redistribute it, with or
without modifications, in source and/or binary form. Except as expressly
stated in this license, no other rights are granted, express or implied, to
you by TroikaTronix.

This software is provided on an "AS IS" basis. TroikaTronix makes no
warranties, express or implied, including without limitation the implied
warranties of non-infringement, merchantability, and fitness for a
particular purpurse, regarding this software or its use and operation alone
or in combination with your products.

In no event shall TroikaTronix be liable for any special, indirect,
incidental, or consequential damages arising in any way out of the use,
reproduction, modification and/or distribution of this software.